@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: axis.c,v 1.60.2.1 2006/10/22 12:09:43 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: axis.c,v 1.60.2.4 2008/01/14 07:27:57 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - axis.c */
@@ -871,7 +871,7 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 	    else
 		gprintf(label, sizeof(label), mark->label ? mark->label : ticfmt[axis], log10_base, mark->position);
 	    /* use NULL instead of label for minitic */
-	    (*callback) (axis, internal, mark->level?NULL:label, mark->level?mgrd:lgrd);
+	    (*callback) (axis, internal, (mark->level>0)?NULL:label, (mark->level>0)?mgrd:lgrd);
 	}
 	if (def->type == TIC_USER)
 	    return;
@@ -989,7 +989,10 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 		if (step > 1.5) {	/* beware rounding errors */
 		    /* {{{  10,100,1000 case */
 		    /* no more than five minitics */
-		    ministart = ministep = (int) (0.2 * step);
+		    if (step < 65535) /* should be MAXINT */
+			ministart = ministep = (int)(0.2 * step);
+		    else
+			ministart = ministep = 0.2 * step;
 		    if (ministep < 1)
 			ministart = ministep = 1;
 		    miniend = step;
@@ -1609,7 +1612,10 @@ get_position_default(struct position *pos, enum position_type default_type)
     }
 
     /* z is not really allowed for a screen co-ordinate, but keep it simple ! */
-    if (equals(c_token, ",")) {
+    if (equals(c_token, ",")
+       /* Partial fix for ambiguous syntax when trailing comma ends a plot command */
+	&& !(isstringvalue(c_token+1))
+       ) {
 	++c_token;
 	get_position_type(&type, &axes);
 	pos->scalez = type;
@@ -1624,8 +1630,8 @@ get_position_default(struct position *pos, enum position_type default_type)
  * Add a single tic mark, with label, to the list for this axis.
  * To avoid duplications and overprints, sort the list and allow
  * only one label per position.
- * EAM - called from set.c during `set xtics`
- *       called from datafile.c during `plot using ::xtic()`
+ * EAM - called from set.c during `set xtics` (level = 0 or 1)
+ *       called from datafile.c during `plot using ::xtic()` (level = -1)
  */
 void
 add_tic_user(AXIS_INDEX axis, char *label, double position, int level)
