@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: color.c,v 1.70.2.2 2007/02/25 13:04:50 mikulik Exp $"); }
+static char *RCSid() { return RCSid("$Id: color.c,v 1.70.2.8 2008/03/08 17:51:42 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - color.c */
@@ -54,7 +54,7 @@ int supply_extended_color_specs = 0;
 #endif
 
 /* Corners of the colour box. */
-static unsigned int cb_x_from, cb_x_to, cb_y_from, cb_y_to;
+static int cb_x_from, cb_x_to, cb_y_from, cb_y_to;
 
 /* Internal prototype declarations: */
 
@@ -209,11 +209,12 @@ void ifilled_quadrangle(gpiPoint* icorners)
 
 	int i;
 
-	/* Colour has changed, thus must apply properties again. That's because
-	   gnuplot has no inner notion of color.
+	/* Colour has changed, so we must apply line properties again.
+	 * FIXME: It would be cleaner to apply the general line properties
+	 * outside this loop, and limit ourselves to apply_pm3dcolor().
 	 */
-	struct lp_style_type lp;
-	lp_use_properties(&lp, pm3d.hidden3d_tag, 1);
+	static struct lp_style_type lp = DEFAULT_LP_STYLE_TYPE;
+	lp_use_properties(&lp, pm3d.hidden3d_tag, 0);
 	term_apply_lp_properties(&lp);
 
 	term->move(icorners[0].x, icorners[0].y);
@@ -237,11 +238,14 @@ filled_quadrangle(gpdPoint * corners)
 #endif
 {
     int i;
+    double x, y;
 #ifndef EXTENDED_COLOR_SPECS
     gpiPoint icorners[4];
 #endif
     for (i = 0; i < 4; i++) {
-	map3d_xy(corners[i].x, corners[i].y, corners[i].z, &icorners[i].x, &icorners[i].y);
+	map3d_xy_double(corners[i].x, corners[i].y, corners[i].z, &x, &y);
+	icorners[i].x = x;
+	icorners[i].y = y;
     }
 
     ifilled_quadrangle(icorners);
@@ -256,10 +260,14 @@ void
 filled_polygon_3dcoords(int points, struct coordinate GPHUGE * coords)
 {
     int i;
+    double x, y;
     gpiPoint *icorners;
     icorners = gp_alloc(points * sizeof(gpiPoint), "filled_polygon3d corners");
-    for (i = 0; i < points; i++)
-	map3d_xy(coords[i].x, coords[i].y, coords[i].z, &icorners[i].x, &icorners[i].y);
+    for (i = 0; i < points; i++) {
+	map3d_xy_double(coords[i].x, coords[i].y, coords[i].z, &x, &y);
+	icorners[i].x = x;
+	icorners[i].y = y;
+    }
 #ifdef EXTENDED_COLOR_SPECS
     if (supply_extended_color_specs) {
 	icorners[0].spec.gray = -1;	/* force solid color */
@@ -279,10 +287,14 @@ void
 filled_polygon_3dcoords_zfixed(int points, struct coordinate GPHUGE * coords, double z)
 {
     int i;
+    double x, y;
     gpiPoint *icorners;
     icorners = gp_alloc(points * sizeof(gpiPoint), "filled_polygon_zfix corners");
-    for (i = 0; i < points; i++)
-	map3d_xy(coords[i].x, coords[i].y, z, &icorners[i].x, &icorners[i].y);
+    for (i = 0; i < points; i++) {
+	map3d_xy_double(coords[i].x, coords[i].y, z, &x, &y);
+	icorners[i].x = x;
+	icorners[i].y = y;
+    }
 #ifdef EXTENDED_COLOR_SPECS
     if (supply_extended_color_specs) {
 	icorners[0].spec.gray = -1;	/* force solid color */
@@ -444,7 +456,7 @@ cbtick_callback(
 	map3d_position_r(&(axis_array[axis].ticdef.offset),
 			 &offsetx, &offsety, "cbtics");
 	/* User-specified different color for the tics text */
-	if (axis_array[axis].ticdef.textcolor.lt != TC_DEFAULT)
+	if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT)
 	    apply_pm3dcolor(&(axis_array[axis].ticdef.textcolor), term);
 	if (color_box.rotation == 'h') {
 	    int y3 = cb_y_from - (term->v_char);
@@ -592,7 +604,7 @@ draw_color_smooth_box(int plot_mode)
 	/* now make boundary around the colour box */
 	if (color_box.border_lt_tag >= 0) {
 	    /* user specified line type */
-	    struct lp_style_type lp;
+	    struct lp_style_type lp = border_lp;
 	    lp_use_properties(&lp, color_box.border_lt_tag, 1);
 	    term_apply_lp_properties(&lp);
 	} else {
@@ -653,9 +665,9 @@ draw_color_smooth_box(int plot_mode)
 	    y += (cb_y_from + cb_y_to) / 2;
 	    if (x<0) x = 0;
 	    if (y<0) y = 0;
-	    if ((*term->text_angle)(TEXT_VERTICAL)) {
+	    if ((*term->text_angle)(CB_AXIS.label.rotate)) {
 		write_multiline(x, y, CB_AXIS.label.text, CENTRE, JUST_TOP,
-				TEXT_VERTICAL, CB_AXIS.label.font);
+				CB_AXIS.label.rotate, CB_AXIS.label.font);
 		(*term->text_angle)(0);
 	    } else {
 		write_multiline(x, y, CB_AXIS.label.text, LEFT, JUST_TOP, 0, CB_AXIS.label.font);
