@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: command.c,v 1.144.2.3 2007/02/11 13:30:30 mikulik Exp $"); }
+static char *RCSid() { return RCSid("$Id: command.c,v 1.144.2.8 2008/01/10 16:08:53 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - command.c */
@@ -1094,15 +1094,23 @@ pause_command()
     }
     if (sleep_time < 0) {
 #ifdef _Windows
-    if (!graphwin.hWndGraph) {
+    if (paused_for_mouse && !graphwin.hWndGraph) {
 	if (interactive) { /* cannot wait for Enter in a non-interactive session without the graph window */
 	    char tmp[512];
+	    if (buf) fprintf(stderr,"%s\n", buf);
 	    fgets(tmp, 512, stdin); /* graphical window not yet initialized, wait for any key here */
 	}
     } else { /* pausing via graphical windows */
+	int tmp = paused_for_mouse;
+	if (buf && paused_for_mouse) fprintf(stderr,"%s\n", buf);
 	if (!Pause(buf)) {
-	    free(buf);
-	    bail_to_command_line();
+	    if (!tmp) {
+		free(buf);
+		bail_to_command_line();
+	    } else {
+		if (!graphwin.hWndGraph) 
+		    bail_to_command_line();
+	    }
 	}
     }
 #elif defined(OS2)
@@ -1341,6 +1349,9 @@ replot_command()
 	int_error(c_token, "cannot replot data coming from stdin");
 #endif
     }
+    if (!term) /* unknown terminal */
+	int_error(c_token, "use 'set term' to set terminal type first");
+
     c_token++;
     SET_CURSOR_WAIT;
     if (term->flags & TERM_INIT_ON_REPLOT)
@@ -2416,6 +2427,7 @@ rlgets(char *s, size_t n, const char *prompt)
 	}
     }
     if (line) {
+	/* s will be NUL-terminated here */
 	safe_strncpy(s, line + leftover, n);
 	leftover += strlen(s);
 	if (line[leftover] == NUL)
@@ -2594,7 +2606,12 @@ cgets_emu(char *str, int len)
 /* this function is called for non-interactive operation. Its usage is
  * like fgets(), but additionally it checks for ipc events from the
  * terminals waitforinput() (if USE_MOUSE, and terminal is
- * mouseable). This function will be used when reading from a pipe. */
+ * mouseable). This function will be used when reading from a pipe.
+ * fgets() reads in at most one less than size characters from stream and
+ * stores them into the buffer pointed to by s.
+ * Reading stops after an EOF or a newline.  If a newline is read, it is
+ * stored into the buffer.  A '\0' is stored  after the last character in
+ * the buffer. */
 static char*
 fgets_ipc(
     char *dest,			/* string to fill */
@@ -2607,17 +2624,20 @@ fgets_ipc(
 	size_t i=0;		/* position inside dest */
 
 	dest[0] = '\0';
-	for (i=0; i < len; i++) {
+	for (i=0; i < len-1; i++) {
 	    c = term->waitforinput();
 	    if ('\n' == c) {
-		dest[i] = '\0';
-		return dest;
+		dest[i] = '\n';
+		i++;
+		break;
 	    } else if (EOF == c) {
+		dest[i] = '\0';
 		return (char*) 0;
 	    } else {
 		dest[i] = c;
 	    }
 	}
+	dest[i] = '\0';
 	return dest;
     } else
 #endif
@@ -2663,6 +2683,8 @@ read_line(const char *prompt)
 		return (1);	/* exit gnuplot */
 	} else {
 	    /* normal line input */
+	    /* gp_input_line must be NUL-terminated for strlen not to pass the
+	     * the bounds of this array */
 	    last = strlen(gp_input_line) - 1;
 	    if (last >= 0) {
 		if (gp_input_line[last] == '\n') {	/* remove any newline */
@@ -2672,12 +2694,10 @@ read_line(const char *prompt)
 			--last;
 		} else if (last + 2 >= gp_input_line_len) {
 		    extend_input_line();
-		    if (gp_input_line[last] != '\\') {
-			/* read rest of line, don't print "> " */
-			start = last + 1;
-			more = TRUE;
-			continue;
-		    }
+		    /* read rest of line, don't print "> " */
+		    start = last + 1;
+		    more = TRUE;
+		    continue;
 		    /* else fall through to continuation handling */
 		} /* if(grow buffer?) */
 		if (gp_input_line[last] == '\\') {
