@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: fit.c,v 1.39 2004/04/13 17:23:53 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: fit.c,v 1.56.2.1 2007/07/19 16:11:23 sfeam Exp $"); }
 #endif
 
 /*  NOTICE: Change of Copyright Status
@@ -50,10 +50,11 @@ static char *RCSid() { return RCSid("$Id: fit.c,v 1.39 2004/04/13 17:23:53 broek
 #include <signal.h>
 
 #include "alloc.h"
-#include "axis.h" 
+#include "axis.h"
 #include "command.h"
 #include "datafile.h"
 #include "eval.h"
+#include "gp_time.h"
 #include "matrix.h"
 #include "plot.h"
 #include "misc.h"
@@ -89,7 +90,7 @@ static int kbhit(void);
 #endif
 
 enum marq_res {
-    OK, ERROR, BETTER, WORSE
+    OK, ML_ERROR, BETTER, WORSE
 };
 typedef enum marq_res marq_res_t;
 
@@ -130,8 +131,6 @@ char fitbuf[256];
 char *fitlogfile = NULL;
 
 #ifdef GP_FIT_ERRVARS
-/* NEW 20030131: should we place parameter errors into user-defined
- * variables?  */
 TBOOLEAN fit_errorvariables = FALSE;
 #endif /* GP_FIT_ERRVARS */
 
@@ -141,7 +140,7 @@ static int max_data;
 static int max_params;
 
 static double epsilon = 1e-5;	/* convergence limit */
-static int maxiter = 0;	
+static int maxiter = 0;
 
 static char fit_script[256];
 
@@ -150,7 +149,7 @@ static char fit_script[256];
 static const char fitlogfile_default[] = "fit.log";
 static const char GNUFITLOG[] = "FIT_LOG";
 
-static const char *FIXED = "# FIXED";
+static const char *GP_FIXED = "# FIXED";
 static const char *FITLIMIT = "FIT_LIMIT";
 static const char *FITSTARTLAMBDA = "FIT_START_LAMBDA";
 static const char *FITLAMBDAFACTOR = "FIT_LAMBDA_FACTOR";
@@ -191,12 +190,13 @@ static TBOOLEAN fit_interrupt __PROTO((void));
 static TBOOLEAN regress __PROTO((double a[]));
 static void show_fit __PROTO((int i, double chisq, double last_chisq, double *a,
 			      double lambda, FILE * device));
+static void log_axis_restriction __PROTO((FILE *log_f, AXIS_INDEX axis));
 static TBOOLEAN is_empty __PROTO((char *s));
 static TBOOLEAN is_variable __PROTO((char *s));
 static double getdvar __PROTO((const char *varname));
 static int getivar __PROTO((const char *varname));
 static void setvar __PROTO((char *varname, struct value data));
-static char *get_next_word __PROTO((char **s, char *subst)); 
+static char *get_next_word __PROTO((char **s, char *subst));
 static double createdvar __PROTO((char *varname, double value));
 static void splitpath __PROTO((char *s, char *p, char *f));
 static void backup_file __PROTO((char *, const char *));
@@ -212,8 +212,7 @@ static void setvarerr __PROTO((char *varname, double value));
 *****************************************************************/
 
 size_t
-wri_to_fil_last_fit_cmd(fp)
-FILE *fp;
+wri_to_fil_last_fit_cmd(FILE *fp)
 {
     if (fp == NULL)
 	return strlen(last_fit_command);
@@ -227,8 +226,7 @@ FILE *fp;
 *****************************************************************/
 
 static RETSIGTYPE
-ctrlc_handle(an_int)
-    int an_int;
+ctrlc_handle(int an_int)
 {
     (void) an_int;		/* avoid -Wunused warning */
     /* reinstall signal handler (necessary on SysV) */
@@ -269,7 +267,7 @@ int getchx __PROTO((void));
 int
 getchx()
 {
-    register int c = getch();
+    int c = getch();
     if (!c || c == 0xE0) {
 	c <<= 8;
 	c |= getch();
@@ -297,7 +295,7 @@ error_ex()
 	log_f = NULL;
     }
     if (func.at) {
-	free(func.at);		/* release perm. action table */
+	free_at(func.at);		/* release perm. action table */
 	func.at = (struct at_type *) NULL;
     }
     /* restore original SIGINT function */
@@ -313,11 +311,7 @@ error_ex()
     Marquardt's nonlinear least squares fit
 *****************************************************************/
 static marq_res_t
-marquardt(a, C, chisq, lambda)
-double a[];
-double **C;
-double *chisq;
-double *lambda;
+marquardt(double a[], double **C, double *chisq, double *lambda)
 {
     int i, j;
     static double *da = 0,	/* delta-step of the parameter */
@@ -356,7 +350,7 @@ double *lambda;
 	for (i = 0; i < num_params; i++)
 	    for (j = 0; j < i; j++)
 		C[num_data + i][j] = 0, C[num_data + j][i] = 0;
-	return analyze_ret ? OK : ERROR;
+	return analyze_ret ? OK : ML_ERROR;
     }
     /* once converged, free dynamic allocated vars */
 
@@ -395,7 +389,7 @@ double *lambda;
 
     if (!analyze(temp_a, tmp_C, tmp_d, &tmp_chisq)) {
 	/* FIXME: will never be reached: always returns TRUE */
-	return ERROR;
+	return ML_ERROR;
     }
     if (tmp_chisq < *chisq) {	/* Success, accept new solution */
 	if (*lambda > MIN_LAMBDA) {
@@ -418,26 +412,18 @@ double *lambda;
 }
 
 
-/* FIXME: in the new code, this function doesn't really do enough to be
- * useful. Maybe it ought to be deleted, i.e. integrated with
- * calculate() ?
- */
 /*****************************************************************
     compute chi-square and numeric derivations
 *****************************************************************/
+/* used by marquardt to evaluate the linearized fitting matrix C and
+ * vector d, fills in only the top part of C and d I don't use a
+ * temporary array zfunc[] any more. Just use d[] instead.  */
+/* FIXME: in the new code, this function doesn't really do enough to
+ * be useful. Maybe it ought to be deleted, i.e. integrated with
+ * calculate() ? */
 static TBOOLEAN
-analyze(a, C, d, chisq)
-double a[];
-double **C;
-double d[];
-double *chisq;
+analyze(double a[], double **C, double d[], double *chisq)
 {
-/*
- *  used by marquardt to evaluate the linearized fitting matrix C
- *  and vector d, fills in only the top part of C and d
- *  I don't use a temporary array zfunc[] any more. Just use
- *  d[] instead.
- */
     int i, j;
 
     *chisq = 0;
@@ -462,10 +448,7 @@ double *chisq;
     compute function values and partial derivatives of chi-square
 *****************************************************************/
 static void
-calculate(zfunc, dzda, a)
-double *zfunc;
-double **dzda;
-double a[];
+calculate(double *zfunc, double **dzda, double a[])
 {
     int k, p;
     double tmp_a;
@@ -517,9 +500,7 @@ double a[];
     call internal gnuplot functions
 *****************************************************************/
 static void
-call_gnuplot(par, data)
-double *par;
-double *data;
+call_gnuplot(double *par, double *data)
 {
     int i;
     struct value v;
@@ -577,7 +558,7 @@ fit_interrupt()
 		    (void) Gcomplex(&v, a[i], 0.0);
 		    setvar(par_name[i], v);
 		}
-		safe_strncpy(input_line, tmp, input_line_len);
+		safe_strncpy(gp_input_line, tmp, gp_input_line_len);
 		(void) do_line();
 	    }
 	}
@@ -590,12 +571,12 @@ fit_interrupt()
     frame routine for the marquardt-fit
 *****************************************************************/
 static TBOOLEAN
-regress(a)
-    double a[];
+regress(double a[])
 {
     double **covar, *dpar, **C, chisq, last_chisq, lambda;
     int iter, i, j;
     marq_res_t res;
+    struct udvt_entry *v;	/* For exporting results to the user */
 
     chisq = last_chisq = INFINITY;
     C = matr(num_data + num_params, num_params);
@@ -606,13 +587,18 @@ regress(a)
     ctrlc_setup();
 
     /* Initialize internal variables and 1st chi-square check */
-    if ((res = marquardt(a, C, &chisq, &lambda)) == ERROR)
+    if ((res = marquardt(a, C, &chisq, &lambda)) == ML_ERROR)
 	Eex("FIT: error occurred during fit");
     res = BETTER;
 
     show_fit(iter, chisq, chisq, a, lambda, STANDARD);
     show_fit(iter, chisq, chisq, a, lambda, log_f);
 
+    /* Reset flag describing fit result status */
+	v = add_udv_by_name("FIT_CONVERGED");
+	v->udv_undef = FALSE;
+	Ginteger(&v->udv_value, 0);
+
     /* MAIN FIT LOOP: do the regression iteration */
 
     /* HBB 981118: initialize new variable 'user_break' */
@@ -628,7 +614,7 @@ regress(a)
  *
  *  I hope that other OSes do it better, if not... add #ifdefs :-(
  *  EMX does not have kbhit.
- * 
+ *
  *  HBB: I think this can be enabled for DJGPP V2. SIGINT is actually
  *  handled there, AFAIK.
  */
@@ -653,7 +639,7 @@ regress(a)
 	}
 	if ((res = marquardt(a, C, &chisq, &lambda)) == BETTER)
 	    show_fit(iter, chisq, last_chisq, a, lambda, STANDARD);
-    } while ((res != ERROR)
+    } while ((res != ML_ERROR)
 	     && (lambda < MAX_LAMBDA)
 	     && ((maxiter == 0) || (iter <= maxiter))
 	     && (res == WORSE
@@ -677,6 +663,9 @@ regress(a)
 	Dblf2("\nThe fit was stopped by the user after %d iterations.\n", iter);
     } else {
 	Dblf2("\nAfter %d iterations the fit converged.\n", iter);
+	v = add_udv_by_name("FIT_CONVERGED");
+	v->udv_undef = FALSE;
+	Ginteger(&v->udv_value, 1);
     }
 
     Dblf2("final sum of squares of residuals : %g\n", chisq);
@@ -686,16 +675,16 @@ regress(a)
 	Dblf2("abs. change during last iteration : %g\n\n", (chisq - last_chisq));
     }
 
-    if (res == ERROR)
+    if (res == ML_ERROR)
 	Eex("FIT: error occurred during fit");
 
     /* compute errors in the parameters */
 
 #ifdef GP_FIT_ERRVARS
-    if (fit_errorvariables) 
+    if (fit_errorvariables)
 	/* Set error variable to zero before doing this */
 	/* Thus making sure they are created */
-	for (i = 0; i < num_params; i++) 
+	for (i = 0; i < num_params; i++)
 	    setvarerr(par_name[i], 0.0);
 #endif
 
@@ -717,10 +706,23 @@ regress(a)
 	for (k = 0; k < num_params; k++)
 	    Dblf3("%-15.15s = %-15g\n", par_name[k], a[k]);
     } else {
-	Dblf2("degrees of freedom (ndf) : %d\n", num_data - num_params);
-	Dblf2("rms of residuals      (stdfit) = sqrt(WSSR/ndf)      : %g\n", sqrt(chisq /
-										  (num_data - num_params)));
-	Dblf2("variance of residuals (reduced chisquare) = WSSR/ndf : %g\n\n", chisq / (num_data - num_params));
+	int ndf          = num_data - num_params; 
+	double stdfit    = sqrt(chisq/ndf);
+	
+	Dblf2("degrees of freedom    (FIT_NDF)                        : %d\n", ndf);
+	Dblf2("rms of residuals      (FIT_STDFIT) = sqrt(WSSR/ndf)    : %g\n", stdfit);
+	Dblf2("variance of residuals (reduced chisquare) = WSSR/ndf   : %g\n\n", chisq/ndf);
+
+	/* Export these to user-accessible variables */
+	v = add_udv_by_name("FIT_NDF");
+	v->udv_undef = FALSE;
+	Ginteger(&v->udv_value, ndf);
+	v = add_udv_by_name("FIT_STDFIT");
+	v->udv_undef = FALSE;
+	Gcomplex(&v->udv_value, stdfit, 0);
+	v = add_udv_by_name("FIT_WSSR");
+	v->udv_undef = FALSE;
+	Gcomplex(&v->udv_value, chisq, 0);
 
 	/* get covariance-, Korrelations- and Kurvature-Matrix */
 	/* and errors in the parameters                     */
@@ -814,21 +816,19 @@ regress(a)
     display actual state of the fit
 *****************************************************************/
 static void
-show_fit(i, chisq, last_chisq, a, lambda, device)
-int i;
-double chisq;
-double last_chisq;
-double *a;
-double lambda;
-FILE *device;
+show_fit(
+    int i,
+    double chisq, double last_chisq,
+    double *a, double lambda,
+    FILE *device)
 {
     int k;
 
     fprintf(device, "\n\n\
-Iteration %d\n\
-WSSR        : %-15g   delta(WSSR)/WSSR   : %g\n\
-delta(WSSR) : %-15g   limit for stopping : %g\n\
-lambda	  : %g\n\n%s parameter values\n\n",
+ Iteration %d\n\
+ WSSR        : %-15g   delta(WSSR)/WSSR   : %g\n\
+ delta(WSSR) : %-15g   limit for stopping : %g\n\
+ lambda	  : %g\n\n%s parameter values\n\n",
 	    i, chisq, chisq > NEARLY_ZERO ? (chisq - last_chisq) / chisq : 0.0,
 	    chisq - last_chisq, epsilon, lambda,
 	    (i > 0 ? "resultant" : "initial set of free"));
@@ -842,8 +842,7 @@ lambda	  : %g\n\n%s parameter values\n\n",
     is_empty: check for valid string entries
 *****************************************************************/
 static TBOOLEAN
-is_empty(s)
-char *s;
+is_empty(char *s)
 {
     while (*s == ' ' || *s == '\t' || *s == '\n')
 	s++;
@@ -855,9 +854,7 @@ char *s;
     get next word of a multi-word string, advance pointer
 *****************************************************************/
 static char *
-get_next_word(s, subst)
-char **s;
-char *subst;
+get_next_word(char **s, char *subst)
 {
     char *tmp = *s;
 
@@ -877,8 +874,7 @@ char *subst;
     check for variable identifiers
 *****************************************************************/
 static TBOOLEAN
-is_variable(s)
-char *s;
+is_variable(char *s)
 {
     while (*s != '\0') {
 	if (!isalnum((unsigned char) *s) && *s != '_')
@@ -903,29 +899,9 @@ init_fit()
 ******************************************************************/
 
 static void
-setvar(varname, data)
-    char *varname;
-    struct value data;
+setvar(char *varname, struct value data)
 {
-    register struct udvt_entry *udv_ptr = first_udv, *last = first_udv;
-
-    /* check if it's already in the table... */
-
-    while (udv_ptr) {
-	last = udv_ptr;
-	if (!strcmp(varname, udv_ptr->udv_name))
-	    break;
-	udv_ptr = udv_ptr->next_udv;
-    }
-
-    if (!udv_ptr) {             /* generate new entry */
-	udv_ptr = (struct udvt_entry *) gp_alloc(sizeof(struct udvt_entry), "fit setvar");
-	udv_ptr->udv_name = NULL;
-	last->next_udv = udv_ptr;
-	udv_ptr->next_udv = NULL;
-    }
-    udv_ptr->udv_name = gp_realloc(udv_ptr->udv_name, strlen(varname) + 1, "user var");
-    memcpy(udv_ptr->udv_name, varname, strlen(varname) + 1);
+    struct udvt_entry *udv_ptr = add_udv_by_name(varname);
     udv_ptr->udv_value = data;
     udv_ptr->udv_undef = FALSE;
 }
@@ -938,15 +914,13 @@ setvar(varname, data)
             and then set it.
 ******************************************************************/
 static void
-setvarerr(varname, value)
-    char *varname;
-    double value;
+setvarerr(char *varname, double value)
 {
 	struct value errval;    /* This will hold the gnuplot value created from value*/
 	char* pErrValName;      /* The name of the (new) error variable */
 
 	/* Create the variable name by appending _err */
-	pErrValName = (char*)gp_alloc(strlen(varname)+sizeof(char)+4, "");
+	pErrValName = gp_alloc(strlen(varname)+sizeof(char)+4, "");
 
 	sprintf(pErrValName,"%s_err",varname);
 	Gcomplex(&errval, value, 0.0);
@@ -959,10 +933,9 @@ setvarerr(varname, value)
     Read INTGR Variable value, return 0 if undefined or wrong type
 *****************************************************************/
 static int
-getivar(varname)
-const char *varname;
+getivar(const char *varname)
 {
-    register struct udvt_entry *udv_ptr = first_udv;
+    struct udvt_entry *udv_ptr = first_udv;
 
     while (udv_ptr) {
 	if (!strcmp(varname, udv_ptr->udv_name))
@@ -980,10 +953,9 @@ const char *varname;
    I dont think it's a problem that it's an integer - div
 *****************************************************************/
 static double
-getdvar(varname)
-const char *varname;
+getdvar(const char *varname)
 {
-    register struct udvt_entry *udv_ptr = first_udv;
+    struct udvt_entry *udv_ptr = first_udv;
 
     for (; udv_ptr; udv_ptr = udv_ptr->next_udv)
 	if (strcmp(varname, udv_ptr->udv_name) == 0)
@@ -999,11 +971,9 @@ const char *varname;
    - create it with value INITIAL_VALUE if not found or undefined
 *****************************************************************/
 static double
-createdvar(varname, value)
-char *varname;
-double value;
+createdvar(char *varname, double value)
 {
-    register struct udvt_entry *udv_ptr = first_udv;
+    struct udvt_entry *udv_ptr = first_udv;
 
     for (; udv_ptr; udv_ptr = udv_ptr->next_udv)
 	if (strcmp(varname, udv_ptr->udv_name) == 0) {
@@ -1031,14 +1001,11 @@ double value;
     Split Identifier into path and filename
 *****************************************************************/
 static void
-splitpath(s, p, f)
-    char *s;
-    char *p;
-    char *f;
+splitpath(char *s, char *p, char *f)
 {
-    register char *tmp;
-    tmp = s + strlen(s) - 1;
-    while (*tmp != '\\' && *tmp != '/' && *tmp != ':' && tmp - s >= 0)
+    char *tmp = s + strlen(s) - 1;
+
+    while (tmp >= s && *tmp != '\\' && *tmp != '/' && *tmp != ':')
 	tmp--;
     /* FIXME HBB 20010121: unsafe! Sizes of 'f' and 'p' are not known.
      * May write past buffer end. */
@@ -1055,8 +1022,7 @@ splitpath(s, p, f)
     write the actual parameters to start parameter file
 *****************************************************************/
 void
-update(pfile, npfile)
-char *pfile, *npfile;
+update(char *pfile, char *npfile)
 {
     char fnam[256], path[256], sstr[256], pname[64], tail[127], *s = sstr, *tmp, c;
     char ifilename[256], *ofilename;
@@ -1156,9 +1122,7 @@ char *pfile, *npfile;
 /* tofile must point to a char array[] or allocated data. See update() */
 
 static void
-backup_file(tofile, fromfile)
-    char *tofile;
-    const char *fromfile;
+backup_file(char *tofile, const char *fromfile)
 {
 #if defined (WIN32) || defined(MSDOS) || defined(VMS)
     char *tmpn;
@@ -1211,6 +1175,39 @@ backup_file(tofile, fromfile)
     Eex3("Could not rename file %s to %s", fromfile, tofile);
 }
 
+/* A modified copy of save.c:save_range(), but this one reports
+ * _current_ values, not the 'set' ones, by default */
+static void
+log_axis_restriction(FILE *log_f, AXIS_INDEX axis)
+{
+    char s[80];
+    AXIS *this_axis = axis_array + axis;
+
+    fprintf(log_f, "\t%s range restricted to [", axis_defaults[axis].name);
+    if (this_axis->autoscale & AUTOSCALE_MIN) {
+	putc('*', log_f);
+    } else if (this_axis->is_timedata) {
+	putc('"', log_f);
+	gstrftime(s, 80, this_axis->timefmt, this_axis->min);
+	fputs(s, log_f);
+	putc('"', log_f);
+    } else {
+	fprintf(log_f, "%#g", this_axis->min);
+    }
+
+    fputs(" : ", log_f);
+    if (this_axis->autoscale & AUTOSCALE_MAX) {
+	putc('*', log_f);
+    } else if (this_axis->is_timedata) {
+	putc('"', log_f);
+	gstrftime(s, 80, this_axis->timefmt, this_axis->max);
+	fputs(s, log_f);
+	putc('"', log_f);
+    } else {
+	fprintf(log_f, "%#g", this_axis->max);
+    }
+    fputs("]\n", log_f);
+}
 
 /*****************************************************************
     Interface to the classic gnuplot-software
@@ -1231,27 +1228,27 @@ fit_command()
     double tmpd;
     time_t timer;
     int token1, token2, token3;
-    char *tmp;
+    char *tmp, *file_name;
 
     c_token++;
 
     /* first look for a restricted x fit range... */
-    
+
     /* put stuff into arrays to simplify access */
     AXIS_INIT3D(FIRST_X_AXIS, 0, 0);
     AXIS_INIT3D(FIRST_Y_AXIS, 0, 0);
     AXIS_INIT3D(FIRST_Z_AXIS, 0, 1);
 
-    /* use global default indices in axis.c to simplify access to 
+    /* use global default indices in axis.c to simplify access to
      * per-axis variables */
     x_axis = FIRST_X_AXIS;
     y_axis = FIRST_Y_AXIS;
-    z_axis = FIRST_Z_AXIS;    
-    
+    z_axis = FIRST_Z_AXIS;
+
     PARSE_NAMED_RANGE(FIRST_X_AXIS, dummy_x);
     PARSE_NAMED_RANGE(FIRST_Y_AXIS, dummy_y);
     /* HBB 980401: new: allow restricting the z range as well */
-    if (equals(c_token, "[")) 
+    if (equals(c_token, "["))
       zrange_token = c_token;
     PARSE_RANGE(FIRST_Z_AXIS);
 
@@ -1261,7 +1258,7 @@ fit_command()
     token1 = c_token;
 
     if (func.at) {
-	free(func.at);
+	free_at(func.at);
 	func.at = NULL;		/* in case perm_at() does int_error */
     }
     dummy_func = &func;
@@ -1284,9 +1281,17 @@ fit_command()
 
     token2 = c_token;
 
-    /* use datafile module to parse the datafile and qualifiers */
+    /* get filename */
+    file_name = try_to_get_string();
+    if (!file_name)
+	int_error(c_token, "missing filename");
 
-    columns = df_open(4);	/* up to 4 using specs allowed */
+    /* use datafile module to parse the datafile and qualifiers */
+    df_set_plot_mode(MODE_QUERY);  /* Does nothing except for binary datafiles */
+    columns = df_open(file_name, 4);	/* up to 4 using specs allowed */
+    free(file_name);
+    if (columns < 0)
+	int_error(NO_CARET,"Can't read data file");
     if (columns == 1)
 	int_error(c_token, "Need 2 to 4 using specs");
     is_a_3d_fit = (columns == 4);
@@ -1361,7 +1366,7 @@ fit_command()
 	printf("Lambda scaling factors reset:  %g\n", lambda_up_factor);
     }
     *fit_script = NUL;
-    if ((tmp = getenv(FITSCRIPT)) != NULL) 
+    if ((tmp = getenv(FITSCRIPT)) != NULL)
 	safe_strncpy(fit_script, tmp, sizeof(fit_script));
 
     {
@@ -1385,6 +1390,12 @@ fit_command()
 	free(line);
     }
 
+    /* HBB 20040201: somebody insisted they need this ... */
+    if (AUTOSCALE_BOTH != (X_AXIS.autoscale & AUTOSCALE_BOTH))
+	log_axis_restriction(log_f, x_axis);
+    if (AUTOSCALE_BOTH != (Y_AXIS.autoscale & AUTOSCALE_BOTH))
+	log_axis_restriction(log_f, y_axis);
+
     max_data = MAX_DATA;
     fit_x = vec(max_data);	/* start with max. value */
     fit_y = vec(max_data);
@@ -1416,6 +1427,7 @@ fit_command()
 	} /* if (need to extend storage space) */
 
 	switch (i) {
+	case DF_MISSING:
 	case DF_UNDEFINED:
 	case DF_FIRST_BLANK:
 	case DF_SECOND_BLANK:
@@ -1506,7 +1518,7 @@ fit_command()
 
     max_params = MAX_PARAMS;	/* HBB 971023: make this resizeable */
 
-    if (!equals(c_token, "via"))
+    if (!equals(c_token++, "via"))
 	int_error(c_token, "Need via and either parameter list or file");
 
     a = vec(max_params);
@@ -1514,24 +1526,26 @@ fit_command()
 				   "fit param");
     num_params = 0;
 
-    if (isstring(++c_token)) {	/* It's a parameter *file* */
+    if (isstringvalue(c_token)) {	/* It's a parameter *file* */
 	TBOOLEAN fixed;
 	double tmp_par;
 	char c, *s;
 	char sstr[MAX_LINE_LEN + 1];
 	FILE *f;
 
-	quote_str(sstr, c_token++, MAX_LINE_LEN);
+	static char *viafile = NULL;
+	free(viafile);			/* Free previous name, if any */
+	viafile = try_to_get_string();	/* Cannot fail since isstringvalue succeeded */
+	fprintf(log_f, "fitted parameters and initial values from file: %s\n\n", viafile);
+	if (!(f = loadpath_fopen(viafile, "r")))
+	    Eex2("could not read parameter-file %s", sstr);
 
 	/* get parameters and values out of file and ignore fixed ones */
 
-	fprintf(log_f, "fitted parameters and initial values from file: %s\n\n", sstr);
-	if (!(f = loadpath_fopen(sstr, "r")))
-	    Eex2("could not read parameter-file %s", sstr);
 	while (TRUE) {
-	    if (!fgets(s = sstr, (int) sizeof(sstr), f))	/* EOF found */
+	    if (!fgets(s = sstr, sizeof(sstr), f))	/* EOF found */
 		break;
-	    if ((tmp = strstr(s, FIXED)) != NULL) {	/* ignore fixed params */
+	    if ((tmp = strstr(s, GP_FIXED)) != NULL) {	/* ignore fixed params */
 		*tmp = NUL;
 		fprintf(log_f, "FIXED:  %s\n", s);
 		fixed = TRUE;
@@ -1633,10 +1647,10 @@ fit_command()
     free(a);
     free(par_name);
     if (func.at) {
-	free(func.at);		/* release perm. action table */
+	free_at(func.at);		/* release perm. action table */
 	func.at = (struct at_type *) NULL;
     }
-    safe_strncpy(last_fit_command, input_line, sizeof(last_fit_command));
+    safe_strncpy(last_fit_command, gp_input_line, sizeof(last_fit_command));
 }
 
 #if defined(ATARI) || defined(MTOS)
@@ -1661,9 +1675,7 @@ static void
 Dblfn(const char *fmt, ...)
 #else
 static void
-Dblfn(fmt, va_alist)
-const char *fmt;
-va_dcl
+Dblfn(const char *fmt, va_dcl)
 #endif
 {
 #ifdef VA_START
@@ -1698,12 +1710,12 @@ getfitlogfile()
 
 	if (tmp != NULL && *tmp != '\0') {
 	    char *tmp2 = tmp + (strlen(tmp) - 1);
-	    
+
 	    /* if given log file name ends in path separator, treat it
 	     * as a directory to store the default "fit.log" in */
 	    if (*tmp2 == '/' || *tmp2 == '\\') {
 		logfile = gp_alloc(strlen(tmp)
-				   + strlen(fitlogfile_default) + 1, 
+				   + strlen(fitlogfile_default) + 1,
 				   "logfile");
 		strcpy(logfile, tmp);
 		strcat(logfile, fitlogfile_default);
