@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: graph3d.c,v 1.157.2.5 2006/12/24 00:03:38 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: graph3d.c,v 1.157.2.16 2008/03/08 17:51:43 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - graph3d.c */
@@ -110,7 +110,7 @@ int iso_samples_2 = ISO_SAMPLES;
 
 double xscale3d, yscale3d, zscale3d;
 
-typedef enum { ALLGRID, FRONTGRID, BACKGRID } WHICHGRID;
+typedef enum { ALLGRID, FRONTGRID, BACKGRID, BORDERONLY } WHICHGRID;
 
 static void plot3d_impulses __PROTO((struct surface_points * plot));
 static void plot3d_lines __PROTO((struct surface_points * plot));
@@ -500,9 +500,9 @@ static void
 place_arrows3d(int layer)
 {
     struct arrow_def *this_arrow;
+    BoundingBox *clip_save = clip_area;
 
-    /* Allow arrows to run off the plot, so long as they are still on
-     * the canvas */
+    /* Allow arrows to run off the plot, so long as they are still on the canvas */
     if (term->flags & TERM_CAN_CLIP)
       clip_area = NULL;
     else
@@ -522,6 +522,7 @@ place_arrows3d(int layer)
 	    FPRINTF((stderr,"place_arrows3d: skipping out-of-bounds arrow\n"));
 	}
     }
+    clip_area = clip_save;
 }
 
 /* we precalculate features of the key, to save lots of nested
@@ -596,7 +597,7 @@ do_3dplot(
 
     /* Allow an absolute setting of base_z via "set xyplane at <z>" */
     if (xyplane.absolute)
-	base_z = xyplane.xyplane_z;
+	base_z = AXIS_LOG_VALUE(0, xyplane.xyplane_z);
 
     /*  see comment accompanying similar tests of x_min/x_max and y_min/y_max
      *  in graphics.c:do_plot(), for history/rationale of these tests */
@@ -640,13 +641,12 @@ do_3dplot(
     /* HBB 20040331: but not if in hidden3d mode */
     if (!hidden3d && grid_layer == 0)
 	draw_3d_graphbox(plots, pcount, ALLGRID, LAYER_BACK);
+    else if (splot_map && border_layer == 0)
+	draw_3d_graphbox(plots, pcount, BORDERONLY, LAYER_BACK);
 
     /* Draw PM3D color key box */
     if (!quick) {
-	can_pm3d = is_plot_with_palette() && !make_palette()
-	    && term->set_color;
-	if (can_pm3d && is_plot_with_colorbox())
-	    draw_color_smooth_box(MODE_SPLOT);
+	can_pm3d = is_plot_with_palette() && !make_palette() && term->set_color;
     }
 
 #ifdef USE_GRID_LAYERS
@@ -734,6 +734,10 @@ do_3dplot(
 	}
     }
 
+    /* Add 'back' color box */
+    if (!quick && can_pm3d && is_plot_with_colorbox() && color_box.layer == LAYER_BACK)
+	draw_color_smooth_box(MODE_SPLOT);
+
     /* Add 'back' rectangles */
     place_rectangles(first_object, 0, 3, &clip_splot_map);
 
@@ -1175,8 +1179,14 @@ do_3dplot(
 			if (use_palette && thiscontour_lp_properties.pm3d_color.type == TC_Z)
 			    set_color( cb2gray( z2cb(cntrs->z) ) );
 			else {
-			    (*t->linetype) (++thiscontour_lp_properties.l_type);
-			    thiscontour_lp_properties.use_palette = 0;
+			    if (prefer_line_styles && label_contours) {
+				struct lp_style_type ls = thiscontour_lp_properties;
+				lp_use_properties(&ls, ++thiscontour_lp_properties.l_type, FALSE);
+				term_apply_lp_properties(&ls);
+			    } else {
+				(*t->linetype) (++thiscontour_lp_properties.l_type);
+				thiscontour_lp_properties.use_palette = 0;
+			    }
 			}
 
 			if (key->visible) {
@@ -1314,13 +1324,17 @@ do_3dplot(
      * the whole shebang now, otherwise only the front part. */
     if (hidden3d || grid_layer == 1)
 	draw_3d_graphbox(plots, pcount, ALLGRID, LAYER_FRONT);
-    else if (splot_map && (border_layer == 1))
-	draw_3d_graphbox(plots, pcount, ALLGRID, LAYER_FRONT);
     else if (grid_layer == -1)
 	draw_3d_graphbox(plots, pcount, FRONTGRID, LAYER_FRONT);
+    if (splot_map && (border_layer == 1))
+	draw_3d_graphbox(plots, pcount, BORDERONLY, LAYER_FRONT);
 
 #endif /* USE_GRID_LAYERS */
 
+    /* Add 'front' color box */
+    if (!quick && can_pm3d && is_plot_with_colorbox() && color_box.layer == LAYER_FRONT)
+	draw_color_smooth_box(MODE_SPLOT);
+
     /* Add 'front' rectangles */
     place_rectangles(first_object, 1, 3, &clip_splot_map);
 
@@ -1374,19 +1388,38 @@ plot3d_impulses(struct surface_points *plot)
     int i;				/* point index */
     unsigned int x, y, xx0, yy0;	/* point in terminal coordinates */
     struct iso_curve *icrvs = plot->iso_crvs;
+    int colortype = plot->lp_properties.pm3d_color.type;
     TBOOLEAN rgb_from_column;
 
     rgb_from_column = can_pm3d && plot->pm3d_color_from_column
-			&& plot->lp_properties.pm3d_color.type == TC_RGB
 			&& plot->lp_properties.pm3d_color.value < 0.0;
 
+    if (colortype == TC_RGB && !rgb_from_column)
+	set_rgbcolor(plot->lp_properties.pm3d_color.lt);
+
     while (icrvs) {
 	struct coordinate GPHUGE *points = icrvs->points;
 
 	for (i = 0; i < icrvs->p_count; i++) {
 
-	    if (rgb_from_column)
-		set_rgbcolor((int)points[i].CRD_COLOR);
+	    switch (colortype) {
+	    case TC_RGB:
+		if (rgb_from_column)
+		    set_rgbcolor((int)points[i].CRD_COLOR);
+		break;
+	    case TC_Z:
+	    case TC_DEFAULT:
+		if (can_pm3d && plot->lp_properties.use_palette) {
+		    if (plot->pm3d_color_from_column)
+			set_color( cb2gray(points[i].CRD_COLOR) );
+		    else
+			set_color( cb2gray( z2cb(points[i].z) ) );
+		}
+		break;
+	    default:
+		/* The other cases were taken care of already */
+		break;
+	    }
 
 	    switch (points[i].type) {
 	    case INRANGE:
@@ -1831,6 +1864,13 @@ cntr3d_lines(struct gnuplot_contours *cntr, struct lp_style_type *lp)
     int i;			/* point index */
     vertex this_vertex;
 
+    /* user may prefer explicit line styles */
+    if (prefer_line_styles && label_contours) {
+	struct lp_style_type ls = *lp;
+	lp_use_properties(&ls, lp->l_type, FALSE);
+	lp = &ls;
+    }
+
     if (draw_contour & CONTOUR_SRF) {
 	map3d_xyz(cntr->coords[0].x, cntr->coords[0].y, cntr->coords[0].z,
 		  &this_vertex);
@@ -2275,6 +2315,8 @@ draw_3d_graphbox(struct surface_points *plot, int plot_num, WHICHGRID whichgrid,
     /* In 'set view map' mode, treat grid as in 2D plots */
     if (splot_map && current_layer != abs(grid_layer))
 	return;
+    if (whichgrid == BORDERONLY)
+	return;
 
     /* Draw ticlabels and axis labels. x axis, first:*/
     if (X_AXIS.ticmode || X_AXIS.label.text) {
@@ -2305,7 +2347,8 @@ draw_3d_graphbox(struct surface_points *plot, int plot_num, WHICHGRID whichgrid,
 	    /* label at xaxis_y + 1/4 of (xaxis_y-other_y) */
 #ifdef USE_GRID_LAYERS /* FIXME: still needed??? what for? */
 	    if ((surface_rot_x <= 90 && BACKGRID != whichgrid) ||
-		(surface_rot_x > 90 && FRONTGRID != whichgrid)) {
+		(surface_rot_x > 90 && FRONTGRID != whichgrid) ||
+		splot_map) {
 #endif
 	    unsigned int x1, y1;
 	    int tmpx, tmpy;
@@ -2398,7 +2441,8 @@ draw_3d_graphbox(struct surface_points *plot, int plot_num, WHICHGRID whichgrid,
 	if (Y_AXIS.label.text) {
 #ifdef USE_GRID_LAYERS
 	    if ((surface_rot_x <= 90 && BACKGRID != whichgrid) ||
-		(surface_rot_x > 90 && FRONTGRID != whichgrid)) {
+		(surface_rot_x > 90 && FRONTGRID != whichgrid) ||
+		splot_map) {
 #endif
 		unsigned int x1, y1;
 		int tmpx, tmpy;
@@ -2645,7 +2689,7 @@ xtick_callback(
 	}
 	TERMCOORD(&v2, x2, y2);
         /* User-specified different color for the tics text */
-	if (axis_array[axis].ticdef.textcolor.lt != TC_DEFAULT)
+	if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT)
 	    apply_pm3dcolor(&(axis_array[axis].ticdef.textcolor), t);
 	angle = axis_array[axis].tic_rotate;
 	if (!(splot_map && angle && term->text_angle(angle)))
@@ -2722,7 +2766,7 @@ ytick_callback(
 	    v2.y -= tic_unity * t->v_tic * axis_array[axis].ticscale;
 	}
         /* User-specified different color for the tics text */
-	if (axis_array[axis].ticdef.textcolor.lt != TC_DEFAULT)
+	if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT)
 	    apply_pm3dcolor(&(axis_array[axis].ticdef.textcolor), t);
 	TERMCOORD(&v2, x2, y2);
 	angle = axis_array[axis].tic_rotate;
@@ -2786,7 +2830,7 @@ ztick_callback(
 	if (!axis_array[axis].tic_in)
 	    x1 -= (term->h_tic) * axis_array[axis].ticscale;
         /* User-specified different color for the tics text */
-	if (axis_array[axis].ticdef.textcolor.lt != TC_DEFAULT)
+	if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT)
 	    apply_pm3dcolor(&(axis_array[axis].ticdef.textcolor), term);
 	write_multiline(x1+offsetx, y1+offsety, text, RIGHT, JUST_CENTRE,
 			0, axis_array[axis].ticdef.font);
@@ -2840,7 +2884,10 @@ map3d_getposition(
 	plot_coords = TRUE;
 	break;
     case graph:
-	*ypos = Y_AXIS.min + *ypos * (Y_AXIS.max - Y_AXIS.min);
+	if (splot_map)
+	    *ypos = Y_AXIS.max - *ypos * (Y_AXIS.max - Y_AXIS.min);
+	else
+	    *ypos = Y_AXIS.min + *ypos * (Y_AXIS.max - Y_AXIS.min);
 	plot_coords = TRUE;
 	break;
     case screen:
@@ -2934,9 +2981,9 @@ map3d_position_r(
 	else
 	    xpos = 0;
 	if (pos->scaley == graph)
-	    ypos = Y_AXIS.min;
+	    ypos = (splot_map) ? Y_AXIS.max : Y_AXIS.min;
 	else
-	    ypos = 0;
+	    ypos = (splot_map) ? Y_AXIS.max : 0;
 	if (pos->scalez == graph)
 	    zpos = Z_AXIS.min;
 	else
@@ -2946,8 +2993,8 @@ map3d_position_r(
 	*y -= yoriginlocal;
     } else {
     /* endpoint `screen' or 'character' coordinates */
-	    *x = xpos;
-	    *y = ypos;
+	*x = xpos;
+	*y = ypos;
     }
     return;
 }
@@ -3124,6 +3171,13 @@ key_sample_point_pm3d(
     if (steps < 2) steps = 2;
     step = ((double)(key_sample_right - key_sample_left)) / steps;
 
+    /* If plot uses a constant color, set it here and then let simpler routine take over */
+    if (plot->lp_properties.use_palette && plot->lp_properties.pm3d_color.type == TC_RGB) {
+	apply_pm3dcolor(&(plot->lp_properties.pm3d_color), term);
+	key_sample_point(xl,yl,pointtype);
+	return;
+    }
+
     /* color gradient only over the cb-values of the surface, if smaller than the
      * cb-axis range (the latter are gray values [0:1]) */
     get_surface_cbminmax(plot, &cbmin, &cbmax);
