@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: mouse.c,v 1.46 2004/04/01 15:50:12 mikulik Exp $"); }
+static char *RCSid() { return RCSid("$Id: mouse.c,v 1.85.2.2 2008/02/28 22:58:25 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - mouse.c */
@@ -36,7 +36,7 @@ static char *RCSid() { return RCSid("$Id: mouse.c,v 1.46 2004/04/01 15:50:12 mik
 
 /*
  * AUTHORS
- * 
+ *
  *   Original Software (October 1999 - January 2000):
  *     Pieter-Tjerk de Boer <ptdeboer@cs.utwente.nl>
  *     Petr Mikulik <mikulik@physics.muni.cz>
@@ -51,11 +51,7 @@ static char *RCSid() { return RCSid("$Id: mouse.c,v 1.46 2004/04/01 15:50:12 mik
 #ifdef USE_MOUSE		/* comment out whole file, otherwise... */
 
 #include "mouse.h"
-#ifdef PM3D
 #include "pm3d.h"
-#endif
-
-
 #include "alloc.h"
 #include "axis.h"
 #include "command.h"
@@ -69,6 +65,14 @@ static char *RCSid() { return RCSid("$Id: mouse.c,v 1.46 2004/04/01 15:50:12 mik
 #include "term_api.h"
 #include "util3d.h"
 
+#ifdef _Windows
+# include "win/winmain.h"
+#endif
+
+#ifdef OS2
+#include "os2/pm_msgs.h"
+#endif
+
 /********************** variables ***********************************************************/
 
 mouse_setting_t mouse_setting = {
@@ -83,6 +87,19 @@ mouse_setting_t mouse_setting = {
     "point pt 1"
 };
 
+/* "usual well-known" keycodes, i.e. those not listed in special_keys in mouse.h
+*/
+static const struct gen_table usual_special_keys[] =
+{
+    { "BackSpace", GP_BackSpace},
+    { "Tab", GP_Tab},
+    { "KP_Enter", GP_KP_Enter},
+    { "Return", GP_Return},
+    { "Escape", GP_Escape},
+    { "Delete", GP_Delete},
+    { NULL, 0}
+};
+
 /* the status of the shift, ctrl and alt keys
 */
 static int modifier_mask = 0;
@@ -143,7 +160,6 @@ static int motion = 0;
 /* values for rot_x and rot_z corresponding to zero position of mouse */
 static float zero_rot_x, zero_rot_z;
 
-typedef void (map_func_type) __PROTO((struct position * pos, unsigned int *x, unsigned int *y, const char *what));
 
 /* bind related stuff */
 
@@ -153,6 +169,7 @@ typedef struct bind_t {
     char modifier;
     char *command;
     char *(*builtin) (struct gp_event_t * ge);
+    TBOOLEAN allwindows;
     struct bind_t *next;
 } bind_t;
 
@@ -182,14 +199,14 @@ static void incr_mousemode __PROTO((const int amount));
 static void incr_clipboardmode __PROTO((const int amount));
 static void UpdateStatuslineWithMouseSetting __PROTO((mouse_setting_t * ms));
 
-static void event_keypress __PROTO((struct gp_event_t * ge));
+static void event_keypress __PROTO((struct gp_event_t * ge, TBOOLEAN current));
 static void ChangeView __PROTO((int x, int z));
 static void event_buttonpress __PROTO((struct gp_event_t * ge));
 static void event_buttonrelease __PROTO((struct gp_event_t * ge));
 static void event_motion __PROTO((struct gp_event_t * ge));
 static void event_modifier __PROTO((struct gp_event_t * ge));
-static void event_print __PROTO((FILE * fp, char *s));
 static void do_save_3dplot __PROTO((struct surface_points *, int, int));
+static void load_mouse_variables __PROTO((double, double, TBOOLEAN, int));
 
 /* builtins */
 static char *builtin_autoscale __PROTO((struct gp_event_t * ge));
@@ -227,24 +244,20 @@ static char *bind_fmt_lhs __PROTO((const bind_t * in));
 static int bind_matches __PROTO((const bind_t * a, const bind_t * b));
 static void bind_display_one __PROTO((bind_t * ptr));
 static void bind_display __PROTO((char *lhs));
+static void bind_all __PROTO((char *lhs));
 static void bind_remove __PROTO((bind_t * b));
 static void bind_append __PROTO((char *lhs, char *rhs, char *(*builtin) (struct gp_event_t * ge)));
-/* void bind_process __PROTO((char *lhs, char *rhs)); */
-/* void bind_remove_all __PROTO((void)); */
 static void recalc_ruler_pos __PROTO((void));
 static void turn_ruler_off __PROTO((void));
-static int nearest_label_tag __PROTO((int x, int y, struct termentry * t, map_func_type *));
+static int nearest_label_tag __PROTO((int x, int y, struct termentry * t));
 static void remove_label __PROTO((int x, int y));
 static void put_label __PROTO((char *label, double x, double y));
-# ifdef OS2
-void send_gpPMmenu __PROTO((FILE * PM_pipe));
-# endif
 
 /********* functions ********************************************/
 
 /* produce a beep */
 static void
-alert(void)
+alert()
 {
 # ifdef GNUPMDRV
     DosBeep(444, 111);
@@ -278,7 +291,7 @@ stpcpy(char *s, const char *p)
 # endif
 
 
-/* a macro to check whether 2D functionality is allowed: 
+/* a macro to check whether 2D functionality is allowed:
    either the plot is a 2D plot, or it is a suitably oriented 3D plot
 */
 # define ALMOST2D      \
@@ -294,15 +307,15 @@ MousePosToGraphPosReal(int xx, int yy, double *x, double *y, double *x2, double
 {
     if (!is_3d_plot) {
 # if 0
-	printf("POS: xleft=%i, xright=%i, ybot=%i, ytop=%i\n", xleft, xright, ybot, ytop);
+	printf("POS: plot_bounds.xleft=%i, plot_bounds.xright=%i, plot_bounds.ybot=%i, plot_bounds.ytop=%i\n", plot_bounds.xleft, plot_bounds.xright, plot_bounds.ybot, plot_bounds.ytop);
 # endif
-	if (xright == xleft)
+	if (plot_bounds.xright == plot_bounds.xleft)
 	    *x = *x2 = 1e38;	/* protection */
 	else {
 	    *x = AXIS_MAPBACK(FIRST_X_AXIS, xx);
 	    *x2 = AXIS_MAPBACK(SECOND_X_AXIS, xx);
 	}
-	if (ytop == ybot)
+	if (plot_bounds.ytop == plot_bounds.ybot)
 	    *y = *y2 = 1e38;	/* protection */
 	else {
 	    *y = AXIS_MAPBACK(FIRST_Y_AXIS, yy);
@@ -349,7 +362,7 @@ MousePosToGraphPosReal(int xx, int yy, double *x, double *y, double *x2, double
 	*x2 = *y2 = 1e38;	/* protection */
     }
     /*
-       Note: there is xleft+0.5 in "#define map_x" in graphics.c, which
+       Note: there is plot_bounds.xleft+0.5 in "#define map_x" in graphics.c, which
        makes no major impact here. It seems that the mistake of the real
        coordinate is at about 0.5%, which corresponds to the screen resolution.
        It would be better to round the distance to this resolution, and thus
@@ -366,7 +379,7 @@ MousePosToGraphPosReal(int xx, int yy, double *x, double *y, double *x2, double
 }
 
 static char *
-xy_format(void)
+xy_format()
 {
     static char format[0xff];
     format[0] = NUL;
@@ -377,7 +390,7 @@ xy_format(void)
 }
 
 static char *
-zoombox_format(void)
+zoombox_format()
 {
     static char format[0xff];
     format[0] = NUL;
@@ -492,8 +505,10 @@ xDateTimeFormat(double x, char *b, int mode)
  * value. Code is now closer to what setup_tics does. */
 #define MKSTR(sp,x,axis)					\
 do {								\
-    if (axis_array[axis].is_timedata) {				\
+    if (axis_array[axis].is_timedata) { 			\
 	char *format = copy_or_invent_formatstring(axis);	\
+	while (strchr(format,'\n'))				\
+	     *(strchr(format,'\n')) = ' ';			\
 	sp+=gstrftime(sp, 40, format, x);			\
     } else							\
 	sp+=sprintf(sp, mouse_setting.fmt ,x);			\
@@ -530,7 +545,7 @@ GetCoordinateString(char *s, double x, double y)
 
 /* formats the ruler information (position, distance,...) into string p
 	(it must be sufficiently long)
-   x, y is the current mouse position in real coords (for the calculation 
+   x, y is the current mouse position in real coords (for the calculation
 	of distance)
 */
 static void
@@ -551,20 +566,36 @@ GetRulerString(char *p, double x, double y)
     dy = DIST(y, ruler.y, FIRST_Y_AXIS);
     sprintf(p, format, ruler.x, ruler.y, dx, dy);
 
-    if (mouse_setting.polardistance && !axis_array[FIRST_X_AXIS].log && !axis_array[FIRST_Y_AXIS].log) {
-	/* polar coords of distance (axes cannot be logarithmic) */
-	double rho = sqrt((x - ruler.x) * (x - ruler.x) + (y - ruler.y) * (y - ruler.y));
-	double phi = (180 / M_PI) * atan2(y - ruler.y, x - ruler.x);
+    /* Previously, the following "if" let the polar coordinates to be shown only
+       for lin-lin plots:
+	    if (mouse_setting.polardistance && !axis_array[FIRST_X_AXIS].log && !axis_array[FIRST_Y_AXIS].log) ...
+       Now, let us support also semilog and log-log plots.
+       Values of mouse_setting.polardistance are:
+	    0 (no polar coordinates), 1 (polar coordinates), 2 (tangent instead of angle).
+    */
+    if (mouse_setting.polardistance) {
+	double rho, phi, rx, ry;
 	char ptmp[69];
-
+	x = AXIS_LOG_VALUE(FIRST_X_AXIS, x);
+	y = AXIS_LOG_VALUE(FIRST_Y_AXIS, y);
+	rx = AXIS_LOG_VALUE(FIRST_X_AXIS, ruler.x);
+	ry = AXIS_LOG_VALUE(FIRST_Y_AXIS, ruler.y);
 	format[0] = '\0';
 	strcat(format, " (");
 	strcat(format, mouse_setting.fmt);
+	rho = sqrt((x - rx) * (x - rx) + (y - ry) * (y - ry)); /* distance */
+	if (mouse_setting.polardistance == 1) { /* (distance, angle) */
+	    phi = (180 / M_PI) * atan2(y - ry, x - rx);
 # ifdef OS2
-	strcat(format, ";% #.4g)");
+	    strcat(format, ";% #.4g)");
 # else
-	strcat(format, ", % #.4gdeg)");
+	    strcat(format, ", % #.4gdeg)");
 # endif
+	} else { /* mouse_setting.polardistance==2: (distance, tangent) */
+	    phi = x - rx;
+	    phi = (phi == 0) ? ((y-ry>0) ? 1e308:-1e308) : (y - ry)/phi;
+	    sprintf(format+strlen(format), ", tangent=%s)", mouse_setting.fmt);
+	}
 	sprintf(ptmp, format, rho, phi);
 	strcat(p, ptmp);
     }
@@ -575,23 +606,34 @@ static struct t_zoom *zoom_head = NULL, *zoom_now = NULL;
 /* Applies the zoom rectangle of  z  by sending the appropriate command
    to gnuplot
 */
+
 static void
 apply_zoom(struct t_zoom *z)
 {
     char s[1024];		/* HBB 20011005: made larger */
-    /* HBB 20011004: new variable, fixing 'unzoom' back to autoscaled
-     * range */
+    /* HBB 20011004: new variable, fixing 'unzoom' back to autoscaled range */
     static t_autoscale autoscale_copies[AXIS_ARRAY_SIZE];
+    int is_splot_map = (is_3d_plot && (splot_map == TRUE));
+    int flip = 0;
 
     if (zoom_now != NULL) {	/* remember the current zoom */
-	zoom_now->xmin = AXIS_DE_LOG_VALUE(FIRST_X_AXIS, axis_array[FIRST_X_AXIS].set_min);
-	zoom_now->ymin = AXIS_DE_LOG_VALUE(FIRST_Y_AXIS, axis_array[FIRST_Y_AXIS].set_min);
-	zoom_now->x2min = AXIS_DE_LOG_VALUE(SECOND_X_AXIS, axis_array[SECOND_X_AXIS].set_min);
-	zoom_now->y2min = AXIS_DE_LOG_VALUE(SECOND_Y_AXIS, axis_array[SECOND_Y_AXIS].set_min);
-	zoom_now->xmax = AXIS_DE_LOG_VALUE(FIRST_X_AXIS, axis_array[FIRST_X_AXIS].set_max);
-	zoom_now->ymax = AXIS_DE_LOG_VALUE(FIRST_Y_AXIS, axis_array[FIRST_Y_AXIS].set_max);
-	zoom_now->x2max = AXIS_DE_LOG_VALUE(SECOND_X_AXIS, axis_array[SECOND_X_AXIS].set_max);
-	zoom_now->y2max = AXIS_DE_LOG_VALUE(SECOND_Y_AXIS, axis_array[SECOND_Y_AXIS].set_max);
+	zoom_now->xmin = axis_array[FIRST_X_AXIS].set_min;
+	zoom_now->xmax = axis_array[FIRST_X_AXIS].set_max;
+	zoom_now->x2min = axis_array[SECOND_X_AXIS].set_min;
+	zoom_now->x2max = axis_array[SECOND_X_AXIS].set_max;
+	zoom_now->was_splot_map = is_splot_map;
+	if (!is_splot_map) { /* 2D plot */
+	    zoom_now->ymin = axis_array[FIRST_Y_AXIS].set_min;
+	    zoom_now->ymax = axis_array[FIRST_Y_AXIS].set_max;
+	    zoom_now->y2min = axis_array[SECOND_Y_AXIS].set_min;
+	    zoom_now->y2max = axis_array[SECOND_Y_AXIS].set_max;
+	} else { /* the opposite, i.e. case 'set view map' */
+	    zoom_now->ymin = axis_array[FIRST_Y_AXIS].set_max;
+	    zoom_now->ymax = axis_array[FIRST_Y_AXIS].set_min;
+	    zoom_now->y2min = axis_array[SECOND_Y_AXIS].set_max;
+	    zoom_now->y2max = axis_array[SECOND_Y_AXIS].set_min;
+	}
+
     }
 
     /* HBB 20011004: implement save/restore of autoscaling if returning to
@@ -610,8 +652,23 @@ apply_zoom(struct t_zoom *z)
 	return;
     }
 
-    sprintf(s, "set xr[%.12g:%.12g]; set yr[%.12g:%.12g]", 
-	       zoom_now->xmin, zoom_now->xmax, zoom_now->ymin, zoom_now->ymax);
+    flip = (is_splot_map && zoom_now->was_splot_map);
+#ifdef HAVE_LOCALE_H
+    if (strcmp(localeconv()->decimal_point,".")) {
+	char *save_locale = gp_strdup(setlocale(LC_NUMERIC,NULL));
+	setlocale(LC_NUMERIC,"C");
+	sprintf(s, "set xr[%.12g:%.12g]; set yr[%.12g:%.12g]",
+	       zoom_now->xmin, zoom_now->xmax, 
+	       (flip) ? zoom_now->ymax : zoom_now->ymin,
+	       (flip) ? zoom_now->ymin : zoom_now->ymax);
+	setlocale(LC_NUMERIC,save_locale);
+	free(save_locale);
+    } else
+#endif
+	sprintf(s, "set xr[%.12g:%.12g]; set yr[%.12g:%.12g]",
+	       zoom_now->xmin, zoom_now->xmax, 
+	       (flip) ? zoom_now->ymax : zoom_now->ymin,
+	       (flip) ? zoom_now->ymin : zoom_now->ymax);
 
     /* HBB 20011004: the final part of 'unzoom to autoscale mode'.
      * Optionally appends 'set autoscale' commands to the string to be
@@ -637,7 +694,7 @@ apply_zoom(struct t_zoom *z)
 			: "");						     \
 	}								     \
     }
-	    
+
     if (zoom_now == zoom_head) {
 	/* new current zoom is the head --> returning to unzoomed
 	 * settings --> restore autoscale */
@@ -646,8 +703,20 @@ apply_zoom(struct t_zoom *z)
     }
 
     if (!is_3d_plot) {
-	sprintf(s + strlen(s), "; set x2r[% #g:% #g]; set y2r[% #g:% #g]",
-		zoom_now->x2min, zoom_now->x2max, zoom_now->y2min, zoom_now->y2max);
+#ifdef HAVE_LOCALE_H
+	if (strcmp(localeconv()->decimal_point,".")) {
+	    char *save_locale = gp_strdup(setlocale(LC_NUMERIC,NULL));
+	    setlocale(LC_NUMERIC,"C");
+	    sprintf(s + strlen(s), "; set x2r[% #g:% #g]; set y2r[% #g:% #g]",
+		zoom_now->x2min, zoom_now->x2max,
+		zoom_now->y2min, zoom_now->y2max);
+	    setlocale(LC_NUMERIC,save_locale);
+	    free(save_locale);
+	} else
+#endif
+	    sprintf(s + strlen(s), "; set x2r[% #g:% #g]; set y2r[% #g:% #g]",
+		zoom_now->x2min, zoom_now->x2max,
+		zoom_now->y2min, zoom_now->y2max);
 	if (zoom_now == zoom_head) {
 	    /* new current zoom is the head --> returning to unzoomed
 	     * settings --> restore autoscale */
@@ -690,12 +759,13 @@ do_zoom(double xmin, double ymin, double x2min, double y2min, double xmax, doubl
     z->ymax = ymax;
     z->x2max = x2max;
     z->y2max = y2max;
+    z->was_splot_map = (is_3d_plot && (splot_map == TRUE)); /* see is_splot_map in apply_zoom() */ 
     apply_zoom(z);
 }
 
 
 static void
-ZoomNext(void)
+ZoomNext()
 {
     if (zoom_now == NULL || zoom_now->next == NULL)
 	alert();
@@ -707,7 +777,7 @@ ZoomNext(void)
 }
 
 static void
-ZoomPrevious(void)
+ZoomPrevious()
 {
     if (zoom_now == NULL || zoom_now->prev == NULL)
 	alert();
@@ -719,7 +789,7 @@ ZoomPrevious(void)
 }
 
 static void
-ZoomUnzoom(void)
+ZoomUnzoom()
 {
     if (zoom_head == NULL || zoom_now == zoom_head)
 	alert();
@@ -776,7 +846,7 @@ incr_clipboardmode(const int amount)
 # define TICS_ON(ti) (((ti)&TICS_MASK)!=NO_TICS)
 
 void
-UpdateStatusline(void)
+UpdateStatusline()
 {
     UpdateStatuslineWithMouseSetting(&mouse_setting);
 }
@@ -857,10 +927,11 @@ UpdateStatuslineWithMouseSetting(mouse_setting_t * ms)
 	(term->put_tmptext) (0, s0);
     }
 }
+#undef MKSTR
 
 
 void
-recalc_statusline(void)
+recalc_statusline()
 {
     MousePosToGraphPosReal(mouse_x, mouse_y, &real_x, &real_y, &real_x2, &real_y2);
     UpdateStatusline();
@@ -914,7 +985,7 @@ builtin_toggle_grid(struct gp_event_t *ge)
     if (!ge) {
 	return "`builtin-toggle-grid`";
     }
-    if (! some_grid_selected()) 
+    if (! some_grid_selected())
 	do_string_replot("set grid");
     else
 	do_string_replot("unset grid");
@@ -936,30 +1007,30 @@ builtin_help(struct gp_event_t *ge)
 static char *
 builtin_toggle_log(struct gp_event_t *ge)
 {
-    if (!ge) {
-#ifdef PM3D
+    if (!ge)
 	return "`builtin-toggle-log` y logscale for plots, z and cb logscale for splots";
-#else
-	return "`builtin-toggle-log` y logscale for plots, z logscale for splots";
-#endif
-    }
     if (is_3d_plot) {
-#ifdef PM3D
 	if (Z_AXIS.log || CB_AXIS.log)
 	    do_string_replot("unset log zcb");
 	else
 	    do_string_replot("set log zcb");
-#else
-	if (Z_AXIS.log)
-	    do_string_replot("unset log z");
-	else
-	    do_string_replot("set log z");
-#endif
     } else {
+#ifdef WITH_IMAGE
+	/* set log cb or log y whether using "with (rgb)image" plot or not */
+	if (is_cb_plot) {
+	    if (CB_AXIS.log)
+		do_string_replot("unset log cb");
+	    else
+		do_string_replot("set log cb");
+	} else {
+#endif
 	if (axis_array[FIRST_Y_AXIS].log)
 	    do_string_replot("unset log y");
 	else
 	    do_string_replot("set log y");
+#ifdef WITH_IMAGE
+	}
+#endif
     }
     return (char *) 0;
 }
@@ -972,17 +1043,10 @@ builtin_nearest_log(struct gp_event_t *ge)
     }
     if (is_3d_plot) {
 	/* 3D-plot: toggle lin/log z axis */
-#ifdef PM3D
 	if (Z_AXIS.log || CB_AXIS.log)
 	    do_string_replot("unset log zcb");
 	else
 	    do_string_replot("set log zcb");
-#else
-	if (Z_AXIS.log)
-	    do_string_replot("unset log z");
-	else
-	    do_string_replot("set log z");
-#endif
     } else {
 	/* 2D-plot: figure out which axis/axes is/are
 	 * close to the mouse cursor, and toggle those lin/log */
@@ -990,19 +1054,19 @@ builtin_nearest_log(struct gp_event_t *ge)
 	 * the bottom, x2 at top, y left and y2 right; it
 	 * would be better to derive that from the ..tics settings */
 	TBOOLEAN change = FALSE;
-	if (mouse_y < ybot + (ytop - ybot) / 4 && mouse_x > xleft && mouse_x < xright) {
+	if (mouse_y < plot_bounds.ybot + (plot_bounds.ytop - plot_bounds.ybot) / 4 && mouse_x > plot_bounds.xleft && mouse_x < plot_bounds.xright) {
 	    do_string(axis_array[FIRST_X_AXIS].log ? "unset log x" : "set log x");
 	    change = TRUE;
 	}
-	if (mouse_y > ytop - (ytop - ybot) / 4 && mouse_x > xleft && mouse_x < xright) {
+	if (mouse_y > plot_bounds.ytop - (plot_bounds.ytop - plot_bounds.ybot) / 4 && mouse_x > plot_bounds.xleft && mouse_x < plot_bounds.xright) {
 	    do_string(axis_array[SECOND_X_AXIS].log ? "unset log x2" : "set log x2");
 	    change = TRUE;
 	}
-	if (mouse_x < xleft + (xright - xleft) / 4 && mouse_y > ybot && mouse_y < ytop) {
+	if (mouse_x < plot_bounds.xleft + (plot_bounds.xright - plot_bounds.xleft) / 4 && mouse_y > plot_bounds.ybot && mouse_y < plot_bounds.ytop) {
 	    do_string(axis_array[FIRST_Y_AXIS].log ? "unset log y" : "set log y");
 	    change = TRUE;
 	}
-	if (mouse_x > xright - (xright - xleft) / 4 && mouse_y > ybot && mouse_y < ytop) {
+	if (mouse_x > plot_bounds.xright - (plot_bounds.xright - plot_bounds.xleft) / 4 && mouse_y > plot_bounds.ybot && mouse_y < plot_bounds.ytop) {
 	    do_string(axis_array[SECOND_Y_AXIS].log ? "unset log y2" : "set log y2");
 	    change = TRUE;
 	}
@@ -1036,7 +1100,7 @@ builtin_toggle_mouse(struct gp_event_t *ge)
 # endif
     }
 # ifdef OS2
-    update_menu_items_PM_terminal();
+    PM_update_menu_items();
 # endif
     UpdateStatusline();
     return (char *) 0;
@@ -1052,17 +1116,25 @@ builtin_toggle_ruler(struct gp_event_t *ge)
 	return (char *) 0;
     if (ruler.on) {
 	turn_ruler_off();
-	if (display_ipc_commands()) {
+	if (display_ipc_commands())
 	    fprintf(stderr, "turning ruler off.\n");
-	}
     } else if (ALMOST2D) {
 	/* only allow ruler, if the plot
 	 * is 2d or a 3d `map' */
+	struct udvt_entry *u;
 	ruler.on = TRUE;
 	ruler.px = ge->mx;
 	ruler.py = ge->my;
 	MousePosToGraphPosReal(ruler.px, ruler.py, &ruler.x, &ruler.y, &ruler.x2, &ruler.y2);
 	(*term->set_ruler) (ruler.px, ruler.py);
+	if ((u = add_udv_by_name("MOUSE_RULER_X"))) {
+	    u->udv_undef = FALSE;
+	    Gcomplex(&u->udv_value,ruler.x,0);
+	}
+	if ((u = add_udv_by_name("MOUSE_RULER_Y"))) {
+	    u->udv_undef = FALSE;
+	    Gcomplex(&u->udv_value,ruler.y,0);
+	}
 	if (display_ipc_commands()) {
 	    fprintf(stderr, "turning ruler on.\n");
 	}
@@ -1117,9 +1189,11 @@ builtin_toggle_polardistance(struct gp_event_t *ge)
     if (!ge) {
 	return "`builtin-toggle-polardistance`";
     }
-    mouse_setting.polardistance = !mouse_setting.polardistance;
+    if (++mouse_setting.polardistance > 2) mouse_setting.polardistance = 0;
+	/* values: 0 (no polar coordinates), 1 (polar coordinates), 2 (tangent instead of angle) */
+    term->set_cursor((mouse_setting.polardistance ? -3:-4), ge->mx, ge->my); /* change cursor type */
 # ifdef OS2
-    update_menu_items_PM_terminal();
+    PM_update_menu_items();
 # endif
     UpdateStatusline();
     if (display_ipc_commands()) {
@@ -1253,7 +1327,7 @@ builtin_cancel_zoom(struct gp_event_t *ge)
 }
 
 static void
-event_keypress(struct gp_event_t *ge)
+event_keypress(struct gp_event_t *ge, TBOOLEAN current)
 {
     int x, y;
     int c, par2;
@@ -1277,12 +1351,53 @@ event_keypress(struct gp_event_t *ge)
     keypress.key = c;
     keypress.modifier = modifier_mask;
 
+    /*
+     * On 'pause mouse keypress' in active window export current keypress 
+     * and mouse coords to user variables. A key with 'bind all' terminates 
+     * a pause even from non-active windows.
+     * Ignore NULL keypress.
+     *
+     * If we are paused for a keystroke, this takes precendence over normal
+     * key bindings. Otherwise, for example typing 'm' would turn off mousing,
+     * which is a bad thing if you are in the  middle of a mousing operation.
+     */
+
+#ifdef _Windows
+    if (paused_for_mouse & PAUSE_KEYSTROKE)
+	kill_pending_Pause_dialog();
+#endif
+    
+    if ((paused_for_mouse & PAUSE_KEYSTROKE) && (c > '\0') && current) {
+	load_mouse_variables(x, y, FALSE, c);
+	return;
+    }
+
     for (ptr = bindings; ptr; ptr = ptr->next) {
 	if (bind_matches(&keypress, ptr)) {
-	    if ((par2 & 1) == 0 && ptr->command) {
-		/* let user defined bindings overwrite the builtin bindings */
+	    struct udvt_entry *keywin;
+	    if ((keywin = add_udv_by_name("MOUSE_KEY_WINDOW"))) {
+		keywin->udv_undef = FALSE;
+		Ginteger(&keywin->udv_value, ge->winid);
+	    }
+	    /* Always honor keys set with "bind all" */
+	    if (ptr->allwindows && ptr->command) {
+		if (current)
+		    load_mouse_variables(x, y, FALSE, c);
+		else
+		    /* FIXME - Better to clear MOUSE_[XY] than to set it wrongly. */
+		    /*         This may be worth a separate subroutine.           */
+		    load_mouse_variables(0, 0, FALSE, c);
 		do_string(ptr->command);
-		return;
+		/* Treat as a current event after we return to x11.trm */
+		ge->type = GE_keypress;
+		break;
+	    /* But otherwise ignore inactive windows */
+	    } else if (!current) {
+		break;
+	    /* Let user defined bindings overwrite the builtin bindings */
+	    } else if ((par2 & 1) == 0 && ptr->command) {
+		do_string(ptr->command);
+		break;
 	    } else if (ptr->builtin) {
 		ptr->builtin(ge);
 	    } else {
@@ -1290,6 +1405,7 @@ event_keypress(struct gp_event_t *ge)
 	    }
 	}
     }
+
 }
 
 
@@ -1344,8 +1460,7 @@ event_buttonpress(struct gp_event_t *ge)
 
     button |= (1 << b);
 
-    FPRINTF((stderr, "(event_buttonpress) mouse_x = %d\n", mouse_x));
-    FPRINTF((stderr, "(event_buttonpress) mouse_y = %d\n", mouse_y));
+    FPRINTF((stderr, "(event_buttonpress) mouse_x = %d\tmouse_y = %d\n", mouse_x, mouse_y));
 
     MousePosToGraphPosReal(mouse_x, mouse_y, &real_x, &real_y, &real_x2, &real_y2);
 
@@ -1355,8 +1470,11 @@ event_buttonpress(struct gp_event_t *ge)
 		/* not bound in 2d graphs */
 	    } else if (2 == b) {
 		/* not bound in 2d graphs */
-	    } else if (3 == b) {
-		/* start zoom */
+	    } else if (3 == b && !replot_disabled && !(paused_for_mouse & PAUSE_CLICK)) {
+		/* start zoom; but ignore it when
+		 *   - replot is disabled, e.g. with inline data, or
+		 *   - during 'pause mouse'
+		 * allow zooming during 'pause mouse key' */
 		setting_zoom_x = mouse_x;
 		setting_zoom_y = mouse_y;
 		setting_zoom_region = TRUE;
@@ -1550,54 +1668,17 @@ event_buttonrelease(struct gp_event_t *ge)
 	    term->set_cursor(0, 0, 0);
     }
 
-    MousePosToGraphPosReal(mouse_x, mouse_y, &real_x, &real_y, &real_x2, &real_y2);
+    /* Export current mouse coords to user-accessible variables also */
+    load_mouse_variables(mouse_x, mouse_y, TRUE, b);
     UpdateStatusline();
 
-    /* Export current mouse coords to user-accessible variables also
-     * FIXME EAM: Maybe don't export if axis ticmode == NO_TICS     */
-    {
-	struct udvt_entry *current;
-
-	if ((current = get_udv("MOUSE_X"))) {
-	    current->udv_undef = FALSE;
-	    Gcomplex(&current->udv_value,real_x,0);
-	}
-	if ((current = get_udv("MOUSE_Y"))) {
-	    current->udv_undef = FALSE;
-	    Gcomplex(&current->udv_value,real_y,0);
-	}
-	if ((current = get_udv("MOUSE_X2"))) {
-	    current->udv_undef = FALSE;
-	    Gcomplex(&current->udv_value,real_x2,0);
-	}
-	if ((current = get_udv("MOUSE_Y2"))) {
-	    current->udv_undef = FALSE;
-	    Gcomplex(&current->udv_value,real_y2,0);
-	}
-	if ((current = get_udv("MOUSE_BUTTON"))) {
-	    current->udv_undef = FALSE;
-	    Ginteger(&current->udv_value,b);
-	}
-	if ((current = get_udv("MOUSE_SHIFT"))) {
-	    current->udv_undef = FALSE;
-	    Ginteger(&current->udv_value, modifier_mask & Mod_Shift);
-	}
-	if ((current = get_udv("MOUSE_ALT"))) {
-	    current->udv_undef = FALSE;
-	    Ginteger(&current->udv_value, modifier_mask & Mod_Alt);
-	}
-	if ((current = get_udv("MOUSE_CTRL"))) {
-	    current->udv_undef = FALSE;
-	    Ginteger(&current->udv_value, modifier_mask & Mod_Ctrl);
-	}
-    }
-    /* Terminate via replot if we are in 'pause mouse' */
-    /* FIXME EAM: untested for mouse devices other than X11 */
-    if (paused_for_mouse) {
-	paused_for_mouse = FALSE;
-	do_string("replot");
-	fputc('\n', stderr);
+#ifdef _Windows
+    if (paused_for_mouse & PAUSE_CLICK) {
+	/* remove pause message box after 'pause mouse' */
+	paused_for_mouse = 0;
+	kill_pending_Pause_dialog();
     }
+#endif
 }
 
 static void
@@ -1648,7 +1729,7 @@ event_motion(struct gp_event_t *ge)
 		return;
 # endif
 	    if (modifier_mask & Mod_Shift) {
-		ticslevel += (1 + fabs(ticslevel))
+		xyplane.ticslevel += (1 + fabs(xyplane.ticslevel))
 		    * (mouse_y - start_y) * 2.0 / term->ymax;
 	    } else {
 
@@ -1675,7 +1756,7 @@ event_motion(struct gp_event_t *ge)
 	if (redraw) {
 	    if (allowmotion) {
 		/* is processing of motions allowed right now?
-		 * then replot while 
+		 * then replot while
 		 * disabling further replots until it completes */
 		allowmotion = FALSE;
 		do_save_3dplot(first_3dplot, plot3d_num, !!(modifier_mask & Mod_Ctrl));
@@ -1730,13 +1811,6 @@ event_plotdone()
 }
 
 
-static void
-event_print(FILE * fp, char *s)
-{
-    fputs(s, fp);
-    fflush(fp);
-}
-
 void
 event_reset(struct gp_event_t *ge)
 {
@@ -1750,15 +1824,29 @@ event_reset(struct gp_event_t *ge)
 	    term->put_tmptext(2, "");
 	}
     }
+
+    if (paused_for_mouse) {
+	paused_for_mouse = 0;
+#ifdef _Windows
+	/* remove pause message box after 'pause mouse' */
+	kill_pending_Pause_dialog();
+#endif
+	/* This hack is necessary for X11 in order to prevent one character */
+	/* of input from being swallowed when the plot window is closed.    */
+	if (term && !strncmp("x11",term->name,3))
+	    ungetc('\n',stdin);
+    }
 }
 
 void
 do_event(struct gp_event_t *ge)
 {
-    if (multiplot || !term) {
-	/* no event processing for multiplot or undefined terminal */
+    if (!term)
+	return;
+
+    if (multiplot && ge->type != GE_fontprops)
+	/* only informational event processing for multiplot */
 	return;
-    }
 
     /* disable `replot` when some data were sent through stdin */
     replot_disabled = plotted_data_from_stdin;
@@ -1767,21 +1855,17 @@ do_event(struct gp_event_t *ge)
 	FPRINTF((stderr, "(do_event) type       = %d\n", ge->type));
 	FPRINTF((stderr, "           mx, my     = %d, %d\n", ge->mx, ge->my));
 	FPRINTF((stderr, "           par1, par2 = %d, %d\n", ge->par1, ge->par2));
-	FPRINTF((stderr, "           text       = %s\n", ge->text));
     }
 
     switch (ge->type) {
-    case GE_stdout:
-	event_print(stdout, ge->text);
-	break;
-    case GE_stderr:
-	event_print(stderr, ge->text);
-	break;
     case GE_plotdone:
 	event_plotdone();
 	break;
     case GE_keypress:
-	event_keypress(ge);
+	event_keypress(ge, TRUE);
+	break;
+    case GE_keypress_old:
+	event_keypress(ge, FALSE);
 	break;
     case GE_modifier:
 	event_modifier(ge);
@@ -1801,8 +1885,9 @@ do_event(struct gp_event_t *ge)
 	    break;
 	event_buttonrelease(ge);
 	break;
-    case GE_cmd:
-	do_string(ge->text);
+    case GE_replot:
+	/* used only by ggi.trm */
+	do_string("replot");
 	break;
     case GE_reset:
 	event_reset(ge);
@@ -1839,9 +1924,7 @@ do_save_3dplot(struct surface_points *plots, int pcount, int quick)
 	replotrequest();
     } else {
 	if (M_TEST_AXIS(X_AXIS) || M_TEST_AXIS(Y_AXIS) || M_TEST_AXIS(Z_AXIS)
-#ifdef PM3D
 	    || M_TEST_AXIS(CB_AXIS)
-#endif
 	    ) {
 		graph_error("axis ranges must be above 0 for log scale!");
 		return;
@@ -1858,7 +1941,7 @@ do_save_3dplot(struct surface_points *plots, int pcount, int quick)
  */
 
 static void
-bind_install_default_bindings(void)
+bind_install_default_bindings()
 {
     bind_remove_all();
     bind_append("a", (char *) 0, builtin_autoscale);
@@ -1905,6 +1988,13 @@ static int
 lookup_key(char *ptr, int *len)
 {
     char **keyptr;
+    /* first, search in the table of "usual well-known" keys */
+    int what = lookup_table_nth(usual_special_keys, ptr);
+    if (what >= 0) {
+	*len = strlen(usual_special_keys[what].key);
+	return usual_special_keys[what].value;
+    }
+    /* second, search in the table of other keys */
     for (keyptr = special_keys; *keyptr; ++keyptr) {
 	if (!strncasecmp(ptr, *keyptr, (*len = strlen(*keyptr)))) {
 	    return keyptr - special_keys + GP_FIRST_KEY;
@@ -1966,6 +2056,15 @@ bind_fmt_lhs(const bind_t * in)
     if (in->key > GP_FIRST_KEY && in->key < GP_LAST_KEY) {
 	sprintf(out, "%s%s", out, special_keys[in->key - GP_FIRST_KEY]);
     } else {
+	int k = 0;
+	for ( ; usual_special_keys[k].value > 0; k++) {
+	    if (usual_special_keys[k].value == in->key) {
+		sprintf(out, "%s%s", out, usual_special_keys[k].key);
+		k = -1;
+		break;
+	    }
+	}
+	if (k >= 0)
 	sprintf(out, "%s%c", out, in->key);
     }
     return out;
@@ -1987,7 +2086,8 @@ bind_matches(const bind_t * a, const bind_t * b)
 static void
 bind_display_one(bind_t * ptr)
 {
-    fprintf(stderr, " %-12s  ", bind_fmt_lhs(ptr));
+    fprintf(stderr, " %-12s ", bind_fmt_lhs(ptr));
+    fprintf(stderr, "%c ", ptr->allwindows ? '*' : ' ');
     if (ptr->command) {
 	fprintf(stderr, "`%s'\n", ptr->command);
     } else if (ptr->builtin) {
@@ -2019,15 +2119,17 @@ bind_display(char *lhs)
 	fprintf(stderr, fmt, "<B3>", "mark zoom region (only for 2d-plots and maps).");
 	fprintf(stderr, fmt, "<B1-Motion>", "change view (rotation). Use <ctrl> to rotate the axes only.");
 	fprintf(stderr, fmt, "<B2-Motion>", "change view (scaling). Use <ctrl> to scale the axes only.");
-	fprintf(stderr, fmt, "<Shift-B2-Motion>", "vertical motion -- change ticslevel");
+	fprintf(stderr, fmt, "<Shift-B2-Motion>", "vertical motion -- change xyplane");
 	fprintf(stderr, "\n");
-	fprintf(stderr, " %-12s  %s\n", "Space", "raise gnuplot console window");
-	fprintf(stderr, " %-12s  %s\n", "q", "quit X11 terminal");
+	fprintf(stderr, " %-12s   %s\n", "Space", "raise gnuplot console window");
+	fprintf(stderr, " %-12s * %s\n", "q", "close this X11 plot window");
 	fprintf(stderr, "\n");
 	for (ptr = bindings; ptr; ptr = ptr->next) {
 	    bind_display_one(ptr);
 	}
 	fprintf(stderr, "\n");
+	fprintf(stderr, "              * indicates this key is active from all plot windows\n");
+	fprintf(stderr, "\n");
 	return;
     }
 
@@ -2116,6 +2218,7 @@ bind_append(char *lhs, char *rhs, char *(*builtin) (struct gp_event_t * ge))
 
     bindings->prev = new;
     new->next = (struct bind_t *) 0;
+    new->allwindows = FALSE;	/* Can be explicitly set later */
     if (!rhs) {
 	new->builtin = builtin;
     } else if (*rhs) {
@@ -2126,7 +2229,7 @@ bind_append(char *lhs, char *rhs, char *(*builtin) (struct gp_event_t * ge))
 }
 
 void
-bind_process(char *lhs, char *rhs)
+bind_process(char *lhs, char *rhs, TBOOLEAN allwindows)
 {
     if (!bindings) {
 	bind_install_default_bindings();
@@ -2135,13 +2238,30 @@ bind_process(char *lhs, char *rhs)
 	bind_display(lhs);
     } else {
 	bind_append(lhs, rhs, 0);
+	if (allwindows)
+	    bind_all(lhs);
     }
     if (lhs)
 	free(lhs);
 }
 
 void
-bind_remove_all(void)
+bind_all(char *lhs)
+{
+    bind_t *ptr;
+    bind_t keypress;
+
+    if (!bind_scan_lhs(&keypress, lhs))
+	return;
+
+    for (ptr = bindings; ptr; ptr = ptr->next) {
+	if (bind_matches(&keypress, ptr))
+	    ptr->allwindows = TRUE;
+    }
+}
+
+void
+bind_remove_all()
 {
     bind_t *ptr;
     bind_t *safe;
@@ -2154,11 +2274,11 @@ bind_remove_all(void)
     bindings = (bind_t *) 0;
 }
 
-/* Ruler is on, thus recalc its (px,py) from (x,y) for the current zoom and 
+/* Ruler is on, thus recalc its (px,py) from (x,y) for the current zoom and
    log axes.
 */
 static void
-recalc_ruler_pos(void)
+recalc_ruler_pos()
 {
     double P, dummy;
     if (is_3d_plot) {
@@ -2190,7 +2310,7 @@ recalc_ruler_pos(void)
 /* Recalculate and replot the ruler after a '(re)plot'. Called from term.c.
 */
 void
-update_ruler(void)
+update_ruler()
 {
     if (!term->set_ruler || !ruler.on)
 	return;
@@ -2199,6 +2319,32 @@ update_ruler(void)
     (*term->set_ruler) (ruler.px, ruler.py);
 }
 
+/* Set ruler on/off, and set its position.
+   Called from set.c for 'set mouse ruler ...' command.
+*/
+void
+set_ruler(TBOOLEAN on, int mx, int my)
+{
+    struct gp_event_t ge;
+    if (ruler.on == FALSE && on == FALSE)
+	return;
+    if (ruler.on == TRUE && on == TRUE && (mx < 0 || my < 0))
+	return;
+    if (ruler.on == TRUE) /* ruler is on => switch it off */
+	builtin_toggle_ruler(&ge);
+    /* now the ruler is off */
+    if (on == FALSE) /* want ruler off */
+	return;
+    if (mx>=0 && my>=0) { /* change ruler position */
+	ge.mx = mx;
+	ge.my = my;
+    } else { /* don't change ruler position */
+	ge.mx = ruler.px;
+	ge.my = ruler.py;
+    }
+    builtin_toggle_ruler(&ge);
+}
+
 /* for checking if we change from plot to splot (or vice versa) */
 int
 plot_mode(int set)
@@ -2214,31 +2360,45 @@ plot_mode(int set)
 }
 
 static void
-turn_ruler_off(void)
+turn_ruler_off()
 {
     if (ruler.on) {
+	struct udvt_entry *u;
 	ruler.on = FALSE;
 	if (term && term->set_ruler) {
 	    (*term->set_ruler) (-1, -1);
 	}
+	if ((u = add_udv_by_name("MOUSE_RULER_X")))
+	    u->udv_undef = TRUE;
+	if ((u = add_udv_by_name("MOUSE_RULER_Y")))
+	    u->udv_undef = TRUE;
+	if (display_ipc_commands()) {
+	    fprintf(stderr, "turning ruler off.\n");
+	}
     }
 }
 
 static int
-nearest_label_tag(int xref, int yref, struct termentry *t, map_func_type * map_func)
+nearest_label_tag(int xref, int yref, struct termentry *t)
 {
     double min = -1;
     int min_tag = -1;
     double diff_squared;
-    unsigned int x, y;
+    int x, y;
     struct text_label *this_label;
     int xd;
     int yd;
 
     for (this_label = first_label; this_label != NULL; this_label = this_label->next) {
-	map_func(&this_label->place, &x, &y, "label");
-	xd = (int) x - (int) xref;
-	yd = (int) y - (int) yref;
+	if (is_3d_plot) {
+	    map3d_position(&this_label->place, &xd, &yd, "label");
+	    xd -= xref;
+	    yd -= yref;
+	} else {
+	    map_position(&this_label->place, &x, &y, "label");
+	    xd = x - xref;
+	    yd = y - yref;
+	}
 	diff_squared = xd * xd + yd * yd;
 	if (-1 == min || min > diff_squared) {
 	    /* now we check if we're within a certain
@@ -2260,8 +2420,7 @@ nearest_label_tag(int xref, int yref, struct termentry *t, map_func_type * map_f
 static void
 remove_label(int x, int y)
 {
-    int tag = nearest_label_tag(x, y, term,
-				is_3d_plot ? map3d_position : map_position);
+    int tag = nearest_label_tag(x, y, term);
     if (-1 != tag) {
 	char cmd[0x40];
 	sprintf(cmd, "unset label %d", tag);
@@ -2277,38 +2436,87 @@ put_label(char *label, double x, double y)
     do_string_replot(cmd);
 }
 
-# ifdef OS2
-/* routine required by pm.trm: fill & send information needed for (un)checking
+#ifdef OS2
+/* routine required by pm.trm: fill in information needed for (un)checking
    menu items in the Presentation Manager terminal
 */
-void
-send_gpPMmenu(FILE * PM_pipe)
+void 
+PM_set_gpPMmenu __PROTO((struct t_gpPMmenu * gpPMmenu))
 {
-    struct t_gpPMmenu gpPMmenu;
-    /* not connected to mouseable gnupmdrv */
-    if (!PM_pipe || !mouseGnupmdrv)
-	return;
-    gpPMmenu.use_mouse = mouse_setting.on;
+    gpPMmenu->use_mouse = mouse_setting.on;
     if (zoom_now == NULL)
-	gpPMmenu.where_zoom_queue = 0;
+	gpPMmenu->where_zoom_queue = 0;
     else {
-	gpPMmenu.where_zoom_queue = (zoom_now == zoom_head) ? 0 : 1;
+	gpPMmenu->where_zoom_queue = (zoom_now == zoom_head) ? 0 : 1;
 	if (zoom_now->prev != NULL)
-	    gpPMmenu.where_zoom_queue |= 2;
+	    gpPMmenu->where_zoom_queue |= 2;
 	if (zoom_now->next != NULL)
-	    gpPMmenu.where_zoom_queue |= 4;
+	    gpPMmenu->where_zoom_queue |= 4;
     }
-    gpPMmenu.polar_distance = mouse_setting.polardistance;
-    putc('#', PM_pipe);
-    fwrite(&gpPMmenu, sizeof(gpPMmenu), 1, PM_pipe);
+    gpPMmenu->polar_distance = mouse_setting.polardistance;
 }
+#endif
 
-/* update menu items in PM terminal */
-void
-update_menu_items_PM_terminal(void)
+/* Save current mouse position to user-accessible variables.
+ * Save the keypress or mouse button that triggered this in MOUSE_KEY,
+ * and define MOUSE_BUTTON if it was a button click.
+ */
+static void
+load_mouse_variables(double x, double y, TBOOLEAN button, int c)
 {
-    send_gpPMmenu(PM_pipe);
+    struct udvt_entry *current;
+
+    MousePosToGraphPosReal(x, y, &real_x, &real_y, &real_x2, &real_y2);
+
+    if ((current = add_udv_by_name("MOUSE_BUTTON"))) {
+	current->udv_undef = !button;
+	Ginteger(&current->udv_value, button?c:-1);
+    }
+    if ((current = add_udv_by_name("MOUSE_KEY"))) {
+	current->udv_undef = FALSE;
+	Ginteger(&current->udv_value,c);
+    }
+#ifdef GP_STRING_VARS
+    if ((current = add_udv_by_name("MOUSE_CHAR"))) {
+	char *keychar = gp_alloc(2,"key_char");
+	keychar[0] = c;
+	keychar[1] = '\0';
+	if (!current->udv_undef)
+	    gpfree_string(&current->udv_value);
+	current->udv_undef = FALSE;
+	Gstring(&current->udv_value,keychar);
+    }
+#endif
+
+    if ((current = add_udv_by_name("MOUSE_X"))) {
+	current->udv_undef = FALSE;
+	Gcomplex(&current->udv_value,real_x,0);
+    }
+    if ((current = add_udv_by_name("MOUSE_Y"))) {
+	current->udv_undef = FALSE;
+	Gcomplex(&current->udv_value,real_y,0);
+    }
+    if ((current = add_udv_by_name("MOUSE_X2"))) {
+	current->udv_undef = FALSE;
+	Gcomplex(&current->udv_value,real_x2,0);
+    }
+    if ((current = add_udv_by_name("MOUSE_Y2"))) {
+	current->udv_undef = FALSE;
+	Gcomplex(&current->udv_value,real_y2,0);
+    }
+    if ((current = add_udv_by_name("MOUSE_SHIFT"))) {
+	current->udv_undef = FALSE;
+	Ginteger(&current->udv_value, modifier_mask & Mod_Shift);
+    }
+    if ((current = add_udv_by_name("MOUSE_ALT"))) {
+	current->udv_undef = FALSE;
+	Ginteger(&current->udv_value, modifier_mask & Mod_Alt);
+    }
+    if ((current = add_udv_by_name("MOUSE_CTRL"))) {
+	current->udv_undef = FALSE;
+	Ginteger(&current->udv_value, modifier_mask & Mod_Ctrl);
+    }
+    return;
 }
-# endif
 
 #endif /* USE_MOUSE */
