@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: parse.c,v 1.22 2004/04/13 17:23:58 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: parse.c,v 1.47 2006/07/08 16:59:39 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - parse.c */
@@ -42,6 +42,10 @@ static char *RCSid() { return RCSid("$Id: parse.c,v 1.22 2004/04/13 17:23:58 bro
 #include "help.h"
 #include "util.h"
 
+/* protection mechanism for parsing string followed by + or - sign */
+static int parse_recursion_level;
+static TBOOLEAN string_result_only = FALSE;
+
 /* Exported globals: the current 'dummy' variable names */
 char c_dummy_var[MAX_NUM_VAR][MAX_ID_LEN+1];
 char set_dummy_var[MAX_NUM_VAR][MAX_ID_LEN+1] = { "x", "y" };
@@ -80,25 +84,31 @@ static int is_builtin_function __PROTO((int t_num));
 static struct at_type *at = NULL;
 static int at_size = 0;
 
-#ifdef GP_ISVAR
-/* isvar - When this variable is true PUSH operations become PUSHV */
-static TBOOLEAN push_vars = TRUE; 
-#endif
-
 static void
-convert(val_ptr, t_num)
-    struct value *val_ptr;
-    int t_num;
+convert(struct value *val_ptr, int t_num)
 {
     *val_ptr = token[t_num].l_val;
 }
 
 
+/* JW 20051126:
+ * Wrapper around const_express() called by try_to_get_string().
+ * Disallows top level + and - operators.
+ * This enables things like set xtics ('-\pi' -pi, '-\pi/2' -pi/2.)
+ */
 struct value *
-const_express(valptr)
-struct value *valptr;
+const_string_express(struct value *valptr)
 {
-    register int tkn = c_token;
+    string_result_only = TRUE;
+    const_express(valptr);
+    string_result_only = FALSE;
+    return (valptr);
+}
+
+struct value *
+const_express(struct value *valptr)
+{
+    int tkn = c_token;
 
     if (END_OF_COMMAND)
 	int_error(c_token, "constant expression required");
@@ -107,30 +117,92 @@ struct value *valptr;
     dummy_func = NULL;
 
     evaluate_at(temp_at(), valptr);	/* run it and send answer back */
+
     if (undefined) {
 	int_error(tkn, "undefined value");
     }
     return (valptr);
 }
 
-
-/* if dummy_dunc == NULL on entry, do not attempt to compile dummy variables
- * - div
+/* Used by plot2d/plot3d/fit:
+ * Parse an expression that may return a string or may return a constant or may
+ * be a dummy function using dummy variables x, y, ...
+ * If any dummy variables are present, set (*atptr) to point to an action table
+ * corresponding to the parsed expression, and return NULL.
+ * Otherwise evaluate the expression and return a string if there is one.
+ * The return value "str" and "*atptr" both point to locally-managed memory,
+ * which must not be freed by the caller!
  */
+char*
+string_or_express(struct at_type **atptr)
+{
+    int i;
+    TBOOLEAN has_dummies;
+
+    static char* str = NULL;
+    free(str);
+    str = NULL;
+
+    if (END_OF_COMMAND)
+	int_error(c_token, "expression expected");
+
+#ifndef GP_STRING_VARS
+    if (isstring(c_token)) {
+	if (atptr)
+	    *atptr = NULL;
+	str = try_to_get_string();
+	return str;
+    }
+#endif
+
+    /* parse expression */
+    temp_at();
+
+    /* check if any dummy variables are used */
+    has_dummies = FALSE;
+    for (i = 0; i < at->a_count; i++) {
+	enum operators op_index = at->actions[i].index;
+	if ( op_index == PUSHD1 || op_index == PUSHD2 || op_index == PUSHD ) {
+	    has_dummies = TRUE;
+	    break;
+	}
+    }
+
+    if (!has_dummies) {
+	/* no dummy variables: evaluate expression */
+	struct value val;
+
+	evaluate_at(at, &val);
+#ifdef GP_STRING_VARS
+	if (!undefined && val.type == STRING)
+	    str = val.v.string_val;
+#endif
+    }
+
+    /* prepare return */
+    if (atptr)
+	*atptr  = at;
+    return str;
+}
+
+
+/* build an action table and return its pointer, but keep a pointer in at
+ * so that we can free it later if the caller hasn't taken over management
+ * of this table.
+ */
+
 struct at_type *
 temp_at()
 {
-    /* build a static action table and return its
-     * pointer */
-
-    if (at != NULL) {
-	free(at);
-	at = NULL;
-    }
+    if (at != NULL)
+	free_at(at);
+    
     at = (struct at_type *) gp_alloc(sizeof(struct at_type), "action table");
 
-    at->a_count = 0;		/* reset action table !!! */
+    memset(at, 0, sizeof(*at));		/* reset action table !!! */
     at_size = MAX_AT_LEN;
+
+    parse_recursion_level = 0;
     parse_expression();
     return (at);
 }
@@ -141,7 +213,7 @@ temp_at()
 struct at_type *
 perm_at()
 {
-    register struct at_type *at_ptr;
+    struct at_type *at_ptr;
     size_t len;
 
     (void) temp_at();
@@ -162,10 +234,9 @@ extend_at()
     FPRINTF((stderr, "Extending at size to %d\n", at_size));
 }
 
-/* moved from eval.c, the function is only called from this module */
+/* Add function number <sf_index> to the current action table */
 static union argument *
-add_action(sf_index)
-enum operators sf_index;	/* index of p-code function */
+add_action(enum operators sf_index)
 {
     if (at->a_count >= at_size) {
 	extend_at();
@@ -175,11 +246,15 @@ enum operators sf_index;	/* index of p-code function */
 }
 
 
+/* For external calls to parse_expressions() 
+ * parse_recursion_level is expected to be 0 */
 static void
 parse_expression()
 {				/* full expressions */
+    parse_recursion_level++;
     accept_logical_OR_expression();
     parse_conditional_expression();
+    parse_recursion_level--;
 }
 
 static void
@@ -289,35 +364,48 @@ parse_primary_expression()
 	 * argument list */
 	if ((c_token + 1 < num_tokens) && equals(c_token + 1, "(")) {
 	    enum operators whichfunc = is_builtin_function(c_token);
+	    struct value num_params;
+	    num_params.type = INTGR;
 
 	    if (whichfunc) {
-
-#ifdef GP_ISVAR
-		/* Check to see if it is isvar */
-		/* Is so then turn off normal variable pushing */
-                /* Push variable definition state instead */
+#ifdef BACKWARDS_COMPATIBLE
+		/* Deprecated syntax:   if (defined(foo)) ...  */
+		/* New syntax:          if (exists("foo")) ... */
 		if (strcmp(ft[whichfunc].f_name,"defined")==0) {
-			push_vars=FALSE;
+		    struct udvt_entry *udv = add_udv(c_token+2);
+		    union argument *foo = add_action(PUSHC);
+		    foo->v_arg.type = INTGR;
+		    foo->v_arg.v.int_val = udv->udv_undef ? 0 : 1;
+		    c_token += 4;  /* skip past "defined ( <foo> ) " */
+		    return;
 		}
-#endif  /*GP_ISVAR*/
-
-		/* it's a standard function */
+#endif
 		c_token += 2;	/* skip fnc name and '(' */
 		parse_expression(); /* parse fnc argument */
+		num_params.v.int_val = 1;
 		while (equals(c_token, ",")) {
-		    c_token += 1;
+		    c_token++;
+		    num_params.v.int_val++;
 		    parse_expression();
 		}
 
 		if (!equals(c_token, ")"))
 		    int_error(c_token, "')' expected");
 		c_token++;
+
+#ifdef GP_STRING_VARS
+		/* So far sprintf is the only built-in function */
+		/* with a variable number of arguments.         */
+		if (!strcmp(ft[whichfunc].f_name,"sprintf"))
+		    add_action(PUSHC)->v_arg = num_params;
+		/* And "words(s)" is implemented as "word(s,-1)" */
+		if (!strcmp(ft[whichfunc].f_name,"words")) {
+		    num_params.v.int_val = -1;
+		    add_action(PUSHC)->v_arg = num_params;
+		}
+#endif
 		(void) add_action(whichfunc);
 
-#ifdef GP_ISVAR
-		 /* Turn normal variable pushing back on */
-		push_vars=TRUE;
-#endif  /*GP_ISVAR*/
 	    } else {
 		/* it's a call to a user-defined function */
 		enum operators call_type = (int) CALL;
@@ -326,9 +414,6 @@ parse_primary_expression()
 		c_token += 2;	/* skip func name and '(' */
 		parse_expression();
 		if (equals(c_token, ",")) { /* more than 1 argument? */
-		    struct value num_params;
-
-		    num_params.type = INTGR;
 		    num_params.v.int_val = 1;
 		    while (equals(c_token, ",")) {
 			num_params.v.int_val += 1;
@@ -373,17 +458,24 @@ parse_primary_expression()
 	    }
 	    /* its a variable, with no dummies active - div */
 	} else {
-#ifdef GP_ISVAR
-            if (push_vars==FALSE)
-		add_action(PUSHV)->udv_arg = add_udv(c_token);
-	    else
-#endif  /* GP_ISVAR */
-		add_action(PUSH)->udv_arg = add_udv(c_token);
-
+	    add_action(PUSH)->udv_arg = add_udv(c_token);
 	    c_token++;
 	}
     }
     /* end if letter */
+
+#ifdef GP_STRING_VARS
+    /* Maybe it's a string constant */
+    else if (isstring(c_token)) {
+	union argument *foo = add_action(PUSHC);
+	foo->v_arg.type = STRING;
+	foo->v_arg.v.string_val = NULL;
+	/* this dynamically allocated string will be freed by free_at() */
+	m_quote_capture(&(foo->v_arg.v.string_val), c_token, c_token);
+	c_token++;
+    }
+#endif
+
     else
 	int_error(c_token, "invalid expression ");
 
@@ -398,6 +490,38 @@ parse_primary_expression()
 	parse_unary_expression();
 	(void) add_action(POWER);
     }
+#ifdef GP_STRING_VARS
+    /* Parse and add actions for range specifier applying to previous entity.
+     * Currently only used to generate substrings, but could also be used to
+     * extract vector slices.
+     */
+    if (equals(c_token, "[")) {
+	/* handle '*' or empty start of range */
+	if (equals(++c_token,"*") || equals(c_token,":")) {
+	    union argument *empty = add_action(PUSHC);
+	    empty->v_arg.type = INTGR;
+	    empty->v_arg.v.int_val = 1;
+	    if (equals(c_token,"*"))
+		c_token++;
+	} else
+	    parse_expression();
+	if (!equals(c_token, ":"))
+	    int_error(c_token, "':' expected");
+	/* handle '*' or empty end of range */
+	if (equals(++c_token,"*") || equals(c_token,"]")) {
+	    union argument *empty = add_action(PUSHC);
+	    empty->v_arg.type = INTGR;
+	    empty->v_arg.v.int_val = 65535; /* should be MAXINT */
+	    if (equals(c_token,"*"))
+		c_token++;
+	} else
+	    parse_expression();
+	if (!equals(c_token, "]"))
+	    int_error(c_token, "']' expected");
+	c_token++;
+	(void) add_action(RANGE);
+    }
+#endif
 }
 
 
@@ -411,7 +535,14 @@ parse_conditional_expression()
     /* create action code for ? : expressions */
 
     if (equals(c_token, "?")) {
-	register int savepc1, savepc2;
+	int savepc1, savepc2;
+
+	/* Fake same recursion level for alternatives
+	 *   set xlabel a>b ? 'foo' : 'bar' -1, 1
+	 * FIXME: This won't work:
+	 *   set xlabel a-b>c ? 'foo' : 'bar'  offset -1, 1
+	 */
+	parse_recursion_level--;
 
 	c_token++;
 	savepc1 = at->a_count;
@@ -426,6 +557,7 @@ parse_conditional_expression()
 	at->actions[savepc1].arg.j_arg = at->a_count - savepc1;
 	parse_expression();
 	at->actions[savepc2].arg.j_arg = at->a_count - savepc2;
+	parse_recursion_level++;
     }
 }
 
@@ -436,7 +568,7 @@ parse_logical_OR_expression()
     /* create action codes for || operator */
 
     while (equals(c_token, "||")) {
-	register int savepc;
+	int savepc;
 
 	c_token++;
 	savepc = at->a_count;
@@ -455,7 +587,7 @@ parse_logical_AND_expression()
     /* create action code for && operator */
 
     while (equals(c_token, "&&")) {
-	register int savepc;
+	int savepc;
 
 	c_token++;
 	savepc = at->a_count;
@@ -509,8 +641,8 @@ parse_AND_expression()
 static void
 parse_equality_expression()
 {
-    /* create action codes for == and !=
-     * operators */
+    /* create action codes for == and != numeric operators
+     * eq and ne string operators */
 
     while (TRUE) {
 	if (equals(c_token, "==")) {
@@ -521,6 +653,16 @@ parse_equality_expression()
 	    c_token++;
 	    accept_relational_expression();
 	    (void) add_action(NE);
+#ifdef GP_STRING_VARS
+	} else if (equals(c_token, "eq")) {
+	    c_token++;
+	    accept_relational_expression();
+	    (void) add_action(EQS);
+	} else if (equals(c_token, "ne")) {
+	    c_token++;
+	    accept_relational_expression();
+	    (void) add_action(NES);
+#endif
 	} else
 	    break;
     }
@@ -562,9 +704,19 @@ parse_relational_expression()
 static void
 parse_additive_expression()
 {
-    /* create action codes for + and - operators */
-
+    /* create action codes for +, - and . operators */
     while (TRUE) {
+#ifdef GP_STRING_VARS
+	if (equals(c_token, ".")) {
+	    c_token++;
+	    accept_multiplicative_expression();
+	    (void) add_action(CONCATENATE);
+	/* If only string results are wanted
+	 * do not accept '-' or '+' at the top level. */
+	} else if (string_result_only && parse_recursion_level == 1)
+	    break;
+	else
+#endif
 	if (equals(c_token, "+")) {
 	    c_token++;
 	    accept_multiplicative_expression();
@@ -627,39 +779,21 @@ parse_unary_expression()
 	parse_primary_expression();
 }
 
-/* FIXME HBB 20010724: These functions (add_udv, add_udf, and
- * is_builtin_function) belong into parse.c, really ! */
 /* find or add value and return pointer */
 struct udvt_entry *
-add_udv(t_num)		
-    int t_num;
+add_udv(int t_num)
 {
-    register struct udvt_entry **udv_ptr = &first_udv;
-
-    /* check if it's already in the table... */
-
-    while (*udv_ptr) {
-	if (equals(t_num, (*udv_ptr)->udv_name))
-	    return (*udv_ptr);
-	udv_ptr = &((*udv_ptr)->next_udv);
-    }
-
-    *udv_ptr = (struct udvt_entry *)
-	gp_alloc(sizeof(struct udvt_entry), "value");
-    (*udv_ptr)->next_udv = NULL;
-    (*udv_ptr)->udv_name = gp_alloc (token_len(t_num)+1, "user var");
-    copy_str((*udv_ptr)->udv_name, t_num, token_len(t_num)+1);
-    (*udv_ptr)->udv_value.type = INTGR;		/* not necessary, but safe! */
-    (*udv_ptr)->udv_undef = TRUE;
-    return (*udv_ptr);
+    char varname[MAX_ID_LEN+1];
+    copy_str(varname, t_num, MAX_ID_LEN);
+    return add_udv_by_name(varname);
 }
 
 
+/* find or add function at index <t_num>, and return pointer */
 struct udft_entry *
-add_udf(t_num)			/* find or add function and return pointer */
-    int t_num;			/* index to token[] */
+add_udf(int t_num)
 {
-    register struct udft_entry **udf_ptr = &first_udf;
+    struct udft_entry **udf_ptr = &first_udf;
 
     int i;
     while (*udf_ptr) {
@@ -689,12 +823,11 @@ add_udf(t_num)			/* find or add function and return pointer */
     return (*udf_ptr);
 }
 
-
+/* return standard function index or 0 */
 static int
-is_builtin_function(t_num)		/* return standard function index or 0 */
-    int t_num;
+is_builtin_function(int t_num)
 {
-    register int i;
+    int i;
 
     for (i = (int) SF_START; ft[i].f_name != NULL; i++) {
 	if (equals(t_num, ft[i].f_name))
@@ -703,30 +836,6 @@ is_builtin_function(t_num)		/* return standard function index or 0 */
     return (0);
 }
 
-/* EAM July 2003 - get_udv() is like add_udv except that it does not require
- * that the udv key be a token in the current command line. */
-struct udvt_entry *
-get_udv(key)		
-    char *key;
-{
-    register struct udvt_entry **udv_ptr = &first_udv;
-
-    /* check if it's already in the table... */
-
-    while (*udv_ptr) {
-	if (!strcmp(key, (*udv_ptr)->udv_name))
-	    return (*udv_ptr);
-	udv_ptr = &((*udv_ptr)->next_udv);
-    }
-
-    *udv_ptr = (struct udvt_entry *)
-	gp_alloc(sizeof(struct udvt_entry), "value");
-    (*udv_ptr)->next_udv = NULL;
-    (*udv_ptr)->udv_name = gp_strdup(key);
-    (*udv_ptr)->udv_undef = TRUE;
-    return (*udv_ptr);
-}
-
 void
 cleanup_udvlist()
 {
@@ -738,6 +847,7 @@ cleanup_udvlist()
 	    udv_dead = udv_ptr->next_udv;
 	    udv_ptr->next_udv = udv_dead->next_udv;
 	    FPRINTF((stderr,"cleanup_udvlist: deleting %s\n",udv_dead->udv_name));
+	    free(udv_dead->udv_name);
 	    free(udv_dead);
 	} else
 	    udv_ptr = udv_ptr->next_udv;
