@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: save.c,v 1.75 2004/04/13 17:23:59 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: save.c,v 1.132.2.5 2007/12/08 10:54:35 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - save.c */
@@ -34,7 +34,6 @@ static char *RCSid() { return RCSid("$Id: save.c,v 1.75 2004/04/13 17:23:59 broe
  * to the extent permitted by applicable law.
 ]*/
 
-#include "axis.h"
 #include "save.h"
 
 #include "command.h"
@@ -52,25 +51,21 @@ static char *RCSid() { return RCSid("$Id: save.c,v 1.75 2004/04/13 17:23:59 broe
 #include "term_api.h"
 #include "util.h"
 #include "variable.h"
-#ifdef PM3D
-# include "pm3d.h"
-# include "getcolor.h"
-#endif
+#include "pm3d.h"
+#include "getcolor.h"
 
 static void save_functions__sub __PROTO((FILE *));
 static void save_variables__sub __PROTO((FILE *));
 static void save_tics __PROTO((FILE *, AXIS_INDEX));
-static void save_position __PROTO((FILE *, struct position *));
+static void save_position __PROTO((FILE *, struct position *, TBOOLEAN offset));
 static void save_zeroaxis __PROTO((FILE *,AXIS_INDEX));
 static void save_set_all __PROTO((FILE *));
-static void save_textcolor __PROTO((FILE *, const struct t_colorspec *));
 
 /*
  *  functions corresponding to the arguments of the GNUPLOT `save` command
  */
 void
-save_functions(fp)
-FILE *fp;
+save_functions(FILE *fp)
 {
     /* I _love_ information written at the top and the end
      * of a human readable ASCII file. */
@@ -81,8 +76,7 @@ FILE *fp;
 
 
 void
-save_variables(fp)
-FILE *fp;
+save_variables(FILE *fp)
 {
 	show_version(fp);
 	save_variables__sub(fp);
@@ -91,8 +85,7 @@ FILE *fp;
 
 
 void
-save_set(fp)
-FILE *fp;
+save_set(FILE *fp)
 {
 	show_version(fp);
 	save_set_all(fp);
@@ -101,8 +94,7 @@ FILE *fp;
 
 
 void
-save_all(fp)
-FILE *fp;
+save_all(FILE *fp)
 {
 	show_version(fp);
 	save_set_all(fp);
@@ -122,10 +114,9 @@ FILE *fp;
  */
 
 static void
-save_functions__sub(fp)
-FILE *fp;
+save_functions__sub(FILE *fp)
 {
-    register struct udft_entry *udf = first_udf;
+    struct udft_entry *udf = first_udf;
 
     while (udf) {
 	if (udf->definition) {
@@ -136,17 +127,20 @@ FILE *fp;
 }
 
 static void
-save_variables__sub(fp)
-FILE *fp;
+save_variables__sub(FILE *fp)
 {
     /* always skip pi */
-    register struct udvt_entry *udv = first_udv->next_udv;
+    struct udvt_entry *udv = first_udv->next_udv;
 
     while (udv) {
 	if (!udv->udv_undef) {
-	    fprintf(fp, "%s = ", udv->udv_name);
-	    disp_value(fp, &(udv->udv_value));
-	    (void) putc('\n', fp);
+	    if (strncmp(udv->udv_name,"GPVAL_",6) 
+	     && strncmp(udv->udv_name,"MOUSE_",6)
+	     && strncmp(udv->udv_name,"NaN",4)) {
+		fprintf(fp, "%s = ", udv->udv_name);
+		disp_value(fp, &(udv->udv_value), TRUE);
+		(void) putc('\n', fp);
+	    }
 	}
 	udv = udv->next_udv;
     }
@@ -157,8 +151,7 @@ FILE *fp;
  * sequence. It's the only 'save' function that will write the
  * current term setting to a file uncommentedly. */
 void
-save_term(fp)
-FILE *fp;
+save_term(FILE *fp)
 {
 	show_version(fp);
 
@@ -182,8 +175,7 @@ FILE *fp;
 
 
 static void
-save_set_all(fp)
-    FILE *fp;
+save_set_all(FILE *fp)
 {
     struct text_label *this_label;
     struct arrow_def *this_arrow;
@@ -214,11 +206,11 @@ set bar %f\n",
 	    (clip_lines2) ? "" : "un",
 	    bar_size);
 
-    if (draw_border)
-	/* HBB 980609: handle border linestyle, too */
-	fprintf(fp, "set border %d lt %d lw %.3f\n",
-		draw_border, border_lp.l_type + 1, border_lp.l_width);
-    else
+    if (draw_border) {
+	fprintf(fp, "set border %d %s", draw_border, border_layer == 0 ? "back" : "front");
+	save_linetype(fp, &border_lp, FALSE);
+	fprintf(fp, "\n");
+    } else
 	fputs("unset border\n", fp);
 
     fprintf(fp, "\
@@ -234,47 +226,37 @@ set y2data%s\n",
 	    axis_array[SECOND_Y_AXIS].is_timedata ? " time" : "");
 
 #define SAVE_TIMEFMT(axis)						\
-    if (strlen(axis_array[axis].timefmt)) 						\
-	fprintf(fp, "set timefmt %s \"%s\"\n", axis_defaults[axis].name,	\
+    if (strlen(axis_array[axis].timefmt)) 				\
+	fprintf(fp, "set timefmt %s \"%s\"\n", axis_defaults[axis].name,\
 		conv_text(axis_array[axis].timefmt));
     SAVE_TIMEFMT(FIRST_X_AXIS);
     SAVE_TIMEFMT(FIRST_Y_AXIS);
     SAVE_TIMEFMT(FIRST_Z_AXIS);
     SAVE_TIMEFMT(SECOND_X_AXIS);
     SAVE_TIMEFMT(SECOND_Y_AXIS);
-#ifdef PM3D
     SAVE_TIMEFMT(COLOR_AXIS);
-#endif
 #undef SAVE_TIMEFMT
-    
+
     if (boxwidth < 0.0)
 	fputs("set boxwidth\n", fp);
     else
-#if USE_ULIG_RELATIVE_BOXWIDTH
 	fprintf(fp, "set boxwidth %g %s\n", boxwidth,
 		(boxwidth_is_absolute) ? "absolute" : "relative");
-#else
-    fprintf(fp, "set boxwidth %g\n", boxwidth);
-#endif /* USE_ULIG_RELATIVE_BOXWIDTH */
 
-#if USE_ULIG_FILLEDBOXES
-    switch(default_fillstyle.fillstyle) {
-    case FS_SOLID: 
-	fprintf(fp, "set style fill solid %f ", default_fillstyle.filldensity / 100.0);
-	break;
-    case FS_PATTERN: 
-	fprintf(fp, "set style fill pattern %d ", default_fillstyle.fillpattern);
-	break;
-    default: 
-	fprintf(fp, "set style fill empty ");
-	break;
-    }
-    if (default_fillstyle.border_linetype == LT_NODRAW)
-	fprintf(fp, "noborder\n");
-    else if (default_fillstyle.border_linetype == LT_UNDEFINED)
-	fprintf(fp, "border\n");
+    fprintf(fp, "set style fill ");
+    save_fillstyle(fp, &default_fillstyle);
+
+#ifdef EAM_OBJECTS
+    /* Default rectangle style */
+    fprintf(fp, "set style rectangle %s fc ",
+	    default_rectangle.layer > 0 ? "front" : 
+	    default_rectangle.layer < 0 ? "behind" : "back");
+    if (default_rectangle.lp_properties.use_palette)
+	save_pm3dcolor(fp, &default_rectangle.lp_properties.pm3d_color);
     else
-	fprintf(fp, "border %d\n",default_fillstyle.border_linetype+1);
+	fprintf(fp, "lt %d",default_rectangle.lp_properties.l_type+1);
+    fprintf(fp, " fillstyle ");
+    save_fillstyle(fp, &default_rectangle.fillstyle);
 #endif
 
     if (dgrid3d)
@@ -293,9 +275,7 @@ set y2data%s\n",
     SAVE_FORMAT(SECOND_X_AXIS);
     SAVE_FORMAT(SECOND_Y_AXIS);
     SAVE_FORMAT(FIRST_Z_AXIS );
-#ifdef PM3D
     SAVE_FORMAT(COLOR_AXIS );
-#endif
 #undef SAVE_FORMAT
 
     fprintf(fp, "set angles %s\n",
@@ -308,7 +288,7 @@ set y2data%s\n",
 	    fprintf(fp, "set grid polar %f\n", polar_grid_angle / ang2rad);
         else
 	    fputs("set grid nopolar\n", fp);
-	
+
 #define SAVE_GRID(axis)					\
 	fprintf(fp, " %s%stics %sm%stics",		\
 		axis_array[axis].gridmajor ? "" : "no",	\
@@ -322,57 +302,95 @@ set y2data%s\n",
 	fputs(" \\\n", fp);
 	SAVE_GRID(SECOND_X_AXIS);
 	SAVE_GRID(SECOND_Y_AXIS);
-#ifdef PM3D
 	SAVE_GRID(COLOR_AXIS);
-#endif
 	fputs("\n", fp);
 #undef SAVE_GRID
 
-	fprintf(fp, "set grid %s\n", (grid_layer==-1) ? "layerdefault" : ((grid_layer==0) ? "back" : "front"));
-    }	
+	fprintf(fp, "set grid %s  ", (grid_layer==-1) ? "layerdefault" : ((grid_layer==0) ? "back" : "front"));
+	save_linetype(fp, &grid_lp, FALSE);
+	fprintf(fp, ", ");
+	save_linetype(fp, &mgrid_lp, FALSE);
+	fputc('\n', fp);
+    }
 
     fprintf(fp, "set key title \"%s\"\n", conv_text(key->title));
     if (!(key->visible))
 	fputs("unset key\n", fp);
     else {
-	switch (key->flag) {
-	case KEY_AUTO_PLACEMENT:
-	    fputs("set key", fp);
-	    switch (key->hpos) {
-	    case TRIGHT:
-		fputs(" right", fp);
+	fputs("set key ", fp);
+	switch (key->region) {
+	case GPKEY_AUTO_INTERIOR_LRTBC:
+	    fputs("inside", fp);
+	    break;
+	case GPKEY_AUTO_EXTERIOR_LRTBC:
+	    fputs("outside", fp);
+	    break;
+	case GPKEY_AUTO_EXTERIOR_MARGIN:
+	    switch (key->margin) {
+	    case GPKEY_TMARGIN:
+		fputs("tmargin", fp);
 		break;
-	    case TLEFT:
-		fputs(" left", fp);
+	    case GPKEY_BMARGIN:
+		fputs("bmargin", fp);
 		break;
-	    case TOUT:
-		fputs(" out", fp);
+	    case GPKEY_LMARGIN:
+		fputs("lmargin", fp);
 		break;
-	    }
-	    switch (key->vpos) {
-	    case TTOP:
-		fputs(" top", fp);
-		break;
-	    case TBOTTOM:
-		fputs(" bottom", fp);
-		break;
-	    case TUNDER:
-		fputs(" below", fp);
+	    case GPKEY_RMARGIN:
+		fputs("rmargin", fp);
 		break;
 	    }
 	    break;
-	case KEY_USER_PLACEMENT:
-	    fputs("set key ", fp);
-	    save_position(fp, &key->user_pos);
+	case GPKEY_USER_PLACEMENT:
+	    save_position(fp, &key->user_pos, FALSE);
 	    break;
 	}
-	fprintf(fp, " %s %sreverse %senhanced box linetype %d linewidth %.3f samplen %g spacing %g width %g height %g %sautotitles\n",
-		key->just == JLEFT ? "Left" : "Right",
+	if (!(key->region == GPKEY_AUTO_EXTERIOR_MARGIN
+	      && (key->margin == GPKEY_LMARGIN || key->margin == GPKEY_RMARGIN))) {
+	    switch (key->hpos) {
+	    case RIGHT:
+		fputs(" right", fp);
+		break;
+	    case LEFT:
+		fputs(" left", fp);
+		break;
+	    case CENTRE:
+		fputs(" center", fp);
+		break;
+	    }
+	}
+	if (!(key->region == GPKEY_AUTO_EXTERIOR_MARGIN
+	      && (key->margin == GPKEY_TMARGIN || key->margin == GPKEY_BMARGIN))) {
+	    switch (key->vpos) {
+	    case JUST_TOP:
+		fputs(" top", fp);
+		break;
+	    case JUST_BOT:
+		fputs(" bottom", fp);
+		break;
+	    case JUST_CENTRE:
+		fputs(" center", fp);
+		break;
+	    }
+	}
+	fprintf(fp, " %s %s %sreverse %senhanced %s ",
+		key->stack_dir == GPKEY_VERTICAL ? "vertical" : "horizontal",
+		key->just == GPKEY_LEFT ? "Left" : "Right",
 		key->reverse ? "" : "no",
 		key->enhanced ? "" : "no",
-		key->box.l_type + 1, key->box.l_width,
-		key->swidth, key->vert_factor, key->width_fix, key->height_fix,
-		key->auto_titles ? "" : "no" );
+		key->auto_titles == COLUMNHEAD_KEYTITLES ? "autotitles columnhead"
+		: key->auto_titles == FILENAME_KEYTITLES ? "autotitles"
+		: "noautotitles" );
+	if (key->box.l_type > LT_NODRAW) {
+	    fputs("box", fp);
+	    save_linetype(fp, &(key->box), FALSE);
+	} else
+	    fputs("nobox", fp);
+	/* Put less common options on a separate line*/
+	fprintf(fp, "\nset key %sinvert samplen %g spacing %g width %g height %g ",
+		key->invert ? "" : "no",
+		key->swidth, key->vert_factor, key->width_fix, key->height_fix);
+	fputc('\n', fp);
     }
 
     fputs("unset label\n", fp);
@@ -381,7 +399,7 @@ set y2data%s\n",
 	fprintf(fp, "set label %d \"%s\" at ",
 		this_label->tag,
 		conv_text(this_label->text));
-	save_position(fp, &this_label->place);
+	save_position(fp, &this_label->place, FALSE);
 
 	switch (this_label->pos) {
 	case LEFT:
@@ -401,35 +419,35 @@ set y2data%s\n",
 	if (this_label->font != NULL)
 	    fprintf(fp, " font \"%s\"", this_label->font);
 	fprintf(fp, " %s", (this_label->layer==0) ? "back" : "front");
+	if (this_label->noenhanced)
+	    fprintf(fp, " noenhanced");
 	save_textcolor(fp, &(this_label->textcolor));
 	if (this_label->lp_properties.pointflag == 0)
 	    fprintf(fp, " nopoint");
 	else {
-	    fprintf(fp, " point linetype %d pointtype %d pointsize %g offset %g,%g",
-		this_label->lp_properties.l_type+1,
-		this_label->lp_properties.p_type+1,
-		this_label->lp_properties.p_size,
-		this_label->hoffset, this_label->voffset);
+	    fprintf(fp, " point");
+	    save_linetype(fp, &(this_label->lp_properties), TRUE);
 	}
+	save_position(fp, &this_label->offset, TRUE);
 	fputc('\n', fp);
     }
     fputs("unset arrow\n", fp);
     for (this_arrow = first_arrow; this_arrow != NULL;
 	 this_arrow = this_arrow->next) {
 	fprintf(fp, "set arrow %d from ", this_arrow->tag);
-	save_position(fp, &this_arrow->start);
+	save_position(fp, &this_arrow->start, FALSE);
 	fputs(this_arrow->relative ? " rto " : " to ", fp);
-	save_position(fp, &this_arrow->end);
-	fprintf(fp, " %s %s %s linetype %d linewidth %.3f",
+	save_position(fp, &this_arrow->end, FALSE);
+	fprintf(fp, " %s %s %s",
 		arrow_head_names[this_arrow->arrow_properties.head],
 		(this_arrow->arrow_properties.layer==0) ? "back" : "front",
 		( (this_arrow->arrow_properties.head_filled==2) ? "filled" :
 		  ( (this_arrow->arrow_properties.head_filled==1) ? "empty" :
-		    "nofilled" )),
-		this_arrow->arrow_properties.lp_properties.l_type + 1,
-		this_arrow->arrow_properties.lp_properties.l_width);
+		    "nofilled" )) );
+	save_linetype(fp, &(this_arrow->arrow_properties.lp_properties), FALSE);
 	if (this_arrow->arrow_properties.head_length > 0) {
-	    static char *msg[] = {"first", "second", "graph", "screen"};
+	    static char *msg[] = {"first", "second", "graph", "screen",
+				  "character"};
 	    fprintf(fp, " size %s %.3f,%.3f,%.3f",
 		    msg[this_arrow->arrow_properties.head_lengthunit],
 		    this_arrow->arrow_properties.head_length,
@@ -438,35 +456,28 @@ set y2data%s\n",
 	}
 	fprintf(fp, "\n");
     }
+    fprintf(fp, "set style increment %s\n", prefer_line_styles ? "userstyles" : "default");
     fputs("unset style line\n", fp);
     for (this_linestyle = first_linestyle; this_linestyle != NULL;
 	 this_linestyle = this_linestyle->next) {
 	fprintf(fp, "set style line %d ", this_linestyle->tag);
-#ifdef PM3D
-	if (this_linestyle->lp_properties.use_palette)
-	    fprintf(fp, "linetype palette ");
-	else
-#endif
-	    fprintf(fp, "linetype %d ", this_linestyle->lp_properties.l_type + 1);
-	fprintf(fp, "linewidth %.3f pointtype %d pointsize %.3f\n",
-		this_linestyle->lp_properties.l_width,
-		this_linestyle->lp_properties.p_type + 1,
-		this_linestyle->lp_properties.p_size);
+	save_linetype(fp, &(this_linestyle->lp_properties), TRUE);
+	fprintf(fp, "\n");
     }
     fputs("unset style arrow\n", fp);
     for (this_arrowstyle = first_arrowstyle; this_arrowstyle != NULL;
 	 this_arrowstyle = this_arrowstyle->next) {
 	fprintf(fp, "set style arrow %d", this_arrowstyle->tag);
-	fprintf(fp, " %s %s %s linetype %d linewidth %.3f",
+	fprintf(fp, " %s %s %s",
 		arrow_head_names[this_arrowstyle->arrow_properties.head],
 		(this_arrowstyle->arrow_properties.layer==0)?"back":"front",
 		( (this_arrowstyle->arrow_properties.head_filled==2)?"filled":
 		  ( (this_arrowstyle->arrow_properties.head_filled==1)?"empty":
-		    "nofilled" )),
-		this_arrowstyle->arrow_properties.lp_properties.l_type + 1,
-		this_arrowstyle->arrow_properties.lp_properties.l_width);
+		    "nofilled" )) );
+	save_linetype(fp, &(this_arrowstyle->arrow_properties.lp_properties), FALSE);
 	if (this_arrowstyle->arrow_properties.head_length > 0) {
-	    static char *msg[] = {"first", "second", "graph", "screen"};
+	    static char *msg[] = {"first", "second", "graph", "screen",
+				  "character"};
 	    fprintf(fp, " size %s %.3f,%.3f,%.3f",
 		    msg[this_arrowstyle->arrow_properties.head_lengthunit],
 		    this_arrowstyle->arrow_properties.head_length,
@@ -475,6 +486,29 @@ set y2data%s\n",
 	}
 	fprintf(fp, "\n");
     }
+
+#ifdef EAM_HISTOGRAMS
+    fprintf(fp, "set style histogram ");
+    switch (histogram_opts.type) {
+	default:
+	case HT_CLUSTERED:
+	    fprintf(fp,"clustered gap %d ",histogram_opts.gap); break;
+	case HT_ERRORBARS:
+	    fprintf(fp,"errorbars gap %d lw %g",histogram_opts.gap,histogram_opts.bar_lw); break;
+	case HT_STACKED_IN_LAYERS:
+	    fprintf(fp,"rowstacked "); break;
+	case HT_STACKED_IN_TOWERS:
+	    fprintf(fp,"columnstacked "); break;
+    }
+    fprintf(fp,"title ");
+    save_position(fp, &histogram_opts.title.offset, TRUE);
+    fprintf(fp, "\n");
+#endif
+
+#ifdef EAM_OBJECTS
+    save_rectangle(fp, 0);
+#endif
+
     fputs("unset logscale\n", fp);
 #define SAVE_LOG(axis)							\
     if (axis_array[axis].log)						\
@@ -485,6 +519,7 @@ set y2data%s\n",
     SAVE_LOG(SECOND_X_AXIS);
     SAVE_LOG(SECOND_Y_AXIS);
     SAVE_LOG(FIRST_Z_AXIS );
+    SAVE_LOG(COLOR_AXIS );
 #undef SAVE_LOG
 
     /* FIXME */
@@ -499,9 +534,12 @@ set encoding %s\n\
 	    encoding_names[encoding],
 	    (polar) ? "" : "un",
 	    (parametric) ? "" : "un");
-    if (decimalsign != NULL)
+    if (decimalsign != NULL) {
+#ifdef HAVE_LOCALE_H
+	fprintf(fp, "set decimalsign locale \"%s\"\n", setlocale(LC_NUMERIC,NULL));
+#endif
 	fprintf(fp, "set decimalsign '%s'\n", decimalsign);
-    else
+    } else
         fprintf(fp, "unset decimalsign\n");
 
     fputs("set view ", fp);
@@ -540,6 +578,11 @@ set isosamples %d, %d\n\
     else
 	fputs("unset clabel\n", fp);
 
+#ifdef GP_MACROS
+    if (expand_macros)
+	fputs("set macros\n", fp);
+#endif
+
     fputs("set mapping ", fp);
     switch (mapping3d) {
     case MAP3D_SPHERICAL:
@@ -562,6 +605,8 @@ set isosamples %d, %d\n\
 	fprintf(fp, "set datafile separator whitespace\n");
     if (strcmp(df_commentschars, DEFAULT_COMMENTS_CHARS))
 	fprintf(fp, "set datafile commentschars '%s'\n", df_commentschars);
+    if (df_fortran_constants)
+	fprintf(fp, "set datafile fortran\n");
 
     save_hidden3doptions(fp);
     fprintf(fp, "set cntrparam order %d\n", contour_order);
@@ -611,31 +656,30 @@ set origin %g,%g\n",
 
     save_zeroaxis(fp, FIRST_X_AXIS);
     save_zeroaxis(fp, FIRST_Y_AXIS);
+    save_zeroaxis(fp, FIRST_Z_AXIS);
     save_zeroaxis(fp, SECOND_X_AXIS);
     save_zeroaxis(fp, SECOND_Y_AXIS);
-    
-    fprintf(fp, "\
-set tics %s\n\
-set ticslevel %g\n\
-set ticscale %g %g\n",
-	    (tic_in) ? "in" : "out",
-	    ticslevel,
-	    ticscale, miniticscale);
 
-#define SAVE_MINI(axis)							  \
-    switch(axis_array[axis].minitics & TICS_MASK) {				  \
-    case 0:								  \
-	fprintf(fp, "set nom%stics\n", axis_defaults[axis].name);		  \
-	break;								  \
-    case MINI_AUTO:							  \
-	fprintf(fp, "set m%stics\n", axis_defaults[axis].name);		  \
-	break;								  \
-    case MINI_DEFAULT:							  \
-	fprintf(fp, "set m%stics default\n", axis_defaults[axis].name);	  \
-	break;								  \
-    case MINI_USER: fprintf(fp, "set m%stics %f\n", axis_defaults[axis].name, \
-			    axis_array[axis].mtic_freq);			  \
-	break;								  \
+    if (xyplane.absolute)
+	fprintf(fp, "set xyplane at %g\n", xyplane.xyplane_z);
+    else
+	fprintf(fp, "set ticslevel %g\n", xyplane.ticslevel);
+
+#define SAVE_MINI(axis)							\
+    switch(axis_array[axis].minitics & TICS_MASK) {			\
+    case 0:								\
+	fprintf(fp, "set nom%stics\n", axis_defaults[axis].name);	\
+	break;								\
+    case MINI_AUTO:							\
+	fprintf(fp, "set m%stics\n", axis_defaults[axis].name);		\
+	break;								\
+    case MINI_DEFAULT:							\
+	fprintf(fp, "set m%stics default\n", axis_defaults[axis].name);	\
+	break;								\
+    case MINI_USER:							\
+	fprintf(fp, "set m%stics %f\n", axis_defaults[axis].name,	\
+		axis_array[axis].mtic_freq);				\
+	break;								\
     }
 
     SAVE_MINI(FIRST_X_AXIS);
@@ -643,9 +687,7 @@ set ticscale %g %g\n",
     SAVE_MINI(FIRST_Z_AXIS);	/* HBB 20000506: noticed mztics were not saved! */
     SAVE_MINI(SECOND_X_AXIS);
     SAVE_MINI(SECOND_Y_AXIS);
-#ifdef PM3D
     SAVE_MINI(COLOR_AXIS);
-#endif
 #undef SAVE_MINI
 
     save_tics(fp, FIRST_X_AXIS);
@@ -653,29 +695,28 @@ set ticscale %g %g\n",
     save_tics(fp, FIRST_Z_AXIS);
     save_tics(fp, SECOND_X_AXIS);
     save_tics(fp, SECOND_Y_AXIS);
-#ifdef PM3D
     save_tics(fp, COLOR_AXIS);
-#endif
 
-#define SAVE_AXISLABEL_OR_TITLE(name,suffix,lab)	\
-    {							\
-	fprintf(fp, "set %s%s \"%s\" %f,%f ",		\
-		name, suffix, conv_text(lab.text),	\
-		lab.xoffset, lab.yoffset);		\
-	fprintf(fp, " font \"%s\"", conv_text(lab.font)); \
-	save_textcolor(fp, &(lab.textcolor)); \
-	fprintf(fp, "\n"); \
-}
+#define SAVE_AXISLABEL_OR_TITLE(name,suffix,lab)			 \
+    {									 \
+	fprintf(fp, "set %s%s \"%s\" ",					 \
+		name, suffix, lab.text ? conv_text(lab.text) : "");	 \
+	fprintf(fp, "\nset %s%s ", name, suffix);			 \
+        save_position(fp, &(lab.offset), TRUE);				 \
+	fprintf(fp, " font \"%s\"", lab.font ? conv_text(lab.font) : "");\
+	save_textcolor(fp, &(lab.textcolor));				 \
+	if (lab.rotate)							 \
+	    fprintf(fp, " rotate by %d", lab.rotate);			 \
+	else								 \
+	    fprintf(fp, " norotate");					 \
+	fprintf(fp, "%s\n", (lab.noenhanced) ? " noenhanced" : "");	 \
+    }
 
     SAVE_AXISLABEL_OR_TITLE("", "title", title);
 
     /* FIXME */
-    fprintf(fp, "set %s \"%s\" %s %srotate %f,%f ",
-	    "timestamp", conv_text(timelabel.text),
-	    (timelabel_bottom ? "bottom" : "top"),
-	    (timelabel_rotate ? "" : "no"),
-	    timelabel.xoffset, timelabel.yoffset);
-    fprintf(fp, " \"%s\"\n", conv_text(timelabel.font));
+    fprintf(fp, "set timestamp %s \n", timelabel_bottom ? "bottom" : "top");
+    SAVE_AXISLABEL_OR_TITLE("", "timestamp", timelabel);
 
     save_range(fp, R_AXIS);
     save_range(fp, T_AXIS);
@@ -685,7 +726,7 @@ set ticscale %g %g\n",
 #define SAVE_AXISLABEL(axis)					\
     SAVE_AXISLABEL_OR_TITLE(axis_defaults[axis].name,"label",	\
 			    axis_array[axis].label)
-	
+
     SAVE_AXISLABEL(FIRST_X_AXIS);
     SAVE_AXISLABEL(SECOND_X_AXIS);
     save_range(fp, FIRST_X_AXIS);
@@ -693,34 +734,42 @@ set ticscale %g %g\n",
 
     SAVE_AXISLABEL(FIRST_Y_AXIS);
     SAVE_AXISLABEL(SECOND_Y_AXIS);
-    save_range(fp, FIRST_Y_AXIS);
-    save_range(fp, SECOND_Y_AXIS);
+    if (splot_map == FALSE) {
+	save_range(fp, FIRST_Y_AXIS);
+	save_range(fp, SECOND_Y_AXIS);
+    } else { /* 'set view map' uses flipped y-axes */
+	splot_map_deactivate();
+	save_range(fp, FIRST_Y_AXIS);
+	save_range(fp, SECOND_Y_AXIS);
+	splot_map_activate();
+    }
 
     SAVE_AXISLABEL(FIRST_Z_AXIS);
     save_range(fp, FIRST_Z_AXIS);
 
-#ifdef PM3D
     SAVE_AXISLABEL(COLOR_AXIS);
     save_range(fp, COLOR_AXIS);
-#endif
 #undef SAVE_AXISLABEL
 #undef SAVE_AXISLABEL_OR_TITLE
 
     fprintf(fp, "set zero %g\n", zero);
-    fprintf(fp, "set lmargin %d\nset bmargin %d\n"
-	    "set rmargin %d\nset tmargin %d\n",
+    fprintf(fp, "set lmargin %g\nset bmargin %g\n"
+	    "set rmargin %g\nset tmargin %g\n",
 	    lmargin, bmargin, rmargin, tmargin);
 
     fprintf(fp, "set locale \"%s\"\n", get_locale());
 
-#ifdef PM3D
-    if (pm3d.where[0]) fprintf(fp, "set pm3d at %s\n", pm3d.where);
+    fputs("set pm3d ", fp);
+    fputs((PM3D_IMPLICIT == pm3d.implicit ? "implicit" : "explicit"), fp);
+    fprintf(fp, " at %s\n", pm3d.where);
     fputs("set pm3d ", fp);
     switch (pm3d.direction) {
-    case PM3D_SCANS_AUTOMATIC: fputs("scansautomatic", fp); break;
-    case PM3D_SCANS_FORWARD: fputs("scansforward", fp); break;
-    case PM3D_SCANS_BACKWARD: fputs("scansbackward", fp); break;
+    case PM3D_SCANS_AUTOMATIC: fputs("scansautomatic\n", fp); break;
+    case PM3D_SCANS_FORWARD: fputs("scansforward\n", fp); break;
+    case PM3D_SCANS_BACKWARD: fputs("scansbackward\n", fp); break;
+    case PM3D_DEPTH: fputs("depthorder\n", fp); break;
     }
+    fprintf(fp, "set pm3d interpolate %d,%d", pm3d.interp_i, pm3d.interp_j);
     fputs(" flush ", fp);
     switch (pm3d.flush) {
     case PM3D_FLUSH_CENTER: fputs("center", fp); break;
@@ -734,16 +783,16 @@ set ticscale %g %g\n",
 #if PM3D_HAVE_SOLID
     fputs((pm3d.solid ? " solid" : " transparent"), fp);
 #endif
-    fputs((PM3D_IMPLICIT == pm3d.implicit ? " implicit" : " explicit"), fp);
     fputs(" corners2color ", fp);
     switch (pm3d.which_corner_color) {
 	case PM3D_WHICHCORNER_MEAN:    fputs("mean", fp); break;
 	case PM3D_WHICHCORNER_GEOMEAN: fputs("geomean", fp); break;
 	case PM3D_WHICHCORNER_MEDIAN:  fputs("median", fp); break;
+	case PM3D_WHICHCORNER_MIN:     fputs("min", fp); break;
+	case PM3D_WHICHCORNER_MAX:     fputs("max", fp); break;
 	default: /* PM3D_WHICHCORNER_C1 ... _C4 */
 	     fprintf(fp, "c%i", pm3d.which_corner_color - PM3D_WHICHCORNER_C1 + 1);
     }
-    if (!pm3d.where[0]) fputs("\nunset pm3d", fp);
     fputs("\n", fp);
 
     /*
@@ -772,7 +821,7 @@ set ticscale %g %g\n",
       fputs( "\nset palette ", fp );
       switch( sm_palette.colorMode ) {
       case SMPAL_COLOR_MODE_RGB:
-	fprintf( fp, "rgbformulae %d, %d, %d\n", sm_palette.formulaR, 
+	fprintf( fp, "rgbformulae %d, %d, %d\n", sm_palette.formulaR,
 		 sm_palette.formulaG, sm_palette.formulaB );
 	break;
       case SMPAL_COLOR_MODE_GRADIENT: {
@@ -801,20 +850,20 @@ set ticscale %g %g\n",
     }
 
     /*
-     *  Save Colorbox info
+     *  Save colorbox info
      */
     if (color_box.where != SMCOLOR_BOX_NO)
 	fprintf(fp,"set colorbox %s\n", color_box.where==SMCOLOR_BOX_DEFAULT ? "default" : "user");
-    fprintf(fp, "set colorbox %sal origin %g,%g size %g,%g ",
-	color_box.rotation ==  'v' ? "vertic" : "horizont",
-	color_box.xorigin, color_box.yorigin,
-	color_box.xsize, color_box.ysize);
+    fprintf(fp, "set colorbox %sal origin ", color_box.rotation ==  'v' ? "vertic" : "horizont");
+    save_position(fp, &color_box.origin, FALSE);
+    fputs(" size ", fp);
+    save_position(fp, &color_box.size, FALSE);
+    fprintf(fp, " %s ", color_box.layer ==  LAYER_FRONT ? "front" : "back");
     if (color_box.border == 0) fputs("noborder", fp);
 	else if (color_box.border_lt_tag < 0) fputs("bdefault", fp);
 		 else fprintf(fp, "border %d", color_box.border_lt_tag);
     if (color_box.where == SMCOLOR_BOX_NO) fputs("\nunset colorbox\n", fp);
 	else fputs("\n", fp);
-#endif
 
     fputs("set loadpath ", fp);
     {
@@ -849,21 +898,24 @@ set ticscale %g %g\n",
 }
 
 static void
-save_tics(fp, axis)
-    FILE *fp;
-    AXIS_INDEX axis;
+save_tics(FILE *fp, AXIS_INDEX axis)
 {
-    if (axis_array[axis].ticmode == NO_TICS) {
+    if ((axis_array[axis].ticmode & TICS_MASK) == NO_TICS) {
 	fprintf(fp, "set no%stics\n", axis_defaults[axis].name);
 	return;
     }
-    fprintf(fp, "set %stics %s %smirror %s ", axis_defaults[axis].name,
+    fprintf(fp, "set %stics %s %s scale %g,%g %smirror %s ",
+	    axis_defaults[axis].name,
 	    ((axis_array[axis].ticmode & TICS_MASK) == TICS_ON_AXIS)
 	    ? "axis" : "border",
+	    (axis_array[axis].tic_in) ? "in" : "out",
+	    axis_array[axis].ticscale, axis_array[axis].miniticscale,
 	    (axis_array[axis].ticmode & TICS_MIRROR) ? "" : "no",
 	    axis_array[axis].tic_rotate ? "rotate" : "norotate");
     if (axis_array[axis].tic_rotate)
     	fprintf(fp,"by %d ",axis_array[axis].tic_rotate);
+    save_position(fp, &axis_array[axis].ticdef.offset, TRUE);
+    fprintf(fp, "\nset %stics ", axis_defaults[axis].name);
     switch (axis_array[axis].ticdef.type) {
     case TIC_COMPUTED:{
 	    fputs("autofreq ", fp);
@@ -892,40 +944,49 @@ save_tics(fp, axis)
 			     axis);
 	}
 	break;
+    case TIC_USER:
+	break;
+    }
 
-    case TIC_USER:{
-	    register struct ticmark *t;
-	    fputs(" (", fp);
-	    for (t = axis_array[axis].ticdef.def.user;
-		 t != NULL; t = t->next) {
-		if (t->label)
-		    fprintf(fp, "\"%s\" ", conv_text(t->label));
-		SAVE_NUM_OR_TIME(fp, (double) t->position, axis);
-		if (t->next) {
-		    fputs(", ", fp);
-		}
-	    }
-	    fputs(")", fp);
-	    break;
-	}
-    }
- 
-    if (axis_array[axis].ticdef.font && *axis_array[axis].ticdef.font) {
+    if (axis_array[axis].ticdef.font && *axis_array[axis].ticdef.font)
         fprintf(fp, " font \"%s\"", axis_array[axis].ticdef.font);
-    }
-    if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT) {
-        fprintf(fp, " textcolor lt %d", axis_array[axis].ticdef.textcolor.lt+1);    }
+
+    if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT)
+        save_textcolor(fp, &axis_array[axis].ticdef.textcolor);
+
     putc('\n', fp);
+
+    if (axis_array[axis].ticdef.def.user) { 
+	struct ticmark *t;
+	fprintf(fp, "set %stics %s ", axis_defaults[axis].name,
+		(axis_array[axis].ticdef.type == TIC_USER) ? "" : "add");
+	fputs(" (", fp);
+	for (t = axis_array[axis].ticdef.def.user; t != NULL; t = t->next) {
+	    if (t->label)
+		fprintf(fp, "\"%s\" ", conv_text(t->label));
+	    SAVE_NUM_OR_TIME(fp, (double) t->position, axis);
+	    if (t->level)
+		fprintf(fp, " %d", t->level);
+	    if (t->next) {
+		fputs(", ", fp);
+	    }
+	}
+	fputs(")\n", fp);
+    }
+
 }
 
 static void
-save_position(fp, pos)
-    FILE *fp;
-    struct position *pos;
+save_position(FILE *fp, struct position *pos, TBOOLEAN offset)
 {
-    static const char *msg[] = { "first ", "second ", "graph ", "screen " };
+    static const char *msg[] = { "first ", "second ", "graph ", "screen ",
+				 "character "};
+ 
+    assert(first_axes == 0 && second_axes == 1 && graph == 2 && screen == 3 &&
+	   character == 4);
 
-    assert(first_axes == 0 && second_axes == 1 && graph == 2 && screen == 3);
+    if (offset)
+	fprintf(fp, " offset ");
 
     fprintf(fp, "%s%g, %s%g, %s%g",
 	    pos->scalex == first_axes ? "" : msg[pos->scalex], pos->x,
@@ -935,9 +996,7 @@ save_position(fp, pos)
 
 
 void
-save_range(fp, axis)
-    FILE *fp;
-    AXIS_INDEX axis;
+save_range(FILE *fp, AXIS_INDEX axis)
 {
     fprintf(fp, "set %srange [ ", axis_defaults[axis].name);
     if (axis_array[axis].set_autoscale & AUTOSCALE_MIN) {
@@ -968,7 +1027,7 @@ save_range(fp, axis)
 	}
 	fputs("] )\n", fp);
 
-	if (axis_array[axis].set_autoscale & (AUTOSCALE_FIXMIN)) 
+	if (axis_array[axis].set_autoscale & (AUTOSCALE_FIXMIN))
 	    fprintf(fp, "set autoscale %sfixmin\n", axis_defaults[axis].name);
 	if (axis_array[axis].set_autoscale & AUTOSCALE_FIXMAX)
 	    fprintf(fp, "set autoscale %sfixmax\n", axis_defaults[axis].name);
@@ -977,23 +1036,56 @@ save_range(fp, axis)
 }
 
 static void
-save_zeroaxis(fp, axis)
-    FILE *fp;
-    AXIS_INDEX axis;
+save_zeroaxis(FILE *fp, AXIS_INDEX axis)
 {
-    fprintf(fp, "set %szeroaxis lt %d lw %.3f\n", axis_defaults[axis].name,
-	    axis_array[axis].zeroaxis.l_type + 1,
-	    axis_array[axis].zeroaxis.l_width);
-
+    fprintf(fp, "set %szeroaxis", axis_defaults[axis].name);
+    save_linetype(fp, &(axis_array[axis].zeroaxis), FALSE);
+    putc('\n', fp);
 }
 
-static void
-save_textcolor( FILE *fp, const struct t_colorspec *tc )
+void
+save_fillstyle(FILE *fp, const struct fill_style_type *fs)
+{
+    switch(fs->fillstyle) {
+    case FS_SOLID:
+	fprintf(fp, " solid %.2f ", fs->filldensity / 100.0);
+	break;
+    case FS_PATTERN:
+	fprintf(fp, " pattern %d ", fs->fillpattern);
+	break;
+    case FS_DEFAULT:
+	fprintf(fp, " default\n");
+	return;
+    default:
+	fprintf(fp, " empty ");
+	break;
+    }
+    if (fs->border_linetype == LT_NODRAW)
+	fprintf(fp, "noborder\n");
+    else if (fs->border_linetype == LT_UNDEFINED)
+	fprintf(fp, "border\n");
+    else
+	fprintf(fp, "border %d\n",fs->border_linetype+1);
+}
+
+void
+save_textcolor(FILE *fp, const struct t_colorspec *tc)
 {
     if (tc->type) {
     	fprintf(fp, " textcolor");
+	save_pm3dcolor(fp, tc);
+    }
+}
+
+
+void
+save_pm3dcolor(FILE *fp, const struct t_colorspec *tc)
+{
+    if (tc->type) {
 	switch(tc->type) {
-	case TC_LT:   fprintf(fp," lt %d", tc->lt+1); 
+	case TC_LT:   fprintf(fp," lt %d", tc->lt+1);
+		      break;
+	case TC_LINESTYLE:   fprintf(fp," linestyle %d", tc->lt);
 		      break;
 	case TC_Z:    fprintf(fp," palette z");
 		      break;
@@ -1001,12 +1093,21 @@ save_textcolor( FILE *fp, const struct t_colorspec *tc )
 		      break;
 	case TC_FRAC: fprintf(fp," palette fraction %4.2f", tc->value);
 		      break;
+	case TC_RGB:  {
+		      const char *color = reverse_table_lookup(pm3d_color_names_tbl, tc->lt);
+		      if (color)
+	    		fprintf(fp," rgb \"%s\" ", color);
+    		      else
+	    		fprintf(fp," rgb \"#%6.6x\" ", tc->lt);
+    		      break;
+	    	      }
+	default:      break;
 	}
     }
 }
 
 void
-save_data_func_style( FILE *fp, char *which, enum PLOT_STYLE style)
+save_data_func_style(FILE *fp, const char *which, enum PLOT_STYLE style)
 {
     switch (style) {
     case LINES:
@@ -1045,16 +1146,19 @@ save_data_func_style( FILE *fp, char *which, enum PLOT_STYLE style)
     case BOXES:
 	fputs("boxes\n", fp);
 	break;
-#ifdef PM3D
+#ifdef EAM_HISTOGRAMS
+    case HISTOGRAMS:
+	fputs("histograms\n", fp);
+	break;
+#endif
     case FILLEDCURVES:
 	fputs("filledcurves ", fp);
-	if (!strcmp(which,"data") || !strcmp(which,"Data"))
+	if (!strcmp(which, "data") || !strcmp(which, "Data"))
 	    filledcurves_options_tofile(&filledcurves_opts_data, fp);
 	else
 	    filledcurves_options_tofile(&filledcurves_opts_func, fp);
 	fputc('\n', fp);
 	break;
-#endif
     case BOXERROR:
 	fputs("boxerrorbars\n", fp);
 	break;
@@ -1079,12 +1183,101 @@ save_data_func_style( FILE *fp, char *which, enum PLOT_STYLE style)
     case CANDLESTICKS:
 	fputs("candlesticks\n", fp);
 	break;
-#ifdef PM3D
     case PM3DSURFACE:
 	fputs("pm3d\n", fp);
 	break;
+#ifdef EAM_DATASTRINGS
+    case LABELPOINTS:
+	fputs("labels\n", fp);
+	break;
+#endif
+#ifdef WITH_IMAGE
+    case IMAGE:
+	fputs("image\n", stderr);
+	break;
+    case RGBIMAGE:
+	fputs("rgbimage\n", stderr);
+	break;
 #endif
     default:
 	fputs("---error!---\n", fp);
     }
 }
+
+void
+save_linetype(FILE *fp, lp_style_type *lp, TBOOLEAN show_point)
+{
+
+    fprintf(fp, " linetype %d", lp->l_type + 1);
+    if (lp->use_palette) {
+	fprintf(fp, " linecolor");
+	if (lp->pm3d_color.type == TC_LT)
+    	    fprintf(fp, " %d", lp->pm3d_color.lt+1);
+	else
+    	    save_pm3dcolor(fp,&(lp->pm3d_color));
+    }
+    fprintf(fp, " linewidth %.3f", lp->l_width);
+
+    if (show_point && lp->pointflag) {
+	fprintf(fp, " pointtype %d", lp->p_type + 1);
+	if (lp->p_size == PTSZ_VARIABLE)
+	    fprintf(fp, " pointsize variable");
+	else if (lp->p_size == PTSZ_DEFAULT)
+	    fprintf(fp, " pointsize default");
+	else
+	    fprintf(fp, " pointsize %.3f", lp->p_size);
+    }
+	
+}
+
+#ifdef EAM_OBJECTS
+
+/* Save/show rectangle <tag> (0 means show all) */
+void
+save_rectangle(FILE *fp, int tag)
+{
+    t_object *this_object;
+    t_rectangle *this_rect;
+    TBOOLEAN showed = FALSE;
+
+    for (this_object = first_object; this_object != NULL; this_object = this_object->next) {
+	if (this_object->object_type == OBJ_RECTANGLE)
+	    this_rect = &this_object->o.rectangle;
+	else
+	    continue;
+	if (tag == 0 || tag == this_object->tag) {
+	    showed = TRUE;
+	    fprintf(fp, "%sobject %2d rect ", (fp==stderr) ? "\t" : "set ",this_object->tag);
+
+	    if (this_rect->type == 1) {
+		fprintf(fp, "center ");
+		save_position(fp, &this_rect->center, FALSE);
+		fprintf(fp, " size ");
+		save_position(fp, &this_rect->extent, FALSE);
+	    } else {
+		fprintf(fp, "from ");
+		save_position(fp, &this_rect->bl, FALSE);
+		fprintf(fp, " to ");
+		save_position(fp, &this_rect->tr, FALSE);
+	    }
+
+	    fprintf(fp, " %s ", this_object->layer > 0 ? "front" : this_object->layer < 0 ? "behind" : "back");
+	    if (this_object->lp_properties.l_width)
+		fprintf(fp, "lw %.1f ",this_object->lp_properties.l_width);
+	    fprintf(fp, "fc ");
+	    if (this_object->lp_properties.l_type == LT_DEFAULT)
+		fprintf(fp,"default");
+	    else if (this_object->lp_properties.use_palette)
+		save_pm3dcolor(fp, &this_object->lp_properties.pm3d_color);
+	    else
+		fprintf(fp, "lt %d",this_object->lp_properties.l_type+1);
+	    fprintf(fp, " fillstyle ");
+	    save_fillstyle(fp, &this_object->fillstyle);
+	}
+    }
+    if (tag > 0 && !showed)
+	int_error(c_token, "rect not found");
+}
+
+#endif
+
