@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: show.c,v 1.185.2.1 2006/10/29 05:50:42 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: show.c,v 1.185.2.3 2007/12/08 10:54:36 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - show.c */
@@ -1052,10 +1052,8 @@ show_version(FILE *fp)
 %s\tThomas Williams, Colin Kelley and many others\n\
 %s\n\
 %s\tType `help` to access the on-line reference manual.\n\
-%s\tThe gnuplot FAQ is available from\n\
-%s\t\t%s\n\
+%s\tThe gnuplot FAQ is available from %s\n\
 %s\n\
-%s\tSend comments and help requests to  <%s>\n\
 %s\tSend bug reports and suggestions to <%s>\n\
 %s\n",
 	    p,			/* empty line */
@@ -1068,10 +1066,8 @@ show_version(FILE *fp)
 	    p,			/* authors */
 	    p,			/* empty line */
 	    p,			/* Type `help` */
-	    p,			/* FAQ is at */
 	    p, faq_location,
 	    p,			/* empty line */
-	    p, help_email,
 	    p, bug_email,
 	    p);			/* empty line */
 
@@ -2255,22 +2251,26 @@ show_colorbox()
     } else {
 	fputs("\tcolor box without border is ", stderr);
     }
+    if (color_box.where != SMCOLOR_BOX_NO) {
+	if (color_box.layer == LAYER_FRONT) fputs("drawn front\n\t", stderr);
+	else fputs("drawn back\n\t", stderr);
+    }
     switch (color_box.where) {
 	case SMCOLOR_BOX_NO:
 	    fputs("NOT drawn\n", stderr);
 	    break;
 	case SMCOLOR_BOX_DEFAULT:
-	    fputs("drawn at DEFAULT position\n", stderr);
+	    fputs("at DEFAULT position\n", stderr);
 	    break;
 	case SMCOLOR_BOX_USER:
-	    fputs("drawn at USER position: ", stderr);
+	    fputs("at USER position: ", stderr);
 	    show_position(&color_box.origin);
 	    fputs("\n\t          size: ", stderr);
 	    show_position(&color_box.size);
 	    fputs("\n", stderr);
 	    break;
 	default: /* should *never* happen */
-	    int_error(NO_CARET, "Argh !");
+	    int_error(NO_CARET, "Argh!");
     }
     fprintf(stderr,"\tcolor gradient is %s in the color box\n",
 	color_box.rotation == 'v' ? "VERTICAL" : "HORIZONTAL");
