@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: term.c,v 1.151.2.1 2006/10/05 23:52:18 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: term.c,v 1.151.2.4 2008/02/05 06:14:55 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - term.c */
@@ -1506,6 +1506,7 @@ change_term(const char *origname, int length)
 {
     int i;
     struct termentry *t = NULL;
+    TBOOLEAN ambiguous = FALSE;
 
     /* For backwards compatibility only */
     char *name = (char *)origname;
@@ -1517,12 +1518,17 @@ change_term(const char *origname, int length)
     for (i = 0; i < TERMCOUNT; i++) {
         if (!strncmp(name, term_tbl[i].name, length)) {
             if (t)
-                return (NULL);  /* ambiguous */
+                ambiguous = TRUE;
             t = term_tbl + i;
+	    /* Exact match is always accepted */
+	    if (length == strlen(term_tbl[i].name)) {
+		ambiguous = FALSE;
+		break;
+	    }
         }
     }
 
-    if (!t)                     /* unknown */
+    if (!t || ambiguous)
         return (NULL);
 
     /* Success: set terminal type now */
@@ -1725,12 +1731,15 @@ init_terminal()
 	if (strchr(term_name,' '))
 	    namelength = strchr(term_name,' ') - term_name;
 
-	/* Force the terminal to initialize default fonts, etc.	*/
-	/* This prevents segfaults and other strangeness if you */
-	/* set GNUTERM to "post" or "png" for example.          */
-	/* Note that gp_input_line[] is blank at this point.	*/
+	/* Force the terminal to initialize default fonts, etc.	This prevents */
+	/* segfaults and other strangeness if you set GNUTERM to "post" or    */
+	/* "png" for example. However, calling X11_options() is expensive due */
+	/* to the fork+execute of gnuplot_x11 and x11 can tolerate not being  */
+	/* initialized until later.                                           */
+	/* Note that gp_input_line[] is blank at this point.	              */
         if (change_term(term_name, namelength)) {
-            term->options();
+            if (strcmp(term->name,"x11"))
+        	term->options();
             return;
         }
         fprintf(stderr, "Unknown or ambiguous terminal name '%s'\n", term_name);
@@ -2441,7 +2450,7 @@ enhanced_recursion(
                             ;   /* do nothing */
                     }
                     localfontname = p;
-                    while ((ch = *p) > ' ' && ch != '=' && ch != '*')
+                    while ((ch = *p) > ' ' && ch != '=' && ch != '*' && ch != '}')
                         ++p;
                     save = *(savepos=p);
                     if (ch == '=') {
