@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: unset.c,v 1.61 2004/04/13 17:24:02 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: unset.c,v 1.99.2.3 2008/03/03 18:04:47 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - unset.c */
@@ -41,6 +41,8 @@ static char *RCSid() { return RCSid("$Id: unset.c,v 1.61 2004/04/13 17:24:02 bro
 #include "contour.h"
 #include "datafile.h"
 #include "fit.h"
+#include "gp_hist.h"
+#include "hidden3d.h"
 #include "misc.h"
 #include "parse.h"
 #include "plot.h"
@@ -50,12 +52,9 @@ static char *RCSid() { return RCSid("$Id: unset.c,v 1.61 2004/04/13 17:24:02 bro
 #include "term_api.h"
 #include "util.h"
 #include "variable.h"
-#ifdef USE_MOUSE
-# include "mouse.h"
-#endif
-
-#ifdef PM3D
 #include "pm3d.h"
+#ifdef BINARY_DATA_FILE
+#  include "datafile.h"
 #endif
 
 static void unset_angles __PROTO((void));
@@ -68,9 +67,7 @@ static void unset_bars __PROTO((void));
 static void unset_border __PROTO((void));
 
 static void unset_boxwidth __PROTO((void));
-#if USE_ULIG_FILLEDBOXES
 static void unset_fillstyle __PROTO((void));
-#endif /* USE_ULIG_FILLEDBOXES */
 static void unset_clabel __PROTO((void));
 static void unset_clip __PROTO((void));
 static void unset_cntrparam __PROTO((void));
@@ -83,20 +80,29 @@ static void unset_fit __PROTO((void));
 static void unset_format __PROTO((void));
 static void unset_grid __PROTO((void));
 static void unset_hidden3d __PROTO((void));
-#if defined(HAVE_LIBREADLINE) && defined(GNUPLOT_HISTORY)
-static void unset_historysize __PROTO((void));
+#ifdef EAM_HISTOGRAMS
+static void unset_histogram __PROTO((void));
 #endif
+static void unset_historysize __PROTO((void));
 static void unset_isosamples __PROTO((void));
 static void unset_key __PROTO((void));
 static void unset_keytitle __PROTO((void));
 static void unset_label __PROTO((void));
 static void delete_label __PROTO((struct text_label * prev, struct text_label * this));
+#ifdef EAM_OBJECTS
+static void unset_object __PROTO((void));
+static void delete_object __PROTO((struct object * prev, struct object * this));
+static void unset_style_rectangle __PROTO(());
+#endif
 static void unset_loadpath __PROTO((void));
 static void unset_locale __PROTO((void));
 static void reset_logscale __PROTO((AXIS_INDEX));
 static void unset_logscale __PROTO((void));
+#ifdef GP_MACROS
+static void unset_macros __PROTO((void));
+#endif
 static void unset_mapping __PROTO((void));
-static void unset_margin __PROTO((int *));
+static void unset_margin __PROTO((float *));
 static void unset_missing __PROTO((void));
 #ifdef USE_MOUSE
 static void unset_mouse __PROTO((void));
@@ -105,19 +111,20 @@ static void unset_mouse __PROTO((void));
 static void unset_multiplot __PROTO((void));
 #endif
 
-static void unset_mtics __PROTO((AXIS_INDEX));
+static void unset_month_day_tics __PROTO((AXIS_INDEX));
+static void unset_minitics __PROTO((AXIS_INDEX));
+#ifdef OLDUNSETTICS
 static void unset_tics_in __PROTO((void));
+#endif /* OLDUNSETTICS */
 
 static void unset_offsets __PROTO((void));
 static void unset_origin __PROTO((void));
 static void unset_output __PROTO((void));
 static void unset_parametric __PROTO((void));
-#ifdef PM3D
 static void unset_pm3d __PROTO((void));
 static void unset_palette __PROTO((void));
 static void reset_colorbox __PROTO((void));
 static void unset_colorbox __PROTO((void));
-#endif
 static void unset_pointsize __PROTO((void));
 static void unset_polar __PROTO((void));
 static void unset_print __PROTO((void));
@@ -125,6 +132,7 @@ static void unset_samples __PROTO((void));
 static void unset_size __PROTO((void));
 static void unset_style __PROTO((void));
 static void unset_surface __PROTO((void));
+static void unset_table __PROTO((void));
 static void unset_terminal __PROTO((void));
 static void unset_tics __PROTO((AXIS_INDEX));
 static void unset_ticscale __PROTO((void));
@@ -138,14 +146,14 @@ static void unset_range __PROTO((AXIS_INDEX));
 static void unset_zeroaxis __PROTO((AXIS_INDEX));
 static void unset_all_zeroaxes __PROTO((void));
 
-static void unset_axislabel_or_title __PROTO((label_struct *));
+static void unset_axislabel_or_title __PROTO((text_label *));
 static void unset_axislabel __PROTO((AXIS_INDEX));
 
 /******** The 'unset' command ********/
 void
 unset_command()
 {
-    static char GPFAR unsetmess[] = 
+    static char GPFAR unsetmess[] =
     "valid unset options:  [] = choose one, {} means optional\n\n\
 \t'angles', 'arrow', 'autoscale', 'bar', 'border', 'boxwidth', 'clabel',\n\
 \t'clip', 'cntrparam', 'colorbox', 'contour', 'dgrid3d', 'decimalsign',\n\
@@ -224,11 +232,9 @@ unset_command()
     case S_HIDDEN3D:
 	unset_hidden3d();
 	break;
-#if defined(HAVE_LIBREADLINE) && defined(GNUPLOT_HISTORY)
     case S_HISTORYSIZE:
 	unset_historysize();
 	break;
-#endif
     case S_ISOSAMPLES:
 	unset_isosamples();
 	break;
@@ -250,6 +256,11 @@ unset_command()
     case S_LOGSCALE:
 	unset_logscale();
 	break;
+#ifdef GP_MACROS
+    case S_MACROS:
+	unset_macros();
+	break;
+#endif
     case S_MAPPING:
 	unset_mapping();
 	break;
@@ -266,10 +277,19 @@ unset_command()
 	unset_margin(&tmargin);
 	break;
     case S_DATAFILE:
+	if (almost_equals(c_token,"fort$ran")) {
+	    df_fortran_constants = FALSE;
+	    c_token++;
+	    break;
+	}
+	df_fortran_constants = FALSE;
 	unset_missing();
 	df_separator = '\0';
 	free(df_commentschars);
-	df_commentschars = strdup(DEFAULT_COMMENTS_CHARS);
+	df_commentschars = gp_strdup(DEFAULT_COMMENTS_CHARS);
+#ifdef BINARY_DATA_FILE
+	df_unset_datafile_binary();
+#endif
 	break;
 #ifdef USE_MOUSE
     case S_MOUSE:
@@ -292,7 +312,6 @@ unset_command()
     case S_PARAMETRIC:
 	unset_parametric();
 	break;
-#ifdef PM3D
     case S_PM3D:
 	unset_pm3d();
 	break;
@@ -302,7 +321,6 @@ unset_command()
     case S_COLORBOX:
 	unset_colorbox();
 	break;
-#endif
     case S_POINTSIZE:
 	unset_pointsize();
 	break;
@@ -312,6 +330,11 @@ unset_command()
     case S_PRINT:
 	unset_print();
 	break;
+#ifdef EAM_OBJECTS
+    case S_OBJECT:
+	unset_object();
+	break;
+#endif
     case S_SAMPLES:
 	unset_samples();
 	break;
@@ -324,11 +347,18 @@ unset_command()
     case S_SURFACE:
 	unset_surface();
 	break;
+    case S_TABLE:
+	unset_table();
+	break;
     case S_TERMINAL:
 	unset_terminal();
 	break;
     case S_TICS:
+#ifdef OLDUNSETTICS
 	unset_tics_in();
+#else
+	unset_tics(AXIS_ARRAY_SIZE);
+#endif /* OLDUNSETTICS */
 	break;
     case S_TICSCALE:
 	unset_ticscale();
@@ -353,61 +383,65 @@ unset_command()
 	break;
 /* FIXME - are the tics correct? */
     case S_MXTICS:
-	unset_mtics(FIRST_X_AXIS);
+	unset_minitics(FIRST_X_AXIS);
 	break;
     case S_XTICS:
 	unset_tics(FIRST_X_AXIS);
 	break;
     case S_XDTICS:
-    case S_XMTICS: 
+    case S_XMTICS:
+	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MYTICS:
-	unset_mtics(FIRST_Y_AXIS);
+	unset_minitics(FIRST_Y_AXIS);
 	break;
     case S_YTICS:
 	unset_tics(FIRST_Y_AXIS);
 	break;
     case S_YDTICS:
-    case S_YMTICS: 
+    case S_YMTICS:
+	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MX2TICS:
-	unset_mtics(SECOND_X_AXIS);
+	unset_minitics(SECOND_X_AXIS);
 	break;
     case S_X2TICS:
 	unset_tics(SECOND_X_AXIS);
 	break;
     case S_X2DTICS:
-    case S_X2MTICS: 
+    case S_X2MTICS:
+	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MY2TICS:
-	unset_mtics(SECOND_Y_AXIS);
+	unset_minitics(SECOND_Y_AXIS);
 	break;
     case S_Y2TICS:
 	unset_tics(SECOND_Y_AXIS);
 	break;
     case S_Y2DTICS:
-    case S_Y2MTICS: 
+    case S_Y2MTICS:
+	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MZTICS:
-	unset_mtics(FIRST_Z_AXIS);
+	unset_minitics(FIRST_Z_AXIS);
 	break;
     case S_ZTICS:
 	unset_tics(FIRST_Z_AXIS);
 	break;
     case S_ZDTICS:
-    case S_ZMTICS: 
+    case S_ZMTICS:
+	unset_month_day_tics(FIRST_X_AXIS);
 	break;
-#ifdef PM3D
     case S_MCBTICS:
-	unset_mtics(COLOR_AXIS);
+	unset_minitics(COLOR_AXIS);
 	break;
     case S_CBTICS:
 	unset_tics(COLOR_AXIS);
 	break;
     case S_CBDTICS:
-    case S_CBMTICS: 
+    case S_CBMTICS:
+	unset_month_day_tics(FIRST_X_AXIS);
 	break;
-#endif
     case S_XDATA:
 	unset_timedata(FIRST_X_AXIS);
 	/* FIXME HBB 20000506: does unsetting these axes make *any*
@@ -424,11 +458,9 @@ unset_command()
     case S_ZDATA:
 	unset_timedata(FIRST_Z_AXIS);
 	break;
-#ifdef PM3D
     case S_CBDATA:
 	unset_timedata(COLOR_AXIS);
 	break;
-#endif
     case S_X2DATA:
 	unset_timedata(SECOND_X_AXIS);
 	break;
@@ -444,11 +476,9 @@ unset_command()
     case S_ZLABEL:
 	unset_axislabel(FIRST_Z_AXIS);
 	break;
-#ifdef PM3D
     case S_CBLABEL:
 	unset_axislabel(COLOR_AXIS);
 	break;
-#endif
     case S_X2LABEL:
 	unset_axislabel(SECOND_X_AXIS);
 	break;
@@ -470,11 +500,9 @@ unset_command()
     case S_ZRANGE:
 	unset_range(FIRST_Z_AXIS);
 	break;
-#ifdef PM3D
     case S_CBRANGE:
 	unset_range(COLOR_AXIS);
 	break;
-#endif
     case S_RRANGE:
 	unset_range(R_AXIS);
 	break;
@@ -493,6 +521,9 @@ unset_command()
     case S_YZEROAXIS:
 	unset_zeroaxis(FIRST_Y_AXIS);
 	break;
+    case S_ZZEROAXIS:
+	unset_zeroaxis(FIRST_Z_AXIS);
+	break;
     case S_X2ZEROAXIS:
 	unset_zeroaxis(SECOND_X_AXIS);
 	break;
@@ -507,6 +538,7 @@ unset_command()
 	int_error(c_token, unsetmess);
 	break;
     }
+    update_gpval_variables(0); /* update GPVAL_ inner variables */
 }
 
 
@@ -535,7 +567,7 @@ unset_arrow()
 	/* get tag */
 	tag = (int) real(const_express(&a));
 	if (!END_OF_COMMAND)
-	    int_error(c_token, "extraneous arguments to set noarrow");
+	    int_error(c_token, "extraneous arguments to unset arrow");
 	for (this_arrow = first_arrow, prev_arrow = NULL;
 	     this_arrow != NULL;
 	     prev_arrow = this_arrow, this_arrow = this_arrow->next) {
@@ -550,14 +582,13 @@ unset_arrow()
 
 
 /* delete arrow from linked list started by first_arrow.
- * called with pointers to the previous arrow (prev) and the 
+ * called with pointers to the previous arrow (prev) and the
  * arrow to delete (this).
  * If there is no previous arrow (the arrow to delete is
  * first_arrow) then call with prev = NULL.
  */
 static void
-delete_arrow(prev, this)
-struct arrow_def *prev, *this;
+delete_arrow(struct arrow_def *prev, struct arrow_def *this)
 {
     if (this != NULL) {		/* there really is something to delete */
 	if (prev != NULL)	/* there is a previous arrow */
@@ -630,13 +661,10 @@ static void
 unset_boxwidth()
 {
     boxwidth = -1.0;
-#if USE_ULIG_RELATIVE_BOXWIDTH
     boxwidth_is_absolute = TRUE;
-#endif
 }
 
 
-#if USE_ULIG_FILLEDBOXES
 /* process 'unset fill' command */
 static void
 unset_fillstyle()
@@ -646,7 +674,6 @@ unset_fillstyle()
     default_fillstyle.fillpattern = 0;
     default_fillstyle.border_linetype = LT_UNDEFINED;
 }
-#endif /* USE_ULIG_FILLEDBOXES */
 
 
 /* process 'unset clabel' command */
@@ -727,13 +754,16 @@ unset_encoding()
 }
 
 
-/* process 'unset encoding' command */
+/* process 'unset decimalsign' command */
 static void
 unset_decimalsign()
 {
     if (decimalsign != NULL)
 	free(decimalsign);
     decimalsign = NULL;
+#ifdef HAVE_LOCALE_H
+    setlocale(LC_NUMERIC,"C");
+#endif
 }
 
 
@@ -777,9 +807,7 @@ unset_format()
 	SET_DEFFORMAT(FIRST_Z_AXIS , set_for_axis);
 	SET_DEFFORMAT(SECOND_X_AXIS, set_for_axis);
 	SET_DEFFORMAT(SECOND_Y_AXIS, set_for_axis);
-#ifdef PM3D
 	SET_DEFFORMAT(COLOR_AXIS   , set_for_axis);
-#endif
     }
 }
 
@@ -812,15 +840,25 @@ unset_hidden3d()
 #endif
 }
 
+#ifdef EAM_HISTOGRAMS
+static void
+unset_histogram()
+{
+    histogram_opts.type = HT_CLUSTERED;
+    histogram_opts.gap = 2;
+}
+#endif
 
-#if defined(HAVE_LIBREADLINE) && defined(GNUPLOT_HISTORY)
 /* process 'unset historysize' command */
 static void
 unset_historysize()
 {
+#ifdef GNUPLOT_HISTORY
     gnuplot_history_size = -1; /* don't ever truncate the history. */
-}
+#else
+    int_error(c_token, "Command 'unset historysize' requires history support.");
 #endif
+}
 
 
 /* process 'unset isosamples' command */
@@ -839,7 +877,7 @@ unset_isosamples()
 
 
 void
-reset_key(void)
+reset_key()
 {
     legend_key temp_key = DEFAULT_KEY_PROPS;
     memcpy(&keyT, &temp_key, sizeof(keyT));
@@ -880,7 +918,7 @@ unset_label()
 	/* get tag */
 	tag = (int) real(const_express(&a));
 	if (!END_OF_COMMAND)
-	    int_error(c_token, "extraneous arguments to set nolabel");
+	    int_error(c_token, "extraneous arguments to unset label");
 	for (this_label = first_label, prev_label = NULL;
 	     this_label != NULL;
 	     prev_label = this_label, this_label = this_label->next) {
@@ -889,20 +927,19 @@ unset_label()
 		return;		/* exit, our job is done */
 	    }
 	}
-	int_error(c_token, "label not found");
+	/* int_warn(c_token, "label not found"); */
     }
 }
 
 
 /* delete label from linked list started by first_label.
- * called with pointers to the previous label (prev) and the 
+ * called with pointers to the previous label (prev) and the
  * label to delete (this).
  * If there is no previous label (the label to delete is
  * first_label) then call with prev = NULL.
  */
 static void
-delete_label(prev, this)
-struct text_label *prev, *this;
+delete_label(struct text_label *prev, struct text_label *this)
 {
     if (this != NULL) {		/* there really is something to delete */
 	if (prev != NULL)	/* there is a previous label */
@@ -915,6 +952,58 @@ struct text_label *prev, *this;
     }
 }
 
+#ifdef EAM_OBJECTS
+/* process 'unset rectangle' command */
+static void
+unset_object()
+{
+    struct value a;
+    struct object *this_object;
+    struct object *prev_object;
+    int tag;
+
+    if (END_OF_COMMAND) {
+	/* delete all objects */
+	while (first_object != NULL)
+	    delete_object((struct object *) NULL, first_object);
+    } else {
+	/* get tag */
+	tag = (int) real(const_express(&a));
+	if (!END_OF_COMMAND)
+	    int_error(c_token, "extraneous arguments to unset rectangle");
+	for (this_object = first_object, prev_object = NULL;
+	     this_object != NULL;
+	     prev_object = this_object, this_object = this_object->next) {
+	    if (this_object->tag == tag) {
+		delete_object(prev_object, this_object);
+		return;		/* exit, our job is done */
+	    }
+	}
+	int_error(c_token, "object not found");
+    }
+}
+
+
+/* delete object from linked list started by first_object.
+ * called with pointers to the previous object (prev) and the
+ * object to delete (this).
+ * If there is no previous object (the object to delete is
+ * first_object) then call with prev = NULL.
+ */
+static void
+delete_object(struct object *prev, struct object *this)
+{
+    if (this != NULL) {		/* there really is something to delete */
+	if (prev != NULL)	/* there is a previous rectangle */
+	    prev->next = this->next;
+	else			/* this = first_object so change first_object */
+	    first_object = this->next;
+	/* FIXME:  Must free contents as well */
+	free (this);
+    }
+}
+#endif
+
 
 /* process 'unset loadpath' command */
 static void
@@ -932,8 +1021,7 @@ unset_locale()
 }
 
 static void
-reset_logscale(axis)
-    AXIS_INDEX axis;
+reset_logscale(AXIS_INDEX axis)
 {
     axis_array[axis].log = FALSE;
     axis_array[axis].base = 0.0;
@@ -952,10 +1040,12 @@ unset_logscale()
 	    reset_logscale(axis);
     } else {
 	int i = 0;
-	/* do reverse search because of "x", "x1", "x2" sequence in axisname_tbl */
+
+	/* do reverse search because of "x", "x1", "x2" sequence in
+	 * axisname_tbl */
 	while (i < token[c_token].length) {
-	    axis = lookup_table_nth_reverse( axisname_tbl, AXIS_ARRAY_SIZE, 
-		       input_line+token[c_token].start_index+i );
+	    axis = lookup_table_nth_reverse(axisname_tbl, AXIS_ARRAY_SIZE,
+					    gp_input_line + token[c_token].start_index + i);
 	    if (axis < 0) {
 		token[c_token].start_index += i;
 		int_error(c_token, "unknown axis");
@@ -967,6 +1057,14 @@ unset_logscale()
     }
 }
 
+#ifdef GP_MACROS
+/* process 'unset macros' command */
+static void
+unset_macros()
+{
+    expand_macros = FALSE;
+}
+#endif
 
 /* process 'unset mapping3d' command */
 static void
@@ -976,10 +1074,9 @@ unset_mapping()
     mapping3d = MAP3D_CARTESIAN;
 }
 
-
 /* process 'unset {blrt}margin' command */
 static void
-unset_margin(int *margin)
+unset_margin(float *margin)
 {
     *margin = -1;
 }
@@ -999,7 +1096,7 @@ unset_mouse()
 {
     mouse_setting.on = 0;
 #ifdef OS2
-    update_menu_items_PM_terminal();
+    PM_update_menu_items();
 #endif
     UpdateStatusline(); /* wipe status line */
 }
@@ -1007,36 +1104,53 @@ unset_mouse()
 
 /* process 'unset mxtics' command */
 static void
-unset_mtics(axis)
-    AXIS_INDEX axis;
+unset_minitics(AXIS_INDEX axis)
 {
-    axis_array[axis].minitics = MINI_DEFAULT;
+    axis_array[axis].minitics = MINI_OFF;
     axis_array[axis].mtic_freq = 10.0;
 }
 
 
 /*process 'unset {x|y|x2|y2|z}tics' command */
 static void
-unset_tics(axis)
-    AXIS_INDEX axis;
+unset_tics(AXIS_INDEX axis)
 {
-    axis_array[axis].ticmode = NO_TICS;
+    struct position tics_nooffset = { character, character, character, 0., 0., 0.};
+    unsigned int istart = 0;
+    unsigned int iend = AXIS_ARRAY_SIZE;
+    unsigned int i;
 
-    if (axis_array[axis].ticdef.font) {
-	free(axis_array[axis].ticdef.font);
-	axis_array[axis].ticdef.font = NULL;
+    if (axis < AXIS_ARRAY_SIZE) {
+	istart = axis;
+	iend = axis + 1;
     }
-    axis_array[axis].ticdef.textcolor.type = TC_DEFAULT;
-    axis_array[axis].ticdef.textcolor.lt = 0;
-    axis_array[axis].ticdef.textcolor.value = 0;
-    axis_array[axis].tic_rotate = 0;
+	
+    for (i = istart; i < iend; ++i) {
+	axis_array[i].ticmode = NO_TICS;
 
-    if (axis_array[axis].ticdef.type == TIC_USER) {
-	free_marklist(axis_array[axis].ticdef.def.user);
-	axis_array[axis].ticdef.def.user = NULL;
+	if (axis_array[i].ticdef.font) {
+	    free(axis_array[i].ticdef.font);
+	    axis_array[i].ticdef.font = NULL;
+	}
+	axis_array[i].ticdef.textcolor.type = TC_DEFAULT;
+	axis_array[i].ticdef.textcolor.lt = 0;
+	axis_array[i].ticdef.textcolor.value = 0;
+	axis_array[i].ticdef.offset = tics_nooffset;
+	axis_array[i].tic_rotate = 0;
+	axis_array[i].ticscale = 1.0;
+	axis_array[i].miniticscale = 0.5;
+	axis_array[i].tic_in = TRUE;
+
+	free_marklist(axis_array[i].ticdef.def.user);
+	axis_array[i].ticdef.def.user = NULL;
     }
 }
 
+static void
+unset_month_day_tics(AXIS_INDEX axis)
+{
+    axis_array[axis].ticdef.type = TIC_COMPUTED;
+}
 
 /* process 'unset offsets' command */
 static void
@@ -1059,12 +1173,16 @@ unset_origin()
 static void
 unset_output()
 {
-    if (multiplot)
+    if (multiplot) {
 	int_error(c_token, "you can't change the output in multiplot mode");
+	return;
+    }
 
     term_set_output(NULL);
-    free(outstr);
-    outstr = NULL; /* means STDOUT */
+    if (outstr) {
+	free(outstr);
+	outstr = NULL; /* means STDOUT */
+    }
 }
 
 
@@ -1090,7 +1208,6 @@ unset_parametric()
     }
 }
 
-#ifdef PM3D
 /* process 'unset palette' command */
 static void
 unset_palette()
@@ -1104,14 +1221,7 @@ unset_palette()
 static void
 reset_colorbox()
 {
-    color_box.where = SMCOLOR_BOX_DEFAULT;
-    color_box.rotation = 'v';
-    color_box.border = 1;
-    color_box.border_lt_tag = -1; /* use default border */
-    color_box.xorigin = 0.9; /* default user origin and size */
-    color_box.yorigin = 0.2;
-    color_box.xsize = 0.1;
-    color_box.ysize = 0.63;
+    color_box = default_color_box;
 }
 
 
@@ -1129,12 +1239,12 @@ unset_colorbox()
 static void
 unset_pm3d()
 {
-    pm3d.where[0] = 0;
+    pm3d.implicit = PM3D_EXPLICIT;
     /* reset styles, required to 'plot something' after e.g. 'set pm3d map' */
     if (data_style == PM3DSURFACE) data_style = POINTSTYLE;
     if (func_style == PM3DSURFACE) func_style = LINES;
 }
-#endif
+
 
 /* process 'unset pointsize' command */
 static void
@@ -1200,12 +1310,16 @@ unset_style()
 	func_style = LINES;
 	while (first_linestyle != NULL)
 	    delete_linestyle((struct linestyle_def *) NULL, first_linestyle);
-#if USE_ULIG_FILLEDBOXES
 	unset_fillstyle();
+#ifdef EAM_OBJECTS
+	unset_style_rectangle();
+#endif
+#ifdef EAM_HISTOGRAMS
+	unset_histogram();
 #endif
 	c_token++;
 	return;
-    } 
+    }
 
     switch(lookup_table(show_style_tbl, c_token)){
     case SHOW_STYLE_DATA:
@@ -1217,26 +1331,45 @@ unset_style()
 	c_token++;
 	break;
     case SHOW_STYLE_LINE:
-	while (first_linestyle != NULL)
-	    delete_linestyle((struct linestyle_def *) NULL, first_linestyle);
 	c_token++;
+	if (END_OF_COMMAND) {
+	    while (first_linestyle != NULL)
+		delete_linestyle((struct linestyle_def *) NULL, first_linestyle);
+	} else {
+	    struct value a;
+	    int tag = (int) real(const_express(&a));
+	    struct linestyle_def *this, *prev;
+	    for (this = first_linestyle, prev = NULL; this != NULL; 
+		 prev = this, this = this->next) {
+		if (this->tag == tag) {
+		    delete_linestyle(prev, this);
+		    break;
+		}
+	    }
+	}
 	break;
-#if USE_ULIG_FILLEDBOXES
     case SHOW_STYLE_FILLING:
 	unset_fillstyle();
 	c_token++;
 	break;
-#endif /* USE_ULIG_FILLEDBOXES */
+#ifdef EAM_HISTOGRAMS
+    case SHOW_STYLE_HISTOGRAM:
+	unset_histogram();
+	c_token++;
+	break;
+#endif
     case SHOW_STYLE_ARROW:
 	unset_arrowstyles();
 	c_token++;
 	break;
+#ifdef EAM_OBJECTS
+    case SHOW_STYLE_RECTANGLE:
+	unset_style_rectangle();
+	c_token++;
+	break;
+#endif
     default:
-#if USE_ULIG_FILLEDBOXES
 	int_error(c_token, "expecting 'data', 'function', 'line', 'fill' or 'arrow'");
-#else
-	int_error(c_token, "expecting 'data', 'function', 'line', 'fill' or 'arrow'");
-#endif /* USE_ULIG_FILLEDBOXES */
     }
 }
 
@@ -1249,6 +1382,17 @@ unset_surface()
 }
 
 
+/* process 'unset table' command */
+static void
+unset_table()
+{
+    if (table_outfile)
+	fclose(table_outfile);
+    table_outfile = NULL;
+    table_mode = FALSE;
+}
+
+
 /* process 'unset terminal' comamnd */
 static void
 unset_terminal()
@@ -1263,20 +1407,52 @@ unset_terminal()
 }
 
 
+#ifdef OLDUNSETTICS
 /* process 'unset tics' command */
 static void
 unset_tics_in()
 {
-    tic_in = TRUE;
+    unsigned int i = AXIS_ARRAY_SIZE;
+    c_token++;
+
+    if (equals(c_token,"x"))
+	i = FIRST_X_AXIS;
+    else if (equals(c_token,"y"))
+	i = FIRST_Y_AXIS;
+    else if (equals(c_token,"z"))
+	i = FIRST_Z_AXIS;
+    else if (equals(c_token,"x2"))
+	i = SECOND_X_AXIS;
+    else if (equals(c_token,"y2"))
+	i = SECOND_Y_AXIS;
+    else if (equals(c_token,"cb"))
+	i = COLOR_AXIS;
+    if (i < AXIS_ARRAY_SIZE)
+	c_token++;
+
+    if (i < AXIS_ARRAY_SIZE)
+	axis_array[i].tic_in = TRUE;
+    else {
+	for (i = 0; i < AXIS_ARRAY_SIZE; ++i)
+	    axis_array[i].tic_in = TRUE;
+    }
 }
+#endif /* OLDUNSETTICS */
 
 
 /* process 'unset ticscale' command */
 static void
 unset_ticscale()
 {
-    ticscale = 1.0;
-    miniticscale = 0.5;
+    unsigned int i;
+
+    int_warn(c_token,
+	     "Deprecated syntax - please use 'set tics scale default'");
+
+    for (i = 0; i < AXIS_ARRAY_SIZE; ++i) {
+	axis_array[i].ticscale = 1.0;
+	axis_array[i].miniticscale = 0.5;
+    }
 }
 
 
@@ -1284,7 +1460,8 @@ unset_ticscale()
 static void
 unset_ticslevel()
 {
-    ticslevel = 0.5;
+    xyplane.ticslevel = 0.5;
+    xyplane.absolute = FALSE;
 }
 
 
@@ -1304,11 +1481,11 @@ unset_timefmt()
     else
 	/* int_error() from inside 'reset' would cause problems */
 	int_warn(c_token, "expected optional axis name");
-	
+
 }
 
 
-/* process 'unset notimestamp' command */
+/* process 'unset timestamp' command */
 static void
 unset_timestamp()
 {
@@ -1340,8 +1517,7 @@ unset_zero()
 
 /* process 'unset {x|y|z|x2|y2}data' command */
 static void
-unset_timedata(axis)
-    AXIS_INDEX axis;
+unset_timedata(AXIS_INDEX axis)
 {
     axis_array[axis].is_timedata = FALSE;
 }
@@ -1349,18 +1525,16 @@ unset_timedata(axis)
 
 /* process 'unset {x|y|z|x2|y2|t|u|v|r}range' command */
 static void
-unset_range(axis)
-    AXIS_INDEX axis;
+unset_range(AXIS_INDEX axis)
 {
     /* FIXME HBB 20000506: do we want to reset the axis autoscale and
      * min/max, too?  */
     axis_array[axis].range_flags = 0;
 }
 
-/* process 'unset {x|y|z|x2|y2}zeroaxis' command */
+/* process 'unset {x|y|x2|y2|z}zeroaxis' command */
 static void
-unset_zeroaxis(axis)
-    AXIS_INDEX axis;
+unset_zeroaxis(AXIS_INDEX axis)
 {
     axis_array[axis].zeroaxis = default_axis_zeroaxis;
 }
@@ -1379,21 +1553,27 @@ unset_all_zeroaxes()
 
 /* process 'unset [xyz]{2}label command */
 static void
-unset_axislabel_or_title(label)
-    label_struct *label;
+unset_axislabel_or_title(text_label *label)
 {
-    strcpy(label->text, "");
-    strcpy(label->font, "");
-    label->xoffset = 0;
-    label->yoffset = 0;
-    label->textcolor.type = TC_DEFAULT;
+    struct position default_offset = { character, character, character, 
+				       0., 0., 0. };
+    if (label) {
+	free(label->text);
+	label->text = NULL;
+	free(label->font);
+	label->font = NULL;
+	label->offset = default_offset;
+	label->textcolor.type = TC_DEFAULT;
+    }
 }
 
 static void
-unset_axislabel(axis)
-    AXIS_INDEX axis;
+unset_axislabel(AXIS_INDEX axis)
 {
+    unset_axislabel_or_title(&axis_array[axis].label);
     axis_array[axis].label = default_axis_label;
+    if (axis == FIRST_Y_AXIS || axis == SECOND_Y_AXIS || axis == COLOR_AXIS)
+	axis_array[axis].label.rotate = TEXT_VERTICAL;
 }
 
 /******** The 'reset' command ********/
@@ -1433,6 +1613,12 @@ reset_command()
     /* delete linestyles */
     while (first_linestyle != NULL)
 	delete_linestyle((struct linestyle_def *) NULL, first_linestyle);
+#ifdef EAM_OBJECTS
+    /* delete objects */
+    while (first_object != NULL)
+	delete_object((struct object *) NULL, first_object);
+    unset_style_rectangle();
+#endif
 
     /* 'polar', 'parametric' and 'dummy' are interdependent, so be
      * sure to keep the order intact */
@@ -1464,8 +1650,9 @@ reset_command()
 	/* 'tics' default is on for some, off for the other axes: */
 	unset_tics(axis);
 	axis_array[axis].ticmode = axis_defaults[axis].ticmode;
-	unset_mtics(axis);
+	unset_minitics(axis);
 	axis_array[axis].ticdef = default_axis_ticdef;
+	axis_array[axis].minitics = MINI_DEFAULT;
 
 	reset_logscale(axis);
     }
@@ -1492,6 +1679,7 @@ reset_command()
     polar_grid_angle = 0;
     grid_layer = -1;
 
+    reset_hidden3doptions();
     hidden3d = FALSE;
 
     label_contours = TRUE;
@@ -1510,9 +1698,10 @@ reset_command()
     unset_cntrparam();
     unset_zero();
     unset_dgrid3d();
-    unset_ticscale();
     unset_ticslevel();
+#ifdef OLDUNSETTICS
     unset_tics_in();
+#endif /* OLDUNSETTICS */
     unset_margin(&bmargin);
     unset_margin(&lmargin);
     unset_margin(&rmargin);
@@ -1520,19 +1709,21 @@ reset_command()
     unset_pointsize();
     unset_encoding();
     unset_decimalsign();
-#ifdef PM3D
     pm3d_reset();
     reset_colorbox();
     reset_palette();
+#ifdef BINARY_DATA_FILE
+    df_unset_datafile_binary();
 #endif
-#if USE_ULIG_FILLEDBOXES
     unset_fillstyle();
+#ifdef EAM_HISTOGRAMS
+    unset_histogram();
 #endif
 
     unset_missing();
     df_separator = '\0';
     free(df_commentschars);
-    df_commentschars = strdup(DEFAULT_COMMENTS_CHARS);
+    df_commentschars = gp_strdup(DEFAULT_COMMENTS_CHARS);
 
     unset_locale();
 #if 0
@@ -1542,10 +1733,28 @@ reset_command()
 #endif
     unset_fit();
 
-    /* garbage collection on space allocated for user variables */
+#if 0
+    /* 01-Jun-2006: Deleting undefined user variables can break user functions.
+     * E.g.    f(x) = a + b;  reset;
+     * f(x) is still defined, sort of, but it holds pointers to a and b, which
+     * have been deleted. Evaluation after a reset can trigger a segfault.
+     */
+    /* Garbage collection on space allocated for user variables */
     cleanup_udvlist();
+#endif
+
+    update_gpval_variables(0); /* update GPVAL_ inner variables */
 
     /* HBB 20000506: set 'interactive' back to its real value: */
     interactive = save_interactive;
 }
 
+#ifdef EAM_OBJECTS
+static void
+unset_style_rectangle()
+{
+    struct object foo = DEFAULT_RECTANGLE_STYLE;
+    default_rectangle = foo;
+    return;
+}
+#endif
