@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util.c,v 1.65 2006/08/22 03:08:12 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util.c,v 1.65.2.3 2008/02/14 17:06:51 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util.c */
@@ -38,7 +38,7 @@ static char *RCSid() { return RCSid("$Id: util.c,v 1.65 2006/08/22 03:08:12 sfea
 
 #include "alloc.h"
 #include "command.h"
-#include "datafile.h"		/* for df_showdata */
+#include "datafile.h"		/* for df_showdata and df_reset_after_error */
 #include "misc.h"
 #include "plot.h"
 /*  #include "setshow.h" */		/* for month names etc */
@@ -381,6 +381,9 @@ gp_strdup(const char *s)
 {
     char *d;
 
+    if (!s)
+	return NULL;
+
 #ifndef HAVE_STRDUP
     d = gp_alloc(strlen(s) + 1, "gp_strdup");
     if (d)
@@ -567,7 +570,8 @@ gprintf(
 	*t++ = '%';
 	/* dont put isdigit first since sideeffect in macro is bad */
 	while (*++format == '.' || isdigit((unsigned char) *format)
-	       || *format == '-' || *format == '+' || *format == ' ')
+	       || *format == '-' || *format == '+' || *format == ' '
+	       || *format == '\'')
 	    *t++ = *format;
 	/*}}} */
 
@@ -912,6 +916,10 @@ int_error(int t_num, const char str[], va_dcl)
 #endif
     fputs("\n\n", stderr);
 
+    /* We are bailing out of nested context without ever reaching */
+    /* the normal cleanup code. Reset any flags before bailing.   */
+    df_reset_after_error();
+
     update_gpval_variables(2);
 
     bail_to_command_line();
