@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util3d.c,v 1.27 2005/09/28 00:19:09 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util3d.c,v 1.27.2.2 2008/03/08 17:51:47 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util3d.c */
@@ -1066,6 +1066,7 @@ draw3d_line_unconditional(
     int linetype)
 {
     unsigned int x1, y1, x2, y2;
+    struct lp_style_type ls = *lp;
 
     /* HBB 20020312: v2 can be NULL, if this call is coming from
     draw_line_hidden. --> redirect to point drawing routine */
@@ -1076,20 +1077,21 @@ draw3d_line_unconditional(
 
     TERMCOORD(v1, x1, y1);
     TERMCOORD(v2, x2, y2);
-    term_apply_lp_properties(lp);
 
-    /* FIXME - This whole set of tests may now be unnecessary */
-    if (lp->use_palette) {
-	double z =  (v1->real_z + v2->real_z) * 0.5;
-	if (lp->pm3d_color.type != TC_RGB)
-	    set_color(cb2gray(z2cb(z)));
-    } else
-	if (linetype != lp->l_type)
-	    (term->linetype)(linetype);
+    /* User-specified line styles */
+    if (prefer_line_styles && linetype >= 0)
+	lp_use_properties(&ls, linetype+1, FALSE);
 
-    /* FIXME HBB 20000621: should this call clip_line, instead? */
-    (term->move)(x1, y1);
-    (term->vector)(x2, y2);
+    /* The usual case of auto-generated line types */
+    else
+	ls.l_type = linetype;
+
+    /* Color by Z value */
+    if (ls.pm3d_color.type == TC_Z)
+	    ls.pm3d_color.value = (v1->real_z + v2->real_z) * 0.5;
+
+    term_apply_lp_properties(&ls);
+    draw_clip_line(x1,y1,x2,y2);
 }
 
 void
