@@ -1,5 +1,5 @@
 /*
- * $Id: gp_cairo.c,v 1.16 2006/09/10 17:49:44 tlecomte Exp $
+ * $Id: gp_cairo.c,v 1.16.2.2 2007/08/20 08:05:16 tlecomte Exp $
  */
 
 /* GNUPLOT - gp_cairo.c */
@@ -904,13 +904,13 @@ void gp_cairo_draw_fillbox(plot_struct *plot, int x, int y, int width, int heigh
 
 
 #ifdef WITH_IMAGE
-/*	'corner[0]' is the upper left corner (in terms of plot location) of
- *	the outer edge of the image.  Similarly, 'corner[1]' is the lower
+/*	corner[0] = (x1,y1) is the upper left corner (in terms of plot location) of
+ *	the outer edge of the image.  Similarly, corner[1] = (x2,y2) is the lower
  *	right corner of the outer edge of the image.  (Outer edge means the
- *	outer extent of the corner pixels, not the middle of the corner
- *	pixels.)  'corner[2]' is the upper left corner of the visible part
- *	of the image, and 'corner[3]' is the lower right corner of the visible
- *	part of the image. */
+ *	outer extent of the corner pixels, not the middle of the corner pixels).
+ *	corner[2] and corner[3] = (x3,y3) and (x4,y4) define a clipping box in
+ *	the primary plot into which all or part of the image will be rendered.
+ */
 void gp_cairo_draw_image(plot_struct *plot, coordval * image, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int M, int N, t_imagecolor color_mode)
 {
 	int m,n;
@@ -962,6 +962,16 @@ void gp_cairo_draw_image(plot_struct *plot, coordval * image, int x1, int y1, in
 	FPRINTF((stderr,"M %d N %lf x1 %d y1 %d\n", M, N, x1, y1));
 	cairo_save( plot->cr );
 
+	/* Set clipping boundaries for image copy.
+	 * The bounds were originally possed in corners[2] and corners[3]
+	 */
+	cairo_move_to(plot->cr, x3, y3);
+	cairo_line_to(plot->cr, x3, y4);
+	cairo_line_to(plot->cr, x4, y4);
+	cairo_line_to(plot->cr, x4, y3);
+	cairo_close_path(plot->cr);
+	cairo_clip(plot->cr);
+
 	pattern = cairo_pattern_create_for_surface( image_surface );
 	/* scale and keep sharp edges */
 	cairo_pattern_set_filter( pattern, CAIRO_FILTER_FAST );
@@ -1474,17 +1484,16 @@ void gp_cairo_fill_pattern(plot_struct *plot, int fillpar)
 
 	cairo_set_source_rgb( pattern_cr, 1.0, 1.0, 1.0);
 	cairo_paint(pattern_cr);
-	cairo_set_line_width(pattern_cr, 1.0/PATTERN_SIZE);
+	cairo_set_line_width(pattern_cr, PATTERN_SIZE/50.);
 	cairo_set_line_cap  (pattern_cr, CAIRO_LINE_CAP_BUTT);
 	cairo_set_source_rgb(pattern_cr, plot->color.r, plot->color.g, plot->color.b);
 
 	switch (fillpar%8) {
 	case 0: /* no fill */
 	default:
-		cairo_set_source_rgb(pattern_cr, 1.0, 1.0, 1.0);
-		cairo_paint(pattern_cr);
 		break;
-	case 1: /* plot->cross-hatch */
+	case 1: /* cross-hatch */
+	case 2: /* double cross-hatch */
 		cairo_move_to(pattern_cr, 0,0);
 		cairo_line_to(pattern_cr, 1.0,1.0);
 		cairo_stroke(pattern_cr);
@@ -1492,98 +1501,58 @@ void gp_cairo_fill_pattern(plot_struct *plot, int fillpar)
 		cairo_line_to(pattern_cr, 1.0,0);
 		cairo_stroke(pattern_cr);
 		break;
-	case 2: /* double plot->cross-hatch */
-		cairo_move_to(pattern_cr, 0,0);
-		cairo_line_to(pattern_cr, 1.0,1.0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 0,1.0);
-		cairo_line_to(pattern_cr, 1.0,0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 0,1.0/2.);
-		cairo_line_to(pattern_cr, 1.0/2.,0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 1.0,1.0/2.);
-		cairo_line_to(pattern_cr, 1.0/2.,1.0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 1.0/2.,0);
-		cairo_line_to(pattern_cr, 1.0,1.0/2.);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 0,1.0/2.);
-		cairo_line_to(pattern_cr, 1.0/2.,1.0);
-		cairo_stroke(pattern_cr);
-		break;
 	case 3: /* solid */
 		cairo_paint(pattern_cr);
 		break;
-	case 4:
-		cairo_move_to(pattern_cr, 0,0);
-		cairo_line_to(pattern_cr, 1.0,1.0);
-		cairo_stroke(pattern_cr);
-		/* take care of the corners */
-		cairo_move_to(pattern_cr, -1.0,0);
-		cairo_line_to(pattern_cr, 1.0,2*1.0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 0,-1.0);
-		cairo_line_to(pattern_cr, 2*1.0,1.0);
-		cairo_stroke(pattern_cr);
-		break;
+	case 4: /* diagonal hatch */
 	case 5:
-		cairo_move_to(pattern_cr, 0,1.0);
-		cairo_line_to(pattern_cr, 1.0,0);
-		cairo_stroke(pattern_cr);
-		/* take care of the corners */
-		cairo_move_to(pattern_cr, -1.0,1.0);
-		cairo_line_to(pattern_cr, 1.0,-1.0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 0,2*1.0);
-		cairo_line_to(pattern_cr, 2*1.0,0);
-		cairo_stroke(pattern_cr);
-		break;
 	case 6:
-		cairo_move_to(pattern_cr, 0,0);
-		cairo_line_to(pattern_cr, 1.0/2.,1.0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 1.0/2.,0);
-		cairo_line_to(pattern_cr, 1.0,1.0);
-		cairo_stroke(pattern_cr);
-		/* take care of the corners */
-		cairo_move_to(pattern_cr, -1.0/2.,0);
-		cairo_line_to(pattern_cr, 1.0/2.,2*1.0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 1.0/2.,-1.0);
-		cairo_line_to(pattern_cr, 3/2.*1.0,1.0);
-		cairo_stroke(pattern_cr);
-		break;
 	case 7:
-		cairo_move_to(pattern_cr, 0,1.0);
-		cairo_line_to(pattern_cr, 1.0/2.,0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 1.0/2.,1.0);
-		cairo_line_to(pattern_cr, 1.0,0);
-		cairo_stroke(pattern_cr);
-		/* take care of the corners */
-		cairo_move_to(pattern_cr, -1.0/2.,1.0);
-		cairo_line_to(pattern_cr, 1.0/2.,-1.0);
-		cairo_stroke(pattern_cr);
-		cairo_move_to(pattern_cr, 1.0/2.,2*1.0);
-		cairo_line_to(pattern_cr, 3/2.*1.0,0);
+		cairo_move_to(pattern_cr, 0.5,0.);
+		cairo_line_to(pattern_cr, 0.5,1.);
 		cairo_stroke(pattern_cr);
 		break;
 	}
 
 	pattern = cairo_pattern_create_for_surface( pattern_surface );
 	cairo_pattern_set_extend( pattern, CAIRO_EXTEND_REPEAT );
-	cairo_destroy( pattern_cr );
 
 	/* compensate the transformation matrix of the main context */
 	cairo_matrix_init_scale(&matrix,
 		1.0/plot->oversampling_scale,
 		1.0/plot->oversampling_scale);
+
+	switch (fillpar%8) {
+	case 0: /* no fill */
+	case 1: /* cross-hatch */
+	case 3: /* solid */
+	default:
+		break;
+	case 2: /* double cross-hatch */
+		cairo_matrix_scale( &matrix, 2.,2.);
+		break;
+	case 4: /* diagonal hatch */
+		cairo_matrix_rotate( &matrix, M_PI/4);
+		break;
+	case 5:
+		cairo_matrix_rotate( &matrix, -M_PI/4);
+		break;
+	case 6:
+		cairo_matrix_rotate( &matrix, M_PI/4);
+		cairo_matrix_scale( &matrix, 2.,2.);
+		break;
+	case 7:
+		cairo_matrix_rotate( &matrix, -M_PI/4);
+		cairo_matrix_scale( &matrix, 2.,2.);
+		break;
+	}
+
 	cairo_pattern_set_matrix(pattern,&matrix);
 
-	cairo_surface_destroy( pattern_surface );
+	cairo_destroy( pattern_cr );
 	cairo_set_source( plot->cr, pattern );
 	cairo_pattern_destroy( pattern );
+	cairo_surface_destroy( pattern_surface );
 }
 
 
